/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * DUMPTUME.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/28/92
 *   MODIFIED : 01/07/93
 *       TABS : 05 09
 *
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Load a tUME Map and print all information in ASCII
 *
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <stdio.h>

#include <echidna/eerrors.h>
#include <echidna/argparse.h>
#include <echidna/btume.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/

short	UseTMGC;			/* Load map as TMGC instead of TMGX	*/
short	DumpColors;			/* Print Color Information			*/
short	DumpLayers;			/* Print Layer Information			*/
short	DumpGraphics;		/* Print Tile Graphics Information	*/
short	IncludeComposite;	/* Print Composite Tileset Rooms	*/

/******************************* M A C R O S ******************************/


/******************************* T A B L E S ******************************/


/***************************** R O U T I N E S ****************************/

/*********************************************************************
 *
 * PrintTileInfoForReals
 *
 * SYNOPSIS
 *		void PrintTileInfoForReals (MapType *map, RoomType *room, short x, short y, short l)
 *
 * PURPOSE
 *		Prints information about a particular tile in a particular room.
 *
 *		This routine was written to demonstrate how to navigate the
 *		data structures to extract information need to create your
 *		own tUMEPACK.
 *
 * INPUT
 *		map		: pointer to a MapType as returned from CreatetUMEMapTMGC|X
 *		room	: pointer to a RoomType as in map->Rooms[1]
 *		x		: valid x coordinate for a tile (0 - (roomwidth  - 1))
 *		y		: valid y coordinate for a tile (0 - (roomheight - 1))
 *		l		: valid layer level (0 - (numlayers - 1))
 *
 * RETURN VALUE
 *		None
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintTileInfoForReals (MapType *map, RoomType *room, short x, short y, short l)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintTileInfoForReals";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		UWORD		 tileID;
		UBYTE		 tilesetID;
		UBYTE		 tileflags;
		PlotType	*plot;
		TilesetType	*ts;

		printf ("--Display-Tile-Info-----------------------------------\n");

		if (x >= room->Width) {
			printf ("Error: An x of %d is outside of room '%s'\n", x, room->Name);
			return;
		}
		if (y >= room->Height) {
			printf ("Error:A y of %d is outside of room '%s'\n", y, room->Name);
			return;
		}
		if (l >= room->LayerCount) {
			printf ("Error:There is no layer %d in room '%s'\n", l, room->Name);
			return;
		}

		plot = ActivateXTRA (room->Layers[l]->Plot[y]);
		tileID    = plot[x].TileID;
		tilesetID = plot[x].TilesetID;
		tileflags = plot[x].PlotFlags;

		if (tilesetID) {
			printf ("tileID    = %4d\n", tileID);
			printf ("tilesetID = %4d\n", tilesetID);
			printf ("tileflags =  $%02x\n", tileflags);

			printf ("tile in room %s at %d %d %d comes from:\n",
				room->Name, x, y, l);

			ts = map->Tilesets[tilesetID];

			printf ("\tFileSpec     = %s\n",  ts->FileSpec);
			printf ("\tUserType     = %6d\n", ts->UserType);
			printf ("\tUserNumber   = %6d\n", ts->UserNumber);
			printf ("\tTileCount    = %6d\n", ts->TileCount);
			printf ("\tWidth        = %6d\n", ts->Width);
			printf ("\tHeight       = %6d\n", ts->Height);
			printf ("\tDepth        = %6d\n", ts->Depth);
			printf ("\tSaveFlags    = 0x%04x\n", ts->SaveFlags);
			printf ("\tFlags        = 0x%04x\n", ts->Flags);
			printf ("\tGuideX       = %6d\n", ts->GuideX);
			printf ("\tGuideY       = %6d\n", ts->GuideY);
			printf ("\tGuideWidth   = %6d\n", ts->GuideWidth);
			printf ("\tGuideHeight  = %6d\n", ts->GuideHeight);
			printf ("\tGuideOnOff   = %6d\n", ts->GuideOnOff);
			printf ("\tGridX        = %6d\n", ts->GridX);
			printf ("\tGridY        = %6d\n", ts->GridY);
			printf ("\tGridWidth    = %6d\n", ts->GridWidth);
			printf ("\tGridHeight   = %6d\n", ts->GridHeight);
			printf ("\tGridOnOff    = %6d\n", ts->GridOnOff);
			printf ("\tZoomDstDup   = %6d\n", ts->ZoomDstDup);
			printf ("\tZoomSrcSkip  = %6d\n", ts->ZoomSrcSkip);
			printf ("\tZoomOnOff    = %6d\n", ts->ZoomOnOff);

			if (ts->Comments) {
				char	**sptr;

				printf ("\tComments:\n");

				sptr = ts->Comments;
				while (*sptr) {
					printf ("%s\n", *sptr++);
				}
			}

			if (ts->GridedTilesRoom) {
				short		 rowsize;
				short		 x2;
				short		 y2;
				short		 xwork;
				short		 ywork;
				RoomType	*comptileroom;

				printf ("\t%dx%d Composite Tiles\n", ts->TilesAcross, ts->TilesDown);
				
				comptileroom = ts->GridedTilesRoom;
				rowsize = comptileroom->Width / ts->TilesAcross;
				x2  = (tileID - 1) % rowsize;
				y2  = (tileID - 1) / rowsize;
				x2 *= ts->TilesAcross;
				y2 *= ts->TilesDown;
				for (ywork = 0; ywork < ts->TilesDown; ywork++) {
					for (xwork = 0; xwork < ts->TilesAcross; xwork++) {
						PrintTileInfoForReals (map, comptileroom, x2 + xwork, y2 + ywork, 0);
					}
				}

			} else {
				if (ts->TMG.TileCount) {
					TileDataGroup	*tdg;
					UWORD			 tcount;
					UBYTE			*tiledata;
					short			 found;
					UWORD			 tilesize;

					printf ("\tTileData\n");

					if (UseTMGC) {
						tilesize = ts->TMG.Width * ts->TMG.Height;
					} else {
						tilesize = ((ts->TMG.Width + 15) / 16) * 2 * ts->TMG.Height * ts->TMG.Depth;
					}

					tcount = tileID - 1;
					found  = FALSE;

					tdg = Head (ts->TMG.TileGroupList);
					while (!found) {
						if (tcount < tdg->NumTiles) {
							found     = TRUE;
							tiledata  = ActivateXTRA (tdg->TileData);
							tiledata += tilesize * tcount;
						} else {
							tcount -= tdg->NumTiles;
							tdg = Next (tdg);
						}
					}

					if (UseTMGC) {
						UWORD	 y;
						UWORD	 x;

				 		for (y = 0; y < ts->TMG.Height; y++) {
				 			for (x = 0; x < ts->TMG.Width; x++) {
				 				printf ("%02x ", *tiledata++);
				 			}
				 			printf ("\n");
				 		}
					} else {
						UWORD	 x;
						UWORD	 y;
						UWORD	 d;
						UWORD	 width = ((ts->Width + 15) / 16);

						for (d = 0; d < ts->TMG.Depth; d++) {
							printf ("\tPlane[%d]\n", d);
							for (y = 0; y < ts->TMG.Height; y++) {
								for (x = 0; x < width; x++) {
									printf ("%02x",  *tiledata++);
									printf ("%02x ", *tiledata++);
								}
								printf ("\n");
							}
						}
					}
				} else {
					printf ("\tNo tile graphics saved in file for this tileset\n");
				}
			}
		} else {
			printf ("tile in room %s at %d,%d,%d is a NULL tile\n",
				room->Name,
				x,
				y,
				l);
		}

	}
} /* PrintTileInfoForReals */

/*********************************************************************
 *
 * PrintTileInfo
 *
 * SYNOPSIS
 *		void PrintTileInfo (MapType *map, short r, short x, short y, short l)
 *
 * PURPOSE
 *		Prints information about a particular tile in a particular room.
 *
 *		This routine was written to demonstrate how to navigate the
 *		data structures to extract information need to create your
 *		own tUMEPACK.
 *
 * INPUT
 *		map		: pointer to a MapType as returned from CreatetUMEMapTMGC|X
 *		r		: valid room index (1 - NumRooms))
 *		x		: valid x coordinate for a tile (0 - (roomwidth  - 1))
 *		y		: valid y coordinate for a tile (0 - (roomheight - 1))
 *		l		: valid layer level (0 - (numlayers - 1))
 *
 * RETURN VALUE
 *		None
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintTileInfo (MapType *map, short r, short x, short y, short l)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintTileInfo";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		RoomType	*room;

		if (r < 1 || r > map->NumRooms) {
			printf ("There is no room #%d\n", r);
			return;
		} 
		room = map->Rooms[r];

		PrintTileInfoForReals (map, room, x, y, l);
	}

} /* PrintTileInfo */

/*********************************************************************
 *
 * PrintPalette
 *
 * SYNOPSIS
 *		void PrintPalette (ColorInfo *ci)
 *
 * PURPOSE
 *		Print all the information from on Room's ColorInfo
 *
 * INPUT
 *		pointer to a colorinfo type as in &room->ColorInfo.
 *
 * RETURN VALUE
 *		None
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintPalette (ColorInfo *ci)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintPalette";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		printf ("\tNumColors   = %6d\n", ci->NumColors);
		{
			UWORD	c;

			printf ("\t------Color-Red-Grn-Blu-Hue-Sat-Val------\n");

			for (c = 0; c < ci->NumColors; c++) {
				printf ("\tColor [%3d] %3d %3d %3d ",
						c,
						ci->Colors[c].red,
						ci->Colors[c].green,
						ci->Colors[c].blue);
				if (ci->HSVs) {
					printf ("%3d %3d %3d\n",
						ci->HSVs[c].Hue,
						ci->HSVs[c].Saturation,
						ci->HSVs[c].Value);
				} else {
					printf ("N/A N/A N/A\n");
				}
			}
		}
		printf ("\tNumCycles   = %6d\n", ci->NumCycles);
		printf ("\tCycleFlag   = %6d\n", ci->CycleFlag);
		if (ci->NumCycles) {
			CycleInfo	*cycle;
			UWORD		 c;
			UWORD		 cnt;

			for (c = 0; c < ci->NumCycles; c++) {
				cycle = ci->Cycles[c];

				printf ("\tCycle [%2d]\n", c);
				printf ("\t\tSpeed     = %5d\n", cycle->Speed);
				printf ("\t\tDirection = %5d\n", cycle->Direction);
				printf ("\t\tFlags     = $%04x\n", cycle->Flags);
				printf ("\t\tNum Cycle Registers = %6d\n", cycle->NumRegs);

				for (cnt = 0; cnt < cycle->NumRegs; cnt++) {
					printf ("\t\t%2d\n", cycle->Registers[cnt]);
				}

				printf ("\t\tNum Cycle Colors    = %6d\n", cycle->NumColors);

				printf ("\t\t-Red-Grn-Blu-Hue-Sat-Val-\n");
				for (cnt = 0; cnt < cycle->NumColors; cnt++) {

					printf ("\t\t %3d %3d %3d %3d %3d %3d\n",
							cycle->Colors[cnt].red,
							cycle->Colors[cnt].green,
							cycle->Colors[cnt].blue,
							cycle->HSVs[cnt].Hue,
							cycle->HSVs[cnt].Saturation,
							cycle->HSVs[cnt].Value);
				}
			}
		}
	}
} /* PrintPalette */

/*********************************************************************
 *
 * PrintLayer
 *
 * SYNOPSIS
 *		void PrintLayer (LayerType *layer, short width, short height)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintLayer (LayerType *layer, short width, short height)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintLayer";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		PlotType	*plot;
		UWORD		 x;
		UWORD		 y;


		for (y = 0; y < height; y++) {
			plot = ActivateXTRA (layer->Plot[y]);
			for (x = 0; x < width; x++) {
				printf ("|%02x %03d %04d", plot->PlotFlags, plot->TilesetID, plot->TileID);
				plot++;
			}
			printf ("\n");
		}
	}
	
} /* PrintLayer */

/*********************************************************************
 *
 * PrintRoom
 *
 * SYNOPSIS
 *		void PrintRoom (RoomType *room)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintRoom (RoomType *room)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintRoom";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		printf ("\t----------------------------------\n");
		printf ("\tName        = %s\n",  room->Name);
		printf ("\tUserType    = %6d\n", room->UserType);
		printf ("\tUserNumber  = %6d\n", room->UserNumber);
		printf ("\tFlags       = 0x%04x\n", room->Flags);
		printf ("\tWidth       = %6d\n", room->Width);
		printf ("\tHeight      = %6d\n", room->Height);
		printf ("\tTileWidth   = %6d\n", room->TileWidth);
		printf ("\tTileHeight  = %6d\n", room->TileHeight);
		printf ("\tGuideX      = %6d\n", room->GuideX);
		printf ("\tGuideY      = %6d\n", room->GuideY);
		printf ("\tGuideWidth  = %6d\n", room->GuideWidth);
		printf ("\tGuideHeight = %6d\n", room->GuideHeight);
		printf ("\tGuideOnOff  = %6d\n", room->GuideOnOff);
		printf ("\tGridX       = %6d\n", room->GridX);
		printf ("\tGridY       = %6d\n", room->GridY);
		printf ("\tGridWidth   = %6d\n", room->GridWidth);
		printf ("\tGridHeight  = %6d\n", room->GridHeight);
		printf ("\tGridOnOff   = %6d\n", room->GridOnOff);
		printf ("\tZoomDstDup  = %6d\n", room->ZoomDstDup);
		printf ("\tZoomSrcSkip = %6d\n", room->ZoomSrcSkip);
		printf ("\tZoomOnOff   = %6d\n", room->ZoomOnOff);

		if (DumpColors) {
			printf ("\t-----------------Palette-----------------\n");
			PrintPalette (&room->ColorInfo);
		}

		printf ("\t-----------------Layers------------------\n");
		printf ("\tLayerCount  = %6d\n", room->LayerCount);
		if (DumpLayers) {
			UWORD	l;

			for (l = 0; l < room->LayerCount; l++) {
				printf ("\tLayer %d\n", l);
				PrintLayer (room->Layers[l], room->Width, room->Height);
			}
		}

	}
} /* PrintRoom */

/*********************************************************************
 *
 * PrintTileset
 *
 * SYNOPSIS
 *		void PrintTileset (TilesetType *ts)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintTileset (TilesetType *ts)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintTileset";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	{
		printf ("\t----------------------------------\n");
		printf ("\tFileSpec     = %s\n",  ts->FileSpec);
		printf ("\tUserType     = %6d\n", ts->UserType);
		printf ("\tUserNumber   = %6d\n", ts->UserNumber);
		printf ("\tTileCount    = %6d\n", ts->TileCount);
		printf ("\tWidth        = %6d\n", ts->Width);
		printf ("\tHeight       = %6d\n", ts->Height);
		printf ("\tDepth        = %6d\n", ts->Depth);
		printf ("\tSaveFlags    = 0x%04x\n", ts->SaveFlags);
		printf ("\tFlags        = 0x%04x\n", ts->Flags);
		printf ("\tGuideX       = %6d\n", ts->GuideX);
		printf ("\tGuideY       = %6d\n", ts->GuideY);
		printf ("\tGuideWidth   = %6d\n", ts->GuideWidth);
		printf ("\tGuideHeight  = %6d\n", ts->GuideHeight);
		printf ("\tGuideOnOff   = %6d\n", ts->GuideOnOff);
		printf ("\tGridX        = %6d\n", ts->GridX);
		printf ("\tGridY        = %6d\n", ts->GridY);
		printf ("\tGridWidth    = %6d\n", ts->GridWidth);
		printf ("\tGridHeight   = %6d\n", ts->GridHeight);
		printf ("\tGridOnOff    = %6d\n", ts->GridOnOff);
		printf ("\tZoomDstDup   = %6d\n", ts->ZoomDstDup);
		printf ("\tZoomSrcSkip  = %6d\n", ts->ZoomSrcSkip);
		printf ("\tZoomOnOff    = %6d\n", ts->ZoomOnOff);

		if (ts->Comments) {
			char	**sptr;

			printf ("\tComments:\n");

			sptr = ts->Comments;
			while (*sptr) {
				printf ("%s\n", *sptr++);
			}
		}

		printf ("\n");

		if (ts->GridedTilesRoom) {
			printf ("\t%dx%d Composite Tiles\n", ts->TilesAcross, ts->TilesDown);
			
			if (DumpGraphics) {
				short		 rowsize;
				short		 x2;
				short		 y2;
				short		 xwork;
				short		 ywork;
				short		 t;
				RoomType	*comptileroom;
				PlotType	*plot;

				comptileroom = ts->GridedTilesRoom;
				rowsize = comptileroom->Width / ts->TilesAcross;
				for (t = 0; t < ts->TileCount; t++) {
					x2  = t % rowsize;
					y2  = t / rowsize;
					x2 *= ts->TilesAcross;
					y2 *= ts->TilesDown;
					printf ("tile #%d\n", t);
					for (ywork = 0; ywork < ts->TilesDown; ywork++) {
						plot  = ActivateXTRA (comptileroom->Layers[0]->Plot[y2 + ywork]);
						plot += x2;
						for (xwork = 0; xwork < ts->TilesAcross; xwork++) {
							printf ("|%02x %03d %04d", plot->PlotFlags, plot->TilesetID, plot->TileID);
							plot++;
						}
						printf ("\n");
					}
				}
			}
		} else if (ts->TMG.TileCount) {
			printf ("\tTMG TileCount        = %3d\n", ts->TMG.TileCount);
			printf ("\tTMG Width            = %3d\n", ts->TMG.Width);
			printf ("\tTMG Height           = %3d\n", ts->TMG.Height);
			printf ("\tTMG Depth            = %3d\n", ts->TMG.Depth);
			printf ("\tTMG TransparentColor = %3d\n", ts->TMG.TransparentColor);
			printf ("\n");

			if (DumpGraphics) {
				if (UseTMGC) {
					TileDataGroup	*tdg;

					printf ("\t--------TMGC Data---------\n");

					tdg = Head (ts->TMG.TileGroupList);
					while (!EndOfList (tdg)) {
						UWORD	 t;
						UWORD	 y;
						UWORD	 x;
						UBYTE	*data;

						data = ActivateXTRA (tdg->TileData);
						for (t = 0; t < tdg->NumTiles; t++) {
							printf ("\tTile #%d\n", t);
							for (y = 0; y < ts->TMG.Height; y++) {
								for (x = 0; x < ts->TMG.Width; x++) {
									printf ("%02x ", *data++);
								}
								printf ("\n");
							}
						}
						tdg = Next (tdg);
					}
				} else {
					TileDataGroup	*tdg;

					printf ("\t--------TMGX Data---------\n");

					tdg = Head (ts->TMG.TileGroupList);
					while (!EndOfList (tdg)) {
						UWORD	 t;
						UWORD	 x;
						UWORD	 y;
						UWORD	 d;
						UBYTE	*data;
						UWORD	 width = ((ts->Width + 15) / 16);

						data = ActivateXTRA (tdg->TileData);
						for (t = 0; t < tdg->NumTiles; t++) {
							printf ("\tTile #%d\n", t);
							for (d = 0; d < ts->TMG.Depth; d++) {
								printf ("\tPlane[%d]\n", d);
								for (y = 0; y < ts->TMG.Height; y++) {
									for (x = 0; x < width; x++) {
										printf ("%02x",  *data++);
										printf ("%02x ", *data++);
									}
									printf ("\n");
								}
							}
						}
						tdg = Next (tdg);
					}
				}
			}
		}
	}
} /* PrintTileset */

/*********************************************************************
 *
 * PrintMap
 *
 * SYNOPSIS
 *		void PrintMap (MapType *map)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void PrintMap (MapType *map)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "PrintMap";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		printf ("NumRooms    = %3d\n", map->NumRooms);
		printf ("NumTilesets = %3d\n", map->NumTilesets);
		printf ("-----------------Rooms-------------------\n");
		{
			RoomType	**roomptr;

			roomptr = map->Rooms + 1;

			while (*roomptr) {
				/*
				** Skip Composite Tileset Rooms
				*/
				if (!(*roomptr)->CompositeTSet || IncludeComposite) {
					PrintRoom (*roomptr);
				}
				roomptr++;
			}
		}
		printf ("-----------------Tilesets-------------------\n");
		{
			TilesetType	**tsptr;

			tsptr = map->Tilesets + 1;

			while (*tsptr) {
				PrintTileset (*tsptr);
				tsptr++;
			}
		}
	}
} /* PrintMap */

/******************************** TEMPLATE ********************************/

#define ARG_INFILES		(newargs[0])
#define ARG_TMGC		(newargs[1])
#define ARG_TILEDATA	(newargs[2])
#define ARG_ROOMDATA	(newargs[3])
#define ARG_COMPOSITE	(newargs[4])
#define ARG_PALETTE		(newargs[5])
#define ARG_DOTILE		(newargs[6])

char Usage[] = "Usage: DUMPTUME INFILE1 [INFILE2...]\n";

ArgSpec Template[] = {
	{ STANDARD_ARG | REQUIRED_ARG | MULTI_ARG,	"INFILEs",	"\tINFILES     = TUME Maps\n",	},
	{ CHRSWITCH_ARG,							"C",		"\t-C          = Load as TMGC\n",	},
	{ CHRSWITCH_ARG,							"T",		"\t-T          = Print Tile Graphics Data\n",	},
	{ CHRSWITCH_ARG,							"R",		"\t-R          = Print Room Tile Data\n",	},
	{ CHRSWITCH_ARG,							"X",		"\t-X          = Include Composite Rooms\n", },
	{ CHRSWITCH_ARG,							"P",		"\t-P          = Print Palette Data\n",	},
	{ CHRKEYWORD_ARG,							"D",		"\t-D<R,X,Y,L> = Display Tile Info\n"
															"\t                Where R is the room number (1-n)\n"
															"\t                Where X is the x position of tile\n"
															"\t                Where Y is the y position of tile\n"
															"\t                Where L is the layer of tile (0-n)\n"
															"\t                Exmaple: -D1,3,2,0\n", },
	{ 0, NULL, NULL,	},
};

/********************************** MAIN **********************************/

int main(int argc, char **argv)
{
	char			**newargs;

	printf ("DumptUME Copyright (c) 1992 Echidna\n");

	newargs = argparse (argc, argv, Template);

	if (!newargs) {
		printf ("%s\n", GlobalErrMsg);
		printarghelp (Usage, Template);
		return EXIT_FAILURE;
	} else {
		char	**tfiles;
		MapType	*map = NULL;

		tfiles           = MULTI_ARGLIST(ARG_INFILES);
		UseTMGC          = SWITCH_VALUE (ARG_TMGC);
		DumpColors       = SWITCH_VALUE (ARG_PALETTE);
		DumpLayers       = SWITCH_VALUE (ARG_ROOMDATA);
		DumpGraphics     = SWITCH_VALUE (ARG_TILEDATA);
		IncludeComposite = SWITCH_VALUE (ARG_COMPOSITE);

		while (*tfiles) {
			if (UseTMGC) {
				map = AppendtUMEMapTMGC (map, *tfiles);
			} else {
				map = AppendtUMEMapTMGX (map, *tfiles);
			}
			if (!map) {
				break;
			}
			tfiles++;
		}

		if (map) {
			PrintMap (map);
		}

		if (ARG_DOTILE) {
			short	x;
			short	y;
			short	l;
			short	r;
			
			sscanf (ARG_DOTILE, "%d , %d , %d , %d", &r, &x, &y, &l);
			PrintTileInfo (map, r, x, y, l);
		}
	}

	if (GlobalErr) {
		printf ("%s\n", GlobalErrMsg);
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}

