/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * ALLOC.H
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 07/21/94
 *   MODIFIED : 10/10/94
 *       TABS : 04 07
 *
 * DESCRIPTION
 *		Debugging heap routines. Based upon No Bugs!: delivering error-free
 *		code in C and C++ by David Thielen, Addison-Wesley, 1992, pages 69..82.
 *
 * HISTORY
 *		03/31/92 (D T) - Created.
 *		08/05/94 (dcc) - Remove typos, make compile with BCC 3.1.
 *		10/03/94 (dcc) - Enhance error reporting, support MEM_malloc(0),
 *							  add checksum.
 *		10/05/94 (dcc) - Added MEM_farmalloc(), MEM_farcalloc(), MEM_farfree()
 *							  and MEM_farrealloc().
 *		10/08/94 (dcc) - Added MEM_SetShowCalls().
 *		10/10/94 (dcc) - Added MEM_SetCheckHeapRate(), MEM_SetShowOOM().
 *
*/

#ifndef ECHIDNA_ALLOC_H
#define ECHIDNA_ALLOC_H

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>

#include <stdlib.h>					/* for calloc(), malloc() free() and size_t */

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/****************** F U N C T I O N   P R O T O T Y P E S *****************/

#if DEBUG
void __SetShowCalls(BOOL f, char *pFile, int iLine);
void __SetCheckHeapRate(int i, char *pFile, int iLine);
BOOL __GetShowOOM(char *pFile, int iLine);
void __SetShowOOM(BOOL f, char *pFile, int iLine);
void *__calloc(size_t cItems, size_t uSize, char *pFile, int iLine, BOOL fCheckHeap);
void *__malloc(size_t uSize, char *pFile, int iLine, BOOL fCheckHeap);
void *__realloc(void *pMem, size_t uSize, char *pFile, int iLine, BOOL fCheckHeap);
void __free(void *pMem, char *pFile, int iLine, BOOL fCheckHeap);
void __ShowHeap(char *pFile, int iLine);

#if __MSDOS16__
void far32 *__farcalloc(ULONG cItems, ULONG ulSize, char *pFile, int iLine, BOOL fCheckHeap);
void far32 *__farmalloc(ULONG ulSize, char *pFile, int iLine, BOOL fCheckHeap);
void far32 *__farrealloc(void far32 *pMem, ULONG ulSize, char *pFile, int iLine, BOOL fCheckHeap);
void __farfree(void far32 *pMem, char *pFile, int iLine, BOOL fCheckHeap);
#endif	/* __MSDOS16__ */

#define MEM_SetShowCalls(f)				__SetShowCalls(f,__FILE__,__LINE__)
#define MEM_SetCheckHeapRate(i)			__SetCheckHeapRate(i,__FILE__,__LINE__)
#define MEM_GetShowOOM()					__GetShowOOM(__FILE__,__LINE__)
#define MEM_SetShowOOM(f)					__SetShowOOM(f,__FILE__,__LINE__)
#define MEM_calloc(cItems,uSize)			__calloc(cItems,uSize,__FILE__,__LINE__,TRUE)
#define MEM_malloc(uSize)					__malloc(uSize,__FILE__,__LINE__,TRUE)
#define MEM_realloc(pBuf,uSize)			__realloc(pBuf,uSize,__FILE__,__LINE__,TRUE)
#define MEM_free(pBuf)						__free(pBuf,__FILE__,__LINE__,TRUE)
#define MEM_ShowHeap()						__ShowHeap(__FILE__,__LINE__)

#if __MSDOS16__
#define MEM_farcalloc(cItems,ulSize)	__farcalloc(cItems,ulSize,__FILE__,__LINE__,TRUE)
#define MEM_farmalloc(ulSize)				__farmalloc(ulSize,__FILE__,__LINE__,TRUE)
#define MEM_farrealloc(pBuf,ulSize)		__farrealloc(pBuf,ulSize,__FILE__,__LINE__,TRUE)
#define MEM_farfree(pBuf)					__farfree(pBuf,__FILE__,__LINE__,TRUE)
#else
#define MEM_farcalloc(cItems,uSize)		__calloc(cItems,uSize,__FILE__,__LINE__,TRUE)
#define MEM_farmalloc(uSize)				__malloc(uSize,__FILE__,__LINE__,TRUE)
#define MEM_farrealloc(pBuf,uSize)		__realloc(pBuf,uSize,__FILE__,__LINE__,TRUE)
#define MEM_farfree(pBuf)					__free(pBuf,__FILE__,__LINE__,TRUE)
#endif	/* __MSDOS16__ */

#else		/* !DEBUG */
#define MEM_SetShowCalls(f)				
#define MEM_SetCheckHeapRate(i)			
#define MEM_GetShowOOM()					(FALSE)
#define MEM_SetShowOOM(f)					
#define MEM_calloc							calloc
#define MEM_malloc							malloc
#define MEM_realloc							realloc
#define MEM_free								free
#define MEM_ShowHeap()						

#if __MSDOS16__
#define MEM_farcalloc						farcalloc
#define MEM_farmalloc						farmalloc
#define MEM_farrealloc						farrealloc
#define MEM_farfree							farfree
#else
#define MEM_farcalloc						calloc
#define MEM_farmalloc						malloc
#define MEM_farrealloc						realloc
#define MEM_farfree							free
#endif	/* __MSDOS16__ */

#endif	/* !DEBUG */

#endif /* ECHIDNA_ALLOC_H */

