/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * HOTTEXT.H
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/18/92
 *   MODIFIED : 10/02/94
 *       TABS : 04 08
 *
 * DESCRIPTION
 *		Externs for routines to manipulate hottext strings.
 *		Hot text is text that has hot key information in it.
 *		Hot text strings are used on buttons and check boxes.
 *	
 * 	If the first char in string is '&' then the next char
 * 	in the string is the hot key that will act like a mouse click
 *		on the area labeled with the hottext.
 * 	The remainder of the string is what gets displayed on the 
 *		clickable area.
 * 	If the hot key char occurs in the displayed string
 *		then it will be underlined (first occurence only).
 *
 *		Examples:
 *
 *			HotText		Button Text		Hot Key
 *			----------	-----------		-------
 *			"&LLoad"	Load        		L	
 *						-          			
 *
 *			"&vSave"	Save				v
 *	              		-
 *			"&eRevert"	Revert				e
 *	             		-
 *
 *  		"&HFlipX"	FlipX				H
 *
 *			"&&Undo"	Undo				&
 *
 *			"&R&Redo"	&Redo				R
 *
 *			"&x"							x
 *			----------	-----------		-------
 *			Note: "&" is an INVALID hot text string, but "" is ok.
 *
 * HISTORY
 *
*/

#ifndef HOTTEXT_H
#define HOTTEXT_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define GetHotTextHotKey(pszHotText)	(		\
	(((char *)(pszHotText))[0] == '&') ? 			\
		(((char *)(pszHotText))[1])  :			\
		(0)										\
)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/


extern short DrawHotText (short x, short y, char *psz);
extern short DrawHotTextN (short x, short y, char *psz, size_t len);
extern short HotTextLen (char *psz);
extern short HotTextLenN (char *psz, size_t len);

#ifdef __cplusplus
}
#endif

#endif /* HOTTEXT_H */

