/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * LSTGADG.H
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/03/92
 *   MODIFIED : 07/26/94
 *       TABS : 04 08
 *
 * DESCRIPTION
 *		Routines to make and manage a list gadget (with up down arrow and 
 *		elevator etc.).
 *		
 *
 * HISTORY
 *
*/

#ifndef LSTGADG_H
#define LSTGADG_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/listfunc.h>
#include <echidna/windows.h>
#include <echidna/hitareas.h>
#include <echidna/scic.h>
#include <echidna/elvtgadg.h>
#include <echidna/naming.h>

/**************************** C O N S T A N T S ***************************/


#define LGAD_EXCLUSIVE			(1 << 0) /* Exlusive or group highlighting */
#define LGAD_H_ELEVATOR		(1 << 1) /* Enable Horizontal elevator */
#define LGAD_V_ELEVATOR		(1 << 2) /* Enable Vertical elevator */
#define LGAD_H_REALTIME_SLIDER	(1 << 3)
#define LGAD_V_REALTIME_SLIDER	(1 << 4)

#define LGAD_NOD_FOCUSED		(((UBYTE)1) << 6)
#define LGAD_NOD_SELECTED		(((UBYTE)1) << 7)


/* Flag masks for 'fWhy' parameter of PFNUPDATE function  */
#define LGAD_FOCUS_CHANGE		(1 << 0)	/* Focus status of some entry has
											 * changed.
											 */
#define LGAD_HIGHLIGHT_CHANGE	(1 << 1)	/* Highlighting on some entry has
											 * changed.
											 */
#define LGAD_DOUBLE_CLICK		(1 << 2)	/* User has double clicked on some
											 * entry.
											 */

/*
 * Position types for SetLGADListPos and SetLGADnodPos
 */
#define LGAD_POS_TOP			0
#define LGAD_POS_MID			1
#define LGAD_POS_BOT			2

/******************************** T Y P E S *******************************/

/* Function that displays whatever data is represented by 'pnod' at the 
 * coordinates (x,y,).
 */
typedef void (* PFNDSPNOD)(NOD *pnod, short x, short y);

/* Function that gets called when focus changes or highlighting changes
 * or double click happens. The parameter 'fWhy' has flags that tell 
 * why function is being called.
 */
typedef void (* PFNUPDATE)(void *pvUserData, UWORD fWhy);

typedef struct {

	/* PRIVATE VARIABLES: Use macros to look at them */

		/* Dimensions of list gadget area including elevators */
		short x, y;				
		short cx, cy;
		short xMaxin, yMaxin;	/* Coords of bottom right corner */
		UWORD Flags;
		short fDrawn;		/* True if LGAD has been drawn since init */

		struct {
			short x, y;
			short cx, cy;
			short xMaxin, 
				  yMaxin;
			short dxHPan;	/* Horizontal panning delta */
			short ieTop;	/* Index of element at top of display */
			short ixLeft;	/* Index of leftmost visible pixel of elements */
			short ceVMax;	/* Max elements that fit in display vertically */
			short ceVExcess;	/* Number of nodes that don't fit in display 
								 * vertically
								 */
			short cxeHExcess;	/* Number of x pixels of widest element that 
							 	 * don't fit in display horizontally.
								 */
			short cdxHPanExcess;/* Number of pan deltas it takes to scroll
								 * widest element onto display. 
								 */
		} D; /* Display area related fields */

		struct {
			PFNDSPNOD	 pfndspnod;	/* Display element function */
			LST			*plst;		/* List of elements */
			short		 c;			/* Number of elements in list */
			short		 cxMax;		/* Width of widest element */			
			short		 cy;  		/* Height of an element */
		} E; /* Element  related fields */

		struct {
			PFNUPDATE	pfnupdate; /* Function called when focus moves or 
									 * highlight changes.
									 */
			void		*pvUserData;/* Data passed to pfnupdate */
			UBYTE	 fExclusive;	/* Only one selection at a time flag */
			UBYTE	 fDragging;		/* True if drag selecting */
									/* Selected nodes will have Priority field
						 			 * of their node struct set.
						 			 */
			UBYTE	 fmTargetState;	/* Highlight target state flag and mask: 
									 * LGAD_NOD_SELECTED or 0 
									 */
			short	 ieDragBase;	/* Dragging range */
			short	 ieDragEnd;
			NOD		*pnodDragEnd;	
		} Sel; /* Selection related fields */

		struct {
			short cpel; /* thickness of elevators */
			struct {
				short	fEnabled; /* True if enabled */
				short	fOn; 	/* True if horizontal elevator is active */
				EGAD	egad;	/* Horizontal elevator gadget */
				short	cDelayUntilRepeat;
			} H;
			struct {
				short	fEnabled; /* True if enabled */
				short	fOn;	/* True if vertical elevator is active */
				EGAD	egad;	/* Vertical elevator gadget */
				short	cDelayUntilRepeat;
			} V;
			
		} Egad; /* Elelvator related fields */

		/* Hitarea array (1 list area +  2 elevators + 1 endarea)*/
		HitArea	rgha[1+2+1];

} LGAD;	/* List Gadget */

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define phaLGAD(plgad)	((plgad)->rgha)

#define InitTextLGAD(plgad,cx,cy,plst,Flags,cpelButtons,adH,adV,cxMax,pfn,pvdata)	\
	InitLGAD ((plgad),(cx),(cy),(plst),(Flags),(cpelButtons),			\
			(adH), (adV),												\
			(cxMax), (GetFontHeight(GetActiveWindow()->Font) + 1),		\
			(GetFontHeight(GetActiveWindow()->Font)),					\
			(DisplayTextLGAD),											\
			(pfn),														\
			(pvdata)													\
			)				

/* Get pointer to first selected node of list */
#define pnodLGADFirstSelected(plgad)	\
	pnodLGADFindSelected (Head((plgad)->E.plst))

/* Get pointer to first selected node after this one */
#define pnodLGADNextSelected(pnod)		\
	pnodLGADFindSelected (Next((pnod)))


#define LGADNodeIsSelected(pnod)	((pnod)->Priority & LGAD_NOD_SELECTED)
#define LGADNodeIsFocused(pnod)	((pnod)->Priority & LGAD_NOD_FOCUSED)

#define ieLGADGetTopElmntNdx(plgad)	((plgad)->D.ieTop)

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern PFNDSPNOD	DisplayTextLGAD;

/** !WARNING!: the list gadget uses the Priority field of the nodes in the
 **	 list as a highlight flag.
 **/

/* set up list gadget. returns TRUE if can make gadget of requested size */
extern short InitLGAD (
					LGAD	*plgad, 
					short	 cx, short cy,
					LST		*plst,
					UWORD	 Flags ,
					short	 cpelButtons,	/* width and height  of buttons */
					short	 cDelayUntilRepeatH,
					short	 cDelayUntilRepeatV,
					short	 cxElementMax,	/* width of widest element */
					short	 cyElement,		/* height of an element */
					short	 dxHPan,		/* Horizontal panning delta */
					PFNDSPNOD	 pfndspnod,	/* element display function */
					PFNUPDATE	 pfnupdate,	/* list update user function */
					void		*pvUserData /* Date passed to pfnupdate */
);

/* Draw the gadget according to its current settings */
extern void	DrawLGAD (LGAD *plgad, short x, short y);

/* Update list of names with new or changed list */
extern void	SetLGADList (LGAD *plgad, LST *plst, short cxElementMax);

extern NOD *pnodLGADFocused (LGAD *plgad);

/* Find first selected node in list (after and including one past in) 
 *	You probably want to use macros:
 *		pnodFirstSelected(plgad) and 
 *		pnodNextSelected(pnod)
 *	to access this function.
 */
extern NOD *pnodLGADFindSelected (NOD *pnod);

extern void SetLGADListPos (LGAD *plgad, short ieNdx, short fpos);
extern void SetLGADnodPos (LGAD *plgad, NodeType *pnod, short fpos);

#ifdef __cplusplus
}
#endif

#endif /* LSTGADG_H */

