/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * PLATFORM.H
 *
 * PROGRAMMER : Richard G. Marquez
 *    VERSION : 00.000
 *    CREATED : 12/22/89
 *   MODIFIED : 09/28/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		C environment defines, machine specific defines, etc.
 *
 *		Either __iAPx86__, __M68000__ or __EGGS_R4400__
 * 		should be defined as 1.
 *
 *		Either __MSDOS__, __AMIGAOS__, __MACOS__ or __EGGS_SGI__
 *		should be defined as 1.
 *
 *		far32 and huge32 should be defined so that the expression
 *
 *			char far32 *sz;
 *
 *		returns a 32-bit char pointer.
 *
 *		Thus on 16-bit Borland C++, far32 == "far" and huge32 == "huge",
 *		while on Watcom C/C++32, far32 == "" and huge32 == ""
 *		(because far on Watcom C/C++32 is actually 48-bits!)
 *
 * HISTORY
 *		09/02/93 (dcc) - per GAT, make processors not selected
 *						 #define'd to 0 instead of left undefined
 *		10/01/93 (GAT) - add Sexify Macros.
 *		12/04/93 (dcc) - add __MSDOS16__ (already added __MSDOS32X__)
 *		09/21/94 (dcc) - add support for 32-bit Borland C++ 4.02
 *
*/

#ifndef ECHIDNA_PLATFORM_H
#define ECHIDNA_PLATFORM_H

#ifdef __cplusplus
extern "C" {
#endif

/**************************** C O N S T A N T S ***************************/

/********************************************/
/* Set processor type:						*/
/* __iAPx86__, __M68000__ or __EGGS_R4400__ */

#undef	__BIGENDIAN__
#undef	__LITTLEENDIAN__

//	+- change this (set to 1 if 80x86, else set to 0)
//	V
#if 1
#ifndef	__iAPx86__
#define	__iAPx86__			1
#endif
#define	__LITTLEENDIAN__	1
#define	__BIGENDIAN__		0
#undef	__M68000__
#define	__M68000__			0
#undef	__EGGS_R4400__
#define	__EGGS_R4400__		0
#endif

//	+- and this (set to 1 if 680x0, else set to 0)
//	V
#if 0
#ifndef	__M68000__
#define	__M68000__			1
#endif
#define	__BIGENDIAN__		1
#define	__LITTLEENDIAN__	0
#undef	__iAPx86__
#define	__iAPx86__			0
#undef	__EGGS_R4400__
#define	__EGGS_R4400__		0
#endif

//	+- and this (set to 1 if R4400, else set to 0)
//	V
#if 0
#ifndef	__EGGS_R4400__
#define	__EGGS_R4400__		1
#endif
#define	__BIGENDIAN__		1
#define	__LITTLEENDIAN__	0
#undef	__iAPx86__
#define	__iAPx86__			0
#undef	__M68000__
#define	__M68000__			0
#endif

/************************************/
/* Set disk operating system:		*/
/* __MSDOS__, __AMIGADOS__,			*/
/* __MACOS__ or __EGGS_SGI__		*/

//	+- change this (set to 1 if MS-DOS, else set to 0)
//	V
#if 1
#ifndef __MSDOS__
#define	__MSDOS__		1
#endif
#undef	__AMIGAOS__
#define	__AMIGAOS__		0
#undef	__MACOS__
#define	__MACOS__		0
#undef	__EGGS_SGI__
#define	__EGGS_SGI__	0
#endif

//	+- and this (set to 1 if Amiga, else set to 0)
//	V
#if 0
#ifndef	__AMIGAOS__
#define	__AMIGAOS__		1
#endif
#undef	__MSDOS__
#define	__MSDOS__		0
#undef	__MACOS__
#define	__MACOS__		0
#undef	__EGGS_SGI__
#define	__EGGS_SGI__	0
#endif

//	+- and this (set to 1 if Macintosh, else set to 0)
//	V
#if 0
#ifndef	__MACOS__
#define	__MACOS__		1
#endif
#undef	__MSDOS__
#define	__MSDOS__		0
#undef	__AMIGAOS__
#define	__AMIGAOS__		0
#undef	__EGGS_SGI__
#define	__EGGS_SGI__	0
#endif

//	+- and this (set to 1 if SGI, else set to 0)
//	V
#if 0
#ifndef	__EGGS_SGI__
#define	__EGGS_SGI__	1
#endif
#undef	__MSDOS__
#define	__MSDOS__		0
#undef	__AMIGAOS__
#define	__AMIGAOS__		0
#undef	__MACOS__
#define	__MACOS__		0
#endif

/***********************************************/
/* you shouldn't need to change the following, */
/* as it should get computed automagically     */

/* These force the identifier to 0 if they are not defined */
/* This guarantees that these are the only ones used in our code. */

#if !defined(AZTEC_C)
#define	AZTEC_C 0
#endif

#if !defined(__BORLANDC__)
#define	__BORLANDC__ 0
#endif

#if !defined(DOS386)
#define DOS386 0
#endif

#if !defined(LATTICE)
#define	LATTICE 0
#endif

#if !defined(__MACSC__)
#define __MACSC__ 0
#endif

#if !defined(_MSC_VER)
#define _MSC_VER 0
#endif

#if !defined(__TURBOC__)
#define	__TURBOC__ 0
#endif

#if !defined(__WATCOMC__)
#define	__WATCOMC__ 0
#endif

#if !defined(__ZTC__)
#define	__ZTC__ 0
#endif

#undef	__MSDOS16__
#define	__MSDOS16__ 0

#undef	__MSDOS32X__
#define	__MSDOS32X__ 0

#undef	__NOT_ANSI__
#define	__NOT_ANSI__ 0

#if __TURBOC__
#if __FLAT__
#undef	__MSDOS32X__
#define	__MSDOS32X__ 1

#undef	far32
#define	far32

#undef	huge32
#define	huge32

#else // !__FLAT__
#undef	__MSDOS16__
#define	__MSDOS16__ 1

#undef	far32
#define	far32	far

#undef	huge32
#define	huge32	huge
#endif // !__FLAT__
#endif

#if (__WATCOMC__ && __DOS__)
#if __386__
#undef	__MSDOS32X__
#define	__MSDOS32X__ 1

#undef	far32
#define	far32

#undef	huge32
#define	huge32

#else
#undef	__MSDOS16__
#define	__MSDOS16__ 1

#undef	far32
#define	far32	far

#undef	huge32
#define	huge32	huge

#endif
#endif

#if __ZTC__
#if DOS386
#undef	__MSDOS32X__
#define	__MSDOS32X__ 1

#undef	far32
#define	far32

#undef	huge32
#define	huge32

#else
#undef	__MSDOS16__
#define	__MSDOS16__ 1

#undef	far32
#define	far32	far

#undef	huge32
#define	huge32	far

#endif
#endif

/* backwards compatibility stuff */
#if __iAPx86__
#undef	__INTEL__
#define	__INTEL__ 1
#else
#undef	__INTEL__
#define	__INTEL__ 0
#endif

/* amiga stuff */
#if __AMIGAOS__

#undef	far32
#define	far32

#undef	huge32
#define	huge32

#if LATTICE
/* we have far, near, and huge as real keywords with SAS/LATTICE C */

#else

#undef	__NOT_ANSI__
#define	__NOT_ANSI__ 1

#endif

#endif

/* mac stuff */
#if __MACOS__

#undef	far32
#define	far32

#undef	huge32
#define huge32

#endif

/* sgi stuff */
#if __EGGS_SGI__

#undef	far32
#define	far32

#undef	huge32
#define	huge32

#endif

/************************************  ************************************/
/************************* Word/Long Sexify Macros ************************/
/************************************  ************************************/

#if __LITTLEENDIAN__

	#define	BigLong(a)		(      ( ((((ULONG)(a)) & 0xFF000000UL) >> 24) |	\
									 ((((ULONG)(a)) & 0x00FF0000UL) >>  8) |	\
									 ((((ULONG)(a)) & 0x0000FF00UL) <<  8) |	\
									 ((((ULONG)(a)) & 0x000000FFUL) << 24) ))
	#define BigWord(a)		((UWORD)(((((UWORD)(a)) & 0x00FFU) << 8) |	\
									 ((((UWORD)(a)) & 0xFF00U) >> 8) ))
									 
	#define LilLong(a)		(a)
	#define	LilWord(a)		(a)
	
	#define BigLong2Native(a)	((a) = ( ((((ULONG)(a)) & 0xFF000000UL) >> 24) |	\
										 ((((ULONG)(a)) & 0x00FF0000UL) >>  8) |	\
										 ((((ULONG)(a)) & 0x0000FF00UL) <<  8) |	\
										 ((((ULONG)(a)) & 0x000000FFUL) << 24) ))
	#define BigWord2Native(a)	((a) = (UWORD)								\
									   (									\
										((((UWORD)(a)) & 0x00FFU) << 8) |	\
										((((UWORD)(a)) & 0xFF00U) >> 8)		\
									   ) )

	#define	LilLong2Native(a)	((a)=(a))
	#define	LilWord2Native(a)	((a)=(a))

#else

	#define	LilLong(a)		(      ( ((((ULONG)(a)) & 0xFF000000UL) >> 24) |	\
									 ((((ULONG)(a)) & 0x00FF0000UL) >>  8) |	\
									 ((((ULONG)(a)) & 0x0000FF00UL) <<  8) |	\
									 ((((ULONG)(a)) & 0x000000FFUL) << 24) ))
	#define LilWord(a)		((UWORD)(((((UWORD)(a)) & 0x00FFU) << 8) |	\
									 ((((UWORD)(a)) & 0xFF00U) >> 8) ))
									 
	#define BigLong(a)		(a)
	#define	BigWord(a)		(a)
	
	#define LilLong2Native(a)	((a) = ( ((((ULONG)(a)) & 0xFF000000UL) >> 24) |	\
										 ((((ULONG)(a)) & 0x00FF0000UL) >>  8) |	\
										 ((((ULONG)(a)) & 0x0000FF00UL) <<  8) |	\
										 ((((ULONG)(a)) & 0x000000FFUL) << 24) ))
	#define LilWord2Native(a)	((a) = (UWORD)								\
									   (									\
										((((UWORD)(a)) & 0x00FFU) << 8) |	\
										((((UWORD)(a)) & 0xFF00U) >> 8)		\
									   ) )

	#define	BigLong2Native(a)	((a)=(a))
	#define	BigWord2Native(a)	((a)=(a))

#endif

#define MakeBigLong(a)	((a) = BigLong((a)))
#define MakeBigWord(a)	((a) = BigWord((a)))
#define MakeLilLong(a)	((a) = LilLong((a)))
#define MakeLilWord(a)	((a) = LilWord((a)))

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_PLATFORM_H */

