/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * READIFF.H
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 05/07/90
 *   MODIFIED : 11/12/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Routines to read an IFF file.
 *
 * HISTORY
 *		10/05/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

#ifndef ECHIDNA_READIFF_H
#define ECHIDNA_READIFF_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <stdlib.h>
#include <string.h>
#if __TURBOC__
#include <alloc.h>
#endif

#include <echidna/alloc.h>
#include <echidna/eio.h>

/**************************** C O N S T A N T S ***************************/

#define PROP_CHUNK	MakeID('P','R','O','P')
#define FORM_CHUNK	MakeID('F','O','R','M')
#define LIST_CHUNK	MakeID('L','I','S','T')
#define CAT_CHUNK	MakeID('C','A','T',' ')

/******************************** T Y P E S *******************************/

typedef struct {
	ULONG		 ID;
	UWORD		 Flags;
	void		*Data;
} IFFData;

struct IFFRoutineID;

typedef struct {
	int					 fd;
	short				 Status;
	struct IFFRoutineID	*FORMTable;
	ULONG				 Size;
	ULONG				 Flags;
	char				 Name[EIO_MAXPATH];
	IFFData				 IFFData;
} IFFTracker;

typedef long (* IFFFuncPtr)(IFFTracker *iff);

typedef struct IFFRoutineID {
	ULONG		 ID;
	IFFFuncPtr	 IFFFunc;
} IFFRoutineID;

typedef struct {
	ULONG		 ID;
	ULONG		 Size;
} ChunkHeader;

/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

#define IFFmovmem(src,dest,len) memcpy((dest),(src),(len))

#if __AMIGAOS__
#define MakeID(a,b,c,d)	( (((long)(a)) << 24L) | (((long)(b)) << 16L) | (((long)(c)) << 8L) | (((long)(d)) << 0L) )
#define LongSex(a)		MakeBigLong(a)
#define WordSex(a)		MakeBigWord(a)
#define ULongSex(a)		MakeBigLong(a)
#define UWordSex(a)		MakeBigWord(a)
#define IFFGmalloc(s)	AllocMem((s),MEMF_CHIP)
#define IFFmalloc		malloc
#define IFFcalloc		calloc
#define IFFfree			free
#define IFFGfree(s)		FreeMem((s))
#endif
#if __MSDOS__
#define MakeID(a,b,c,d)	( (((ULONG)(a)) << 0) | (((ULONG)(b)) << 8) | (((ULONG)(c)) << 16) | (((ULONG)(d)) << 24) )
#define LongSex(a)		MakeBigLong(a)
#define WordSex(a)		MakeBigWord(a)

#if __MSDOS32X__
#define IFFmalloc		MEM_malloc
#define IFFcalloc		MEM_calloc
#define IFFfree			MEM_free
#else
#define IFFmalloc		MEM_farmalloc
#define IFFcalloc		MEM_farcalloc
#define IFFfree			MEM_farfree
#endif
#define IFFGmalloc(s)	AllocMemOnPara((s))
#define IFFGfree(s)		FreeMemOnPara((s))
#endif

#if __MACOS__
#define MakeID(a,b,c,d)	( (((long)(a)) << 24L) | (((long)(b)) << 16L) | (((long)(c)) << 8L) | (((long)(d)) << 0L) )
#define LongSex(a)		MakeBigLong(a)
#define WordSex(a)		MakeBigWord(a)
#define ULongSex(a)		MakeBigLong(a)
#define UWordSex(a)		MakeBigWord(a)
#define IFFGmalloc		MEM_malloc
#define IFFmalloc		MEM_malloc
#define IFFcalloc		MEM_calloc
#define IFFfree			MEM_free
#define IFFGfree		MEM_free
#endif

#if __EGGS_SGI__
#define MakeID(a,b,c,d)	( (((long)(a)) << 24L) | (((long)(b)) << 16L) | (((long)(c)) << 8L) | (((long)(d)) << 0L) )
#define LongSex(a)		MakeBigLong(a)
#define WordSex(a)		MakeBigWord(a)
#define ULongSex(a)		MakeBigLong(a)
#define UWordSex(a)		MakeBigWord(a)
#define IFFGmalloc		MEM_malloc
#define IFFmalloc		MEM_malloc
#define IFFcalloc		MEM_calloc
#define IFFfree			MEM_free
#define IFFGfree		MEM_free
#endif

/****************** F U N C T I O N   P R O T O T Y P E S *****************/

extern long IFFRead (IFFTracker *iff, void *data, long size);
extern long IFFSkip (IFFTracker *iff, long size);
extern long GetChunkHeader (IFFTracker *iff, ChunkHeader *chunkhdr);
extern long CallIDFunc (long id, IFFRoutineID *ifffunc, IFFTracker *iff);
extern long ReadIFF (IFFTracker *iff);
extern long SkipChunk (IFFTracker *iff);

#ifdef __cplusplus
}
#endif

#endif /* ECHIDNA_READIFF_H */

