/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * TREES.H
 *
 * PROGRAMMER : R.G. Marquez
 *    VERSION : 01.000
 *    CREATED : 07/31/89
 *   MODIFIED : 10/05/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 *	HISTORY
 *		07/31/89 (RGM) - Created header file.
 *		08/06/89 (RGM) - Changed from BSTfunctions.h to
 *                       trees.h. by adding generic tree functions.
 *		08/25/89 (RGM) - Made BSTfind & BSTremove functions again,
 *                       took out EOL in BSTprint, added TBSTcopy.
 *		08/27/89 (RGM) - Re-re did TBSTcopy, 'cause didn't get
 *                       backed up.
 *		01/26/90 (GAT) - Added nessasary Prototypes.  Added
 *                       TreeCFuncPtr, TreeDFuncPtr, TreePFuncPtr.
 *		02/05/92 (dcc) - Fixed typedef for TreeCFuncPtr.
 *						 Added InitBST(), TreeType typedef.
 *		10/05/94 (dcc) - updated to use <echidna/alloc.h>
 *		10/05/94 (dcc) - added '{' and '}' to definition of BTfree(),
 *						 NewTreeNode()
 *
*/

#ifndef ECHIDNA_TREES_H
#define ECHIDNA_TREES_H

#include <echidna/platform.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <echidna/etypes.h>
#include <echidna/alloc.h>

#if __NOT_ANSI__
#define __TREESIZETYPE__(elem)	(elem)
#else
#include <stdlib.h>
#define __TREESIZETYPE__(elem)	(size_t)(elem)
#endif

/*###########################   TYPEDEFS   ###############################*/

typedef struct TreeNodeDef *TreeNodePtrType;
typedef struct TreeNodeDef {
	void            *data;
	TreeNodePtrType leftchild,
	                rightchild;
} TreeNodeType;

typedef struct TreeNodeDef *TreeType;

typedef int   (*TreeCFuncPtr)(void *, void *);
typedef void *(*TreeCPFuncPtr)(TreeNodePtrType);
typedef void  (*TreeDFuncPtr)(TreeNodePtrType);
typedef void  (*TreePFuncPtr)(TreeNodePtrType);
typedef void  (*TreeP2FuncPtr)(TreeNodePtrType, TreePFuncPtr);

/*###########################   DEFINES   ################################*/


#define TREENODESIZE (sizeof (TreeNodeType))



/*========== GENERIC TREE support ==========*/

#define LEFTCHILD(ARG1)		(((TreeNodePtrType) (ARG1))->leftchild)
#define RIGHTCHILD(ARG1)	(((TreeNodePtrType) (ARG1))->rightchild)
#define DATA(ARG1)			(((TreeNodePtrType) (ARG1))->data)

#define BTnew(PTR_ADDRESS)	(*(PTR_ADDRESS)) = MEM_calloc (__TREESIZETYPE__(1), TREENODESIZE)
#define BTfree(PTR_ADDRESS)	{ MEM_free(*(PTR_ADDRESS));*(PTR_ADDRESS) = NULL; }

#define NewTreeNode(PTR_ADDRESS,DATA_ADDRESS) 		\
	{												\
		BTnew(PTR_ADDRESS);							\
		if (*(PTR_ADDRESS))							\
		{											\
			DATA(*(PTR_ADDRESS)) = DATA_ADDRESS;	\
		}											\
	}

#define InsertLeft(SOURCE,TARGET)	LEFTCHILD(TARGET) = SOURCE
#define InsertRight(SOURCE,TARGET)	RIGHTCHILD(TARGET) = SOURCE



/*========== BINARY SEARCH TREE support ==========*/

#define	InitBST(ptr) *(ptr) = NULL;
#define CopyTree(ARG1,ARG2,ARG3,ARG4)	TBSTcopy(ARG1,ARG2,ARG3,ARG4)
#define BSTclose(ARG1,ARG2,ARG3)		BTclose(ARG1,ARG2,ARG3)
#define BSTprint(ARG1,ARG2)				BTprint(ARG1,ARG2)

/*###########################  FUNCTIONS  ################################*/

#if __NOT_ANSI__

extern BOOL BSTinsert ();
extern void *BSTfind ();
extern void *BSTremove ();

extern void *TBSTcopy ();
extern void *TBSTfind ();
extern void *TBSTremove ();

extern void BTclose ();
extern void BTprint ();

#else


extern BOOL BSTinsert (TreeNodePtrType *A, void *x, TreeCFuncPtr);
extern void *BSTfind (TreeNodePtrType *A, void *x, TreeCFuncPtr);
extern void *BSTremove (TreeNodePtrType *A, void *x, TreeCFuncPtr);

extern void *TBSTcopy (TreeNodePtrType *A, TreeCPFuncPtr, TreeCFuncPtr, TreeDFuncPtr);
extern void *TBSTfind (TreeNodePtrType *A, void *x, TreeCFuncPtr);
extern void *TBSTremove (TreeNodePtrType *A, void *x, TreeCFuncPtr);

void BTclose (TreeNodePtrType *A, TreeCFuncPtr, TreeDFuncPtr);
extern void BTprint (TreeNodePtrType *A, TreePFuncPtr);
extern void BTRprint (TreeNodePtrType *A, TreeP2FuncPtr);

#endif

#ifdef __cplusplus
}
#endif

#endif /* trees.h */



#if 0 /*<><><><><><><><><><><><> DOCUMENTATION <><><><><><><><><><><><>*/



/** USAGE SUMMARY :


*** BINARY SEARCH Tree (DATA Related) ***

	STATUS =
		BSTinsert (&TREE_PTR, DATA_PTR, &COMPARE_FN);

	FOUND_DATA_PTR =
		BSTfind (&TREE_PTR, DATA_PTR, &COMPARE_FN);

	FOUND_DATA_PTR =
		BSTremove (&TREE_PTR, DATA_PTR, &COMPARE_FN, &DISPOSE_FN);

		BSTclose (&TREE_PTR, &COMPARE_FN, &DISPOSE_FN);

		BSTprint (&TREE_PTR, &PRINT_FN);



*** (Tree) BINARY SEARCH Tree (NODE Related) ***

	CLONE_TREE =
		TBSTcopy (&TREE_PTR, &COPY_FN, &COMPARE_FN, &DISPOSE_FN);

						or
	CLONE_TREE =
		CopyTree (&TREE_PTR, &COPY_FN, &COMPARE_FN, &DISPOSE_FN);


	FOUND_NODE_PTR =
		TBSTfind (&TREE_PTR, DATA_PTR, &COMPARE_FN);

	FOUND_NODE_PTR =
		TBSTremove (&TREE_PTR, DATA_PTR,



*** Generic Binary Tree related ***

	BTclose (&TREE_PTR, &COMPARE_FN, &DISPOSE_FN);

	BTprint (&TREE_PTR, &PRINT_FN);



*** Generic Tree related ***

	current_node = LEFTCHILD (TREE_NODE_PTR);
	LEFTCHILD (TREE_NODE_PTR) = new_node;

	current_node = RIGHTCHILD (TREE_NODE_PTR);
	RIGHTCHILD (TREE_NODE_PTR) = new_node;

	data_ptr = DATA (TREE_NODE_PTR);
	DATA (TREE_NODE_PTR) = &new_data;

	BTnew (&NEW_NODE_PTR);

	BTfree (&OLD_NODE_PTR);

	NewTreeNode (&NEW_NODE_PTR, &MY_DATA);

	InsertLeft (NEW_NODE_PTR, CURRENT_NODE_PTR);

	InsertRight (NEW_NODE_PTR, CURRENT_NODE_PTR);


END USAGE SUMMARY **/


/*********************************************************************
 *
 * InitBST {macro}
 *
 * SYNOPSIS
 *		void InitBST(TreeType *ptr)
 *
 * PURPOSE
 *		Initialize a TreeType structure to an empty binary search tree.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *		02/05/92 Wednesday (dcc) - created.
 *
 *
 * SEE ALSO
 *
*/



/**************************************************************************
**
** BSTinsert
**
** SYNOPSIS
**	BOOL BSTinsert (TreeNodePtrType *A,
**	                void            *x,
**	                int             (*comparefn)())
**
** USAGE
**	STATUS = BSTinsert (&TREE_PTR, DATA_PTR, &COMPARE_FN);
**
** FUNCTION
**	Insert a new data element into the BST.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	DATA_PTR    : A pointer to the data element to insert.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**
** RESULTS
**	Returns : TRUE if insert successful, FALSE if not.
**
** METHOD
**	If the BST is completely empty, we create a new node with hooks to
**	the data and change the tree pointer to point to our new tree.
**
**	If the BST has elements, we search until we find the correct
**	position for the data.  If the data already exists, we insert it as
**	the the node's right child (FIFO implementation).  If not, we insert
**	the new node as a leaf.
**
** BUGS
**	Use RAID (tm).
**
** HISTORY
**	07/31/89 Monday   - Finished. (RGM)
**
**	08/05/89 Saturday - Converted to FIFO implementation; deleted count,
**	                    deleted &DISPOSE_FN argument. (RGM)
**
** SEE ALSO
**	BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** BSTfind
**
** SYNOPSIS
**	void *BSTfind (TreeNodePtrType *ARG1,
**	               void            *ARG2,
**	               int             (*ARG3)() )
**
** USAGE
**	FOUND_DATA_PTR = BSTfind (&TREE_PTR, DATA_PTR, &COMPARE_FN);
**
** FUNCTION
**	Find a data element in a BST.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	DATA_PTR    : A pointer to the data element to find.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**
** RESULTS
**	Returns : A pointer to a data element, or NULL if not found.
**
** METHOD
**	Uses "TBSTfind()" and "DATA()".
**
** BUGS
**	Or use BlackFlag. . .
**
** HISTORY
**	07/31/89 Monday   - Finished. (RGM)
**
**	08/05/89 Saturday - Converted to a macro.
**
**	08/25/89 Friday   - Demacrofied ("Rise from your grave. . ."). (RGM)
**
** SEE ALSO
**	BSTinsert, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose, BTprint,
**	DATA (m).
*/



/**************************************************************************
**
** BSTremove
**
** SYNOPSIS
**	void *BSTremove (TreeNodePtrType *ARG1,
**	                 void            *ARG2,
**	                 int             (*ARG3)(),
**	                 void            (*ARG4)() )
**
** USAGE
**	FOUND_DATA_PTR = BSTremove (&TREE_PTR, DATA_PTR,
**                              &COMPARE_FN, &DISPOSE_FN);
**
** FUNCTION
**	Return a pointer to a matching BST data element, and delete the
**	node pointing to it.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	DATA_PTR    : A pointer to the data element to delete.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**	&DISPOSE_FN : The address of you dispose function.  Your dispose
**	              function should de-allocate the memory used by a data
**	              element when passed a pointer to it.
**
** RESULTS
**	Returns : A pointer to the matching data element, or NULL if not found.
**
** METHOD
**	Uses "TBSTfind()" and "DATA()".
**
** BUGS
**	Ha.  Tell me another.
**
** HISTORY
**	07/31/89 Monday   - Finished. (RGM)
**
**	08/05/89 Saturday - Changed to a macro.
**
**	08/25/89 Friday   - Demacrofied.  More filling.  Tastes gross. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** BSTclose	{MACRO}
**
** SYNOPSIS
**	void BSTclose (TreeNodePtrType *ARG1,
**	               int             (*ARG2)(),
**	               void            (*ARG3)() )
**
** USAGE
**	BSTclose (&TREE_PTR, &COMPARE_FN, &DISPOSE_FN);
**
** FUNCTION
**	Closes out a binary search tree.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**	&DISPOSE_FN : The address of you dispose function.  Your dispose
**	              function should de-allocate the memory used by a data
**	              element when passed a pointer to it.
**
** RESULTS
**	Returns : nothing.
**
** METHOD
**	Directly call "BTclose()".
**
** BUGS
**	E malone.
**
** HISTORY
**	08/01/89 Tuesday - Finished and documented. (RGM)
**
**	08/05/89 Saturday - Changed to macro. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** BSTprint	{MACRO}
**
** SYNOPSIS
**	void BSTprint (TreeNodePtrType *ARG1,
**	               void            (*ARG2)() )
**
** USAGE
**	BSTprint (&TREE_PTR, &PRINT_FN);
**
** FUNCTION
**	Prints out a binary search tree.
**
** INPUTS
**	&TREE_PTR : The address of your tree pointer variable.
**	&PRINT_FN : The address of your print function.  Your print function
**	            should take a pointer to a data element and print that data.
**
** RESULTS
**	Returns : nothing.
**
** METHOD
**	Direct call to "BTprint()".
**
** BUGS
**	None.
**
** HISTORY
**	08/01/89 Tuesday  - Finished and documented. (RGM)
**
**	08/05/89 Saturday - Changed to macro. (RGM)
**
**	08/26/89 Saturday - Got rid of EOL. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** TBSTcopy
**
** SYNOPSIS
**	void *TBSTcopy (TreeNodePtrType *A,
**	               void           *(*copyfn)(),
**	               int             (*comparefn)(),
**	               void            (*disposefn)() )
**
** USAGE
**	CLONE_TREE = TBSTcopy (&TREE_PTR, &COPY_FN, &COMPARE_FN, &DISPOSE_FN);
**
** FUNCTION
**	Clones a binary search tree.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	&COPY_FN    : The address of your copy data function.  Your copy data
**	              function should take the address of a piece of your data
**	              and make a duplicate of that data.  It should return a
**	              void pointer (void *) to the newly created data (the
**	              clone), or NULL upon failure.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**	&DISPOSE_FN : The address of you dispose function.  Your dispose
**	              function should de-allocate the memory used by a data
**	              element when passed a pointer to it.
**
** RESULTS
**	Returns : A pointer to a clone of the source tree.
**
** METHOD
**	Initialize an 'anchor' pointer for the destination tree.
**	Call a function ("rCopyBST") to recursively read data from the source
**	tree and put it in the 'anchored' (destination) tree.
**
** BUGS
**	E malone.
**
** HISTORY
**
**	08/25/89 Friday   - Created. (RGM)
**
**	08/26/89 Saturday - Made it use COPY_FN to copy the data, instead of
**	                    pointing to the same data in the source tree (which
**	                    it wasn't s'posta do ;)  ). (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTfind, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** TBSTfind
**
** SYNOPSIS
**	void *TBSTfind (TreeNodePtrType *A,
**	               void            *x,
**	               int             (*comparefn)() )
**
** USAGE
**	FOUND_NODE_PTR = TBSTfind (&TREE_PTR, DATA_PTR, &COMPARE_FN);
**
** FUNCTION
**	Find a data element node in a BST.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	DATA_PTR    : A pointer to the data element to find.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**
** RESULTS
**	Returns : A pointer to a data element node, or NULL if not found.
**
** METHOD
**	Traverses tree non-recursively.
**
** BUGS
**	Yea.  Right.  Give me a break.
**
** HISTORY
**	07/31/89 Monday   - Finished. (RGM)
**
**	08/05/89 Saturday - Converted from BSTfind to TBSTfind and supplied
**                       a MACRO for BSTfind. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTremove, BTclose, BTprint.
*/



/**************************************************************************
**
** TBSTremove
**
** SYNOPSIS
**	void *TBSTremove (TreeNodePtrType *A,
**	                 void            *x,
**	                 int             (*comparefn)(),
**	                 void            (*disposefn)() )
**
** USAGE
**	FOUND_NODE_PTR = TBSTremove (&TREE_PTR, DATA_PTR,
**                              &COMPARE_FN, &DISPOSE_FN);
**
** FUNCTION
**	Returns a pointer to a matching BST data node, and deletes the node.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	DATA_PTR    : A pointer to the data element to remove.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**	&DISPOSE_FN : The address of you dispose function.  Your dispose
**	              function should de-allocate the memory used by a data
**	              element when passed a pointer to it.
**
** RESULTS
**	Returns : A pointer to the matching data node, or NULL if not found.
**
** METHOD
**	Find the node (if any) containing matching data.  If it exists,
**	delete it using the user supplied DISPOSE_FN.
**
** BUGS
**	Now I find that just plain hard to believe.
**
** HISTORY
**	08/01/89 Tuesday  - Finished and documented. (RGM)
**
**	08/05/89 Saturday - Took out count.  Changed from BSTremove to
**	                    TBSTremove and added a MACRO for BSTremove. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, BTclose, BTprint.
*/



/**************************************************************************
**
** BTclose
**
** SYNOPSIS
**	void BTclose (TreeNodePtrType *A,
**	               int             (*comparefn)(),
**	               void            (*disposefn)() )
**
** USAGE
**	BTclose (&TREE_PTR, &COMPARE_FN, &DISPOSE_FN);
**
** FUNCTION
**	Closes out a binary tree.
**
** INPUTS
**	&TREE_PTR   : The address of your tree pointer variable.
**	&COMPARE_FN : The address of your compare function.  Your compare
**	              function should accept two pointers to like data
**	              elements, and should return an int such that :
**	                     DATA1 > DATA2  ==>  return  1
**	                     DATA1 = DATA2  ==>  return  0
**	                     DATA1 < DATA2  ==>  return -1
**	&DISPOSE_FN : The address of you dispose function.  Your dispose
**	              function should de-allocate the memory used by a data
**	              element when passed a pointer to it.
**
** RESULTS
**	Returns : nothing.
**
** METHOD
**	Delete data elements postorder using DISPOSE_FN and recursion.
**
** BUGS
**	E malone.
**
** HISTORY
**	08/01/89 Tuesday - Finished and documented. (RGM)
**
**	08/05/89 Saturday - Changed from BSTclose to generic BTclose.  Added
**	                    MACRO for BSTclose. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTprint.
*/


/**************************************************************************
**
** BTprint
**
** SYNOPSIS
**	void BTprint (TreeNodePtrType *A,
**	               void            (*printfn)() )
**
** USAGE
**	BTprint (&TREE_PTR, &PRINT_FN);
**
** FUNCTION
**	Prints out a binary tree.
**
** INPUTS
**	&TREE_PTR : The address of your tree pointer variable.
**	&PRINT_FN : The address of your print function.  Your print function
**	            should take a pointer to a data element and print that data.
**
** RESULTS
**	Returns : nothing.
**
** METHOD
**	Print a tree inorder using printf() and recursion.
**
** BUGS
**	None.
**
** HISTORY
**	08/01/89 Tuesday  - Finished and documented. (RGM)
**
**	08/05/89 Saturday - Took out count printing. (RGM)
**
**	08/26/89 Saturday - Took out auto EOL. (RGM)
**
** SEE ALSO
**	BSTinsert, BSTfind, BSTremove, BSTclose (m), BSTprint (m),
**	TBSTcopy, TBSTfind, TBSTremove, BTclose.
*/



/**************************************************************************
**
** LEFTCHILD	{MACRO}
**
** SYNOPSIS
**	LEFTCHILD (TreeNodePtrType arg1)
**
** USAGE
**	current_node = LEFTCHILD (TREE_NODE_PTR);
**
**	              and/or
**	            
**	LEFTCHILD (TREE_NODE_PTR) = new_node;
**
** FUNCTION
**	Accesses the 'left child' pointer of a tree node.
**
** INPUTS
**	TREE_NODE_PTR : A pointer that points to a binary tree node.
**
** RESULTS
**	Returns the 'left child' of a tree node. 
**
** METHOD
**	Give me a break.  This is a simple MACRO.
**
** BUGS
**	(SEE METHOD)
**
** HISTORY
**
**	08/06/89 Sunday - Finished. (RGM)
**
** SEE ALSO
**	RIGHTCHILD (m), DATA (m), InsertLeft (m), InsertRight (m).
*/



/**************************************************************************
**
** RIGHTCHILD	{MACRO}
**
** SYNOPSIS
**	RIGHTCHILD (TreeNodePtrType arg1)
**
** USAGE
**	current_node = RIGHTCHILD (TREE_NODE_PTR);
**
**	              and/or
**	            
**	RIGHTCHILD (TREE_NODE_PTR) = new_node;
**
** FUNCTION
**	Accesses the 'right child' of a tree node.
**
** INPUTS
**	TREE_NODE_PTR : A pointer that points to a binary tree node.
**
** RESULTS
**	Returns the 'right child' of a tree node. 
**
** METHOD
**	Give me another break.  This is a simple MACRO.
**
** BUGS
**	(SEE METHOD)
**
** HISTORY
**
**	08/06/89 Sunday - Finished. (RGM)
**
** SEE ALSO
**	LEFTCHILD (m), DATA (m), InsertLeft (m), InsertRight (m).
*/



/**************************************************************************
**
** DATA	{MACRO}
**
** SYNOPSIS
**	DATA (TreeNodePtrType arg1)
**
** USAGE
**	data_ptr = DATA (TREE_NODE_PTR);
**
**	           and/or
**
**	DATA (TREE_NODE_PTR) = &new_data;
**
** FUNCTION
**	Accesses the data field (data address) of a tree node.
**
** INPUTS
**	TREE_NODE_PTR : A pointer that points to a binary tree node.
**** RESULTS
**	Returns the data field of the tree node (as an address to the data).
**
** METHOD
**	Give me a third break.  This is a simple MACRO.
**
** BUGS
**	Beatles, ants, roaches, wasps.
**
** HISTORY
**
**	08/06/89 Sunday - Finished. (RGM)
**
** SEE ALSO
**	LEFTCHILD (m), RIGHTCHILD (m), InsertLeft (m), InsertRight (m).
*/



/**************************************************************************
**
** BTnew	{MACRO}
**
** SYNOPSIS
**	void BTnew (void *PTR_ADDRESS)
**
** USAGE
**	BTnew (&NEW_NODE_PTR);
**
** FUNCTION
**	Allocates and zeros out space for a new tree node.  Changes the pointer
**	to point to the new node.
**
** INPUTS
**	&NEW_NODE_PTR : The address of the new node's pointer.
**
** RESULTS
**	Sets the pointer to either the new node, or to NULL (on failure).
**
** METHOD
**	Uses "MEM_calloc()" to allocate and zero out memory.
**
** BUGS
**          + +
**   ()  -:====\  + + +
**   @@     + +\\======
** /\{}/\         + + +
** /\{}/\  : .           /\  /\
** /\||/\   .            /\O/\
**           
** HISTORY
**
**	08/06/89 Sunday - Finished.  Added ASCII bugs. (RGM)
**
** SEE ALSO
**	BTfree (m), NewTreeNode (m).
*/



/**************************************************************************
**
** BTfree	{MACRO}
**
** SYNOPSIS
**	void BTfree (void *PTR_ADDRESS)
**
** USAGE
**	BTfree (&OLD_NODE_PTR);
**
** FUNCTION
**	De-allocates the space of a previously allocated tree node.  'NULLs' out
**	the pointer.
**
** INPUTS
**	&OLD_NODE_PTR : The address of the node's pointer.
**
** RESULTS
**	De-allocates the node and set the pointer to NULL.
**
** METHOD
**	Uses "free()" to de-allocate memory.
**
** BUGS
**	maine () {inst i; fur (i=0; aye < INFINITY; eye++);}
**
** HISTORY
**
**	08/06/89 Sunday - Finished.  And I still say you are an idiot. (RGM)
**
** SEE ALSO
**	BTnew (m), NewTreeNode (m).
*/



/**************************************************************************
**
** NewTreeNode	{MACRO}
**** SYNOPSIS
**	void NewTreeNode (void **PTR_ADDRESS, void *DATA_ADDRESS)
**
** USAGE
**	NewTreeNode (&NEW_NODE_PTR, &MY_DATA);
**
** FUNCTION
**	Allocates and initializes a new binary tree node with the provided
**	data pointer.
**
** INPUTS
**	&NEW_NODE_PTR : The address of the new node's pointer.
**	&MY_DATA      : The Address of the to-be-inserted data.
**
** RESULTS
**	NEW_NODE_PTR will contain the address of the newly created node, or
**	NULL upon failure.
**
** METHOD
**	Uses BTnew to allocate the node.
**
** BUGS
**	Can I mumble dogface the bananna patch?
**
** HISTORY
**	1492     Saturday - Columbus sails the ocean blue.
**
** SEE ALSO
**	BTnew (m), BTfree (m).
*/



/**************************************************************************
**
** InsertLeft	{MACRO}
**
** SYNOPSIS
**	void InsertLeft (TreeNodePtrType SOURCE, TreeNodePtrType TARGET)
**
** USAGE
**	InsertLeft (NEW_NODE_PTR, CURRENT_NODE_PTR);
**
** FUNCTION
**	Inserts a new binary tree node to the 'left' of a given node.
**
** INPUTS
**	NEW_NODE_PTR     : A pointer to the node to insert.
**	CURRENT_NODE_PTR : A pointer to the 'target' node.
**
** RESULTS
**	The target node's 'leftchild' field will point to the new node.
**
** METHOD
**	Uses "LEFTCHILD" and assignment.
**
** BUGS
**	No way Jose'.
**
** HISTORY
**	1,000,000 BC  Friday - Gluut invents oooze. (RGM)
**
** SEE ALSO
**	LEFTCHILD (m), RIGHTCHILD (m), DATA (m), InsertRight (m).
*/



/**************************************************************************
**
** InsertRight	{MACRO}
**
** SYNOPSIS
**	void InsertRight (TreeNodePtrType SOURCE, TreeNodePtrType TARGET)
**
** USAGE
**	InsertRight (NEW_NODE_PTR, CURRENT_NODE_PTR);
**
** FUNCTION
**	Inserts a new binary tree node to the 'right' of a given node.
**
** INPUTS
**	NEW_NODE_PTR     : A pointer to the node to insert.
**	CURRENT_NODE_PTR : A pointer to the 'target' node.
**
** RESULTS
**	The target node's 'rightchild' field will point to the new node.
**
** METHOD
**	Uses "RIGHTCHILD" and assignment.
**
** BUGS
**	Nope.  To do it RIGHTCHILD.  I got my mind set on you.
**
** HISTORY
**	999,999 BC  Friday - Beta test of oooze ready. Gluut claims "it
**	                     no vaporware."  (RGM)
**
** SEE ALSO
**	LEFTCHILD (m), RIGHTCHILD (m), DATA (m), InsertLeft (m).
*/

#endif /*<><><><><><><><><><><> END DOCUMENTATION <><><><><><><><><><><><>*/

