/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * events.c
 *
 * Events and State Variables
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
*/
#include <echidna/platform.h>
#include "switches.h"

#include <echidna/menus2.h>
#include <echidna/listfunc.h>
#include "events.H"

#define LOCAL	static
#define FORWARD	extern
				/* To distingish between actual      */
				/* references and forward references */


/*
 * State variables.
 */
short		ShowTitleSTATE;
short		WLTS;
short		WLTE;
short		WLC2;
short		SpaceSTATE;
short		CycleColorsSTATE;
short		SwankyModeSTATE;
short		ScrollLockSTATE;
short		LockRoomSTATE;
short		ShowOnlyFloorSTATE;
short		JamPaletteSTATE;
short		ShowBrushSTATE;
short		XFlipBrushSTATE;
short		YFlipBrushSTATE;

/*
 * Extern all functions.
 */
extern short		NMap ();
extern short		LMap ();
extern short		AMap ();
extern short		SMap1 ();
extern short		SMap2 ();
extern short		tumeInfo ();
extern short		EditRoom ();
extern short		DrawRandom ();
extern short		ToggleTitleBar ();
extern short		About ();
extern short		Quit ();
extern short		QuitAndExit ();
extern short		TE_LoadTilesFullTiled ();
extern short		TE_LoadTilesFullTiledNoBlank ();
extern short		TE_LoadTilesCookieCutter ();
extern short		TE_LoadTilesBoxed ();
extern short		US ();
extern short		US2 ();
extern short		CRoom ();
extern short		DRoom ();
extern short		LRoom ();
extern short		SRoom ();
extern short		SRoomAll ();
extern short		ClearRoom ();
extern short		ClearRoom2 ();
extern short		ClearRoom3 ();
extern short		WCreate ();
extern short		WClose ();
extern short		WPRoom ();
extern short		WNRoom ();
extern short		WFRoom ();
extern short		WLS ();
extern short		WLE ();
extern short		WLC ();
extern short		WLCl ();
extern short		SpaceToggle ();
extern short		RightScroll ();
extern short		LeftScroll ();
extern short		UpScroll ();
extern short		DownScroll ();
extern short		RightScrollMultiple ();
extern short		LeftScrollMultiple ();
extern short		UpScrollMultiple ();
extern short		DownScrollMultiple ();
extern short		CenterRoomOnCursor ();
extern short		FloorUp ();
extern short		FloorDown ();
extern short		OpenLayer ();
extern short		CloseLayer ();
extern short		RoomStatus ();
extern short		VersionStatus ();
extern short		CopyrightStatus ();
extern short		CursorStatus ();
extern short		UserStatus ();
extern short		TUserStatus ();
extern short		ToggleCycleColors ();
extern short		SetTheColors ();
extern short		ToggleSwankyMode ();
extern short		ScrollLock ();
extern short		ScrollLockKEY ();
extern short		Undo ();
extern short		RenameRoom ();
extern short		RInfo ();
extern short		TUser ();
extern short		URoom ();
extern short		ToggleLockRoom ();
extern short		ToggleLInvisi ();
extern short		ToggleLLock ();
extern short		ToggleLAll ();
extern short		StripDownBlockCopy ();
extern short		CopyColors1 ();
extern short		CopyColors2 ();
extern short		CopyColors3 ();
extern short		ShowOnlyFloor ();
extern short		NTIMark ();
extern short		LastMenuEvent ();
extern short		IncGColor ();
extern short		DecGColor ();
extern short		ZGColor ();
extern short		TileComment1 ();
extern short		TileComment2 ();
extern short		TE_TogglePointer ();
extern short		TE_Debug ();
extern short		SMap3 ();
extern short		TE_SaveTilesAsBrushes ();
extern short		TE_MakeColor1 ();
extern short		TE_MakeColor2 ();
extern short		TE_MakeColor3 ();
extern short		TE_MakeColor4 ();
extern short		TE_MakeColorRevert ();
extern short		ToggleJamPalette ();
extern short		ToggleShowBrush ();
extern short		XFlipBrush ();
extern short		YFlipBrush ();

/*
 * Forward all States.
 */
FORWARD EventState	ClearMapState;
FORWARD EventState	LoadMapState;
FORWARD EventState	AppendMapState;
FORWARD EventState	SaveMapState;
FORWARD EventState	ZaveMapState;
FORWARD EventState	tumeInfoState;
FORWARD EventState	EditRoomState;
FORWARD EventState	DrawRandomState;
FORWARD EventState	ToggleTitleBarState;
FORWARD EventState	AboutState;
FORWARD EventState	QuitState;
FORWARD EventState	QuitAndExitState;
FORWARD EventState	LoadTilesFullTiledState;
FORWARD EventState	LoadTilesFullTiledNoBlankState;
FORWARD EventState	LoadTilesCookieCutterState;
FORWARD EventState	LoadTilesBoxedState;
FORWARD EventState	SelectBlockState;
FORWARD EventState	SelectSquareState;
FORWARD EventState	CreateRoomState;
FORWARD EventState	DeleteRoomState;
FORWARD EventState	LoadRoomState;
FORWARD EventState	SaveRoomState;
FORWARD EventState	SaveRoomAllState;
FORWARD EventState	ClearRoomState;
FORWARD EventState	ClearRoom2State;
FORWARD EventState	ClearRoom3State;
FORWARD EventState	WCreateState;
FORWARD EventState	WCloseState;
FORWARD EventState	PreviousRoomState;
FORWARD EventState	NextRoomState;
FORWARD EventState	FlipPanesState;
FORWARD EventState	WLockToSourceState;
FORWARD EventState	WLockToEditState;
FORWARD EventState	WLockToSameState;
FORWARD EventState	WLockClearState;
FORWARD EventState	SpaceToggleState;
FORWARD EventState	RightScrollState;
FORWARD EventState	LeftScrollState;
FORWARD EventState	UpScrollState;
FORWARD EventState	DownScrollState;
FORWARD EventState	RightScrollMultipleState;
FORWARD EventState	LeftScrollMultipleState;
FORWARD EventState	UpScrollMultipleState;
FORWARD EventState	DownScrollMultipleState;
FORWARD EventState	CenterRoomOnCursorState;
FORWARD EventState	FloorUpState;
FORWARD EventState	FloorDownState;
FORWARD EventState	OpenLayerState;
FORWARD EventState	CloseLayerState;
FORWARD EventState	RoomStatusState;
FORWARD EventState	VersionStatusState;
FORWARD EventState	CopyrightStatusState;
FORWARD EventState	CursorStatusState;
FORWARD EventState	UserStatusState;
FORWARD EventState	TileStatusState;
FORWARD EventState	ToggleCycleColorsState;
FORWARD EventState	SetTheColorsState;
FORWARD EventState	ToggleSwankyModeState;
FORWARD EventState	ScrollLockState;
FORWARD EventState	ScrollLockKEYState;
FORWARD EventState	UndoState;
FORWARD EventState	RenameRoomState;
FORWARD EventState	RoomInfoState;
FORWARD EventState	TilesetUserState;
FORWARD EventState	RoomUserState;
FORWARD EventState	ToggleLockRoomState;
FORWARD EventState	ToggleLInvisiState;
FORWARD EventState	ToggleLLockState;
FORWARD EventState	ToggleLAllState;
FORWARD EventState	StripDownBlockCopyState;
FORWARD EventState	CopyColors1State;
FORWARD EventState	CopyColors2State;
FORWARD EventState	CopyColors3State;
FORWARD EventState	ShowOnlyFloorState;
FORWARD EventState	NTIMarkState;
FORWARD EventState	LastMenuEventState;
FORWARD EventState	IncBackColorState;
FORWARD EventState	DecBackColorState;
FORWARD EventState	ZeroBackColorState;
FORWARD EventState	TileComment1State;
FORWARD EventState	TileComment2State;
FORWARD EventState	HideCursorState;
FORWARD EventState	DebugState;
FORWARD EventState	GroupSaveTilesState;
FORWARD EventState	SaveTilesAsBrushesState;
FORWARD EventState	MakeColor1State;
FORWARD EventState	MakeColor2State;
FORWARD EventState	MakeColor3State;
FORWARD EventState	MakeColor4State;
FORWARD EventState	MakeColorRevertState;
FORWARD EventState	ToggleJamPaletteState;
FORWARD EventState	ToggleShowBrushState;
FORWARD EventState	XFlipBrushState;
FORWARD EventState	YFlipBrushState;


/*
 * Exclude Lists.
 */

/*
 * Exclude list for 'WLockToSource'
 */
FORWARD ListType	__WLockToSourceExcList;
FORWARD StateTracker	__WLockToSourceExcWLockClear;

StateTracker __WLockToSourceExcWLockToEdit = {
	{
		(NodeType *)&__WLockToSourceExcWLockClear,
		(NodeType *)&__WLockToSourceExcList,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockToEditState,
};

StateTracker __WLockToSourceExcWLockClear = {
	{
		(NodeType *)&__WLockToSourceExcList.Tail,
		(NodeType *)&__WLockToSourceExcWLockToEdit,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockClearState,
};

ListType __WLockToSourceExcList = {
	(NodeType *)&__WLockToSourceExcWLockToEdit,
	NULL,
	(NodeType *)&__WLockToSourceExcWLockClear,
	sizeof (ListType),
	0,
	NULL,
};

/*
 * Exclude list for 'WLockToEdit'
 */
FORWARD ListType	__WLockToEditExcList;
FORWARD StateTracker	__WLockToEditExcWLockClear;

StateTracker __WLockToEditExcWLockToSource = {
	{
		(NodeType *)&__WLockToEditExcWLockClear,
		(NodeType *)&__WLockToEditExcList,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockToSourceState,
};

StateTracker __WLockToEditExcWLockClear = {
	{
		(NodeType *)&__WLockToEditExcList.Tail,
		(NodeType *)&__WLockToEditExcWLockToSource,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockClearState,
};

ListType __WLockToEditExcList = {
	(NodeType *)&__WLockToEditExcWLockToSource,
	NULL,
	(NodeType *)&__WLockToEditExcWLockClear,
	sizeof (ListType),
	0,
	NULL,
};

/*
 * Exclude list for 'WLockClear'
 */
FORWARD ListType	__WLockClearExcList;
FORWARD StateTracker	__WLockClearExcWLockToEdit;

StateTracker __WLockClearExcWLockToSource = {
	{
		(NodeType *)&__WLockClearExcWLockToEdit,
		(NodeType *)&__WLockClearExcList,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockToSourceState,
};

StateTracker __WLockClearExcWLockToEdit = {
	{
		(NodeType *)&__WLockClearExcList.Tail,
		(NodeType *)&__WLockClearExcWLockToSource,
		sizeof (StateTracker),
		0,
		NULL,
	},
	&WLockToEditState,
};

ListType __WLockClearExcList = {
	(NodeType *)&__WLockClearExcWLockToSource,
	NULL,
	(NodeType *)&__WLockClearExcWLockToEdit,
	sizeof (ListType),
	0,
	NULL,
};

/*
 * The States.
 */
FORWARD ListType	EventListx;

/*
 * 'ClearMap' State
 */
FORWARD EventState	LoadMapState;
EventState ClearMapState = {
	{
		(NodeType *)&LoadMapState,
		(NodeType *)&EventListx.Head,
		sizeof (EventState),
		0,
		"ClearMap",
	},
	NMap,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadMap' State
 */
FORWARD EventState	AppendMapState;
EventState LoadMapState = {
	{
		(NodeType *)&AppendMapState,
		(NodeType *)&ClearMapState,
		sizeof (EventState),
		0,
		"LoadMap",
	},
	LMap,
	NULL,
	NULL,
	NULL,
};

/*
 * 'AppendMap' State
 */
FORWARD EventState	SaveMapState;
EventState AppendMapState = {
	{
		(NodeType *)&SaveMapState,
		(NodeType *)&LoadMapState,
		sizeof (EventState),
		0,
		"AppendMap",
	},
	AMap,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SaveMap' State
 */
FORWARD EventState	ZaveMapState;
EventState SaveMapState = {
	{
		(NodeType *)&ZaveMapState,
		(NodeType *)&AppendMapState,
		sizeof (EventState),
		0,
		"SaveMap",
	},
	SMap1,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ZaveMap' State
 */
FORWARD EventState	tumeInfoState;
EventState ZaveMapState = {
	{
		(NodeType *)&tumeInfoState,
		(NodeType *)&SaveMapState,
		sizeof (EventState),
		0,
		"ZaveMap",
	},
	SMap2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'tumeInfo' State
 */
FORWARD EventState	EditRoomState;
EventState tumeInfoState = {
	{
		(NodeType *)&EditRoomState,
		(NodeType *)&ZaveMapState,
		sizeof (EventState),
		0,
		"tumeInfo",
	},
	tumeInfo,
	NULL,
	NULL,
	NULL,
};

/*
 * 'EditRoom' State
 */
FORWARD EventState	DrawRandomState;
EventState EditRoomState = {
	{
		(NodeType *)&DrawRandomState,
		(NodeType *)&tumeInfoState,
		sizeof (EventState),
		0,
		"EditRoom",
	},
	EditRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'DrawRandom' State
 */
FORWARD EventState	ToggleTitleBarState;
EventState DrawRandomState = {
	{
		(NodeType *)&ToggleTitleBarState,
		(NodeType *)&EditRoomState,
		sizeof (EventState),
		0,
		"DrawRandom",
	},
	DrawRandom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleTitleBar' State
 */
FORWARD EventState	AboutState;
EventState ToggleTitleBarState = {
	{
		(NodeType *)&AboutState,
		(NodeType *)&DrawRandomState,
		sizeof (EventState),
		0,
		"ToggleTitleBar",
	},
	ToggleTitleBar,
	&ShowTitleSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'About' State
 */
FORWARD EventState	QuitState;
EventState AboutState = {
	{
		(NodeType *)&QuitState,
		(NodeType *)&ToggleTitleBarState,
		sizeof (EventState),
		0,
		"About",
	},
	About,
	NULL,
	NULL,
	NULL,
};

/*
 * 'Quit' State
 */
FORWARD EventState	QuitAndExitState;
EventState QuitState = {
	{
		(NodeType *)&QuitAndExitState,
		(NodeType *)&AboutState,
		sizeof (EventState),
		0,
		"Quit",
	},
	Quit,
	NULL,
	NULL,
	NULL,
};

/*
 * 'QuitAndExit' State
 */
FORWARD EventState	LoadTilesFullTiledState;
EventState QuitAndExitState = {
	{
		(NodeType *)&LoadTilesFullTiledState,
		(NodeType *)&QuitState,
		sizeof (EventState),
		0,
		"QuitAndExit",
	},
	QuitAndExit,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadTilesFullTiled' State
 */
FORWARD EventState	LoadTilesFullTiledNoBlankState;
EventState LoadTilesFullTiledState = {
	{
		(NodeType *)&LoadTilesFullTiledNoBlankState,
		(NodeType *)&QuitAndExitState,
		sizeof (EventState),
		0,
		"LoadTilesFullTiled",
	},
	TE_LoadTilesFullTiled,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadTilesFullTiledNoBlank' State
 */
FORWARD EventState	LoadTilesCookieCutterState;
EventState LoadTilesFullTiledNoBlankState = {
	{
		(NodeType *)&LoadTilesCookieCutterState,
		(NodeType *)&LoadTilesFullTiledState,
		sizeof (EventState),
		0,
		"LoadTilesFullTiledNoBlank",
	},
	TE_LoadTilesFullTiledNoBlank,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadTilesCookieCutter' State
 */
FORWARD EventState	LoadTilesBoxedState;
EventState LoadTilesCookieCutterState = {
	{
		(NodeType *)&LoadTilesBoxedState,
		(NodeType *)&LoadTilesFullTiledNoBlankState,
		sizeof (EventState),
		0,
		"LoadTilesCookieCutter",
	},
	TE_LoadTilesCookieCutter,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadTilesBoxed' State
 */
FORWARD EventState	SelectBlockState;
EventState LoadTilesBoxedState = {
	{
		(NodeType *)&SelectBlockState,
		(NodeType *)&LoadTilesCookieCutterState,
		sizeof (EventState),
		0,
		"LoadTilesBoxed",
	},
	TE_LoadTilesBoxed,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SelectBlock' State
 */
FORWARD EventState	SelectSquareState;
EventState SelectBlockState = {
	{
		(NodeType *)&SelectSquareState,
		(NodeType *)&LoadTilesBoxedState,
		sizeof (EventState),
		0,
		"SelectBlock",
	},
	US,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SelectSquare' State
 */
FORWARD EventState	CreateRoomState;
EventState SelectSquareState = {
	{
		(NodeType *)&CreateRoomState,
		(NodeType *)&SelectBlockState,
		sizeof (EventState),
		0,
		"SelectSquare",
	},
	US2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CreateRoom' State
 */
FORWARD EventState	DeleteRoomState;
EventState CreateRoomState = {
	{
		(NodeType *)&DeleteRoomState,
		(NodeType *)&SelectSquareState,
		sizeof (EventState),
		0,
		"CreateRoom",
	},
	CRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'DeleteRoom' State
 */
FORWARD EventState	LoadRoomState;
EventState DeleteRoomState = {
	{
		(NodeType *)&LoadRoomState,
		(NodeType *)&CreateRoomState,
		sizeof (EventState),
		0,
		"DeleteRoom",
	},
	DRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LoadRoom' State
 */
FORWARD EventState	SaveRoomState;
EventState LoadRoomState = {
	{
		(NodeType *)&SaveRoomState,
		(NodeType *)&DeleteRoomState,
		sizeof (EventState),
		0,
		"LoadRoom",
	},
	LRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SaveRoom' State
 */
FORWARD EventState	SaveRoomAllState;
EventState SaveRoomState = {
	{
		(NodeType *)&SaveRoomAllState,
		(NodeType *)&LoadRoomState,
		sizeof (EventState),
		0,
		"SaveRoom",
	},
	SRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SaveRoomAll' State
 */
FORWARD EventState	ClearRoomState;
EventState SaveRoomAllState = {
	{
		(NodeType *)&ClearRoomState,
		(NodeType *)&SaveRoomState,
		sizeof (EventState),
		0,
		"SaveRoomAll",
	},
	SRoomAll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ClearRoom' State
 */
FORWARD EventState	ClearRoom2State;
EventState ClearRoomState = {
	{
		(NodeType *)&ClearRoom2State,
		(NodeType *)&SaveRoomAllState,
		sizeof (EventState),
		0,
		"ClearRoom",
	},
	ClearRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ClearRoom2' State
 */
FORWARD EventState	ClearRoom3State;
EventState ClearRoom2State = {
	{
		(NodeType *)&ClearRoom3State,
		(NodeType *)&ClearRoomState,
		sizeof (EventState),
		0,
		"ClearRoom2",
	},
	ClearRoom2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ClearRoom3' State
 */
FORWARD EventState	WCreateState;
EventState ClearRoom3State = {
	{
		(NodeType *)&WCreateState,
		(NodeType *)&ClearRoom2State,
		sizeof (EventState),
		0,
		"ClearRoom3",
	},
	ClearRoom3,
	NULL,
	NULL,
	NULL,
};

/*
 * 'WCreate' State
 */
FORWARD EventState	WCloseState;
EventState WCreateState = {
	{
		(NodeType *)&WCloseState,
		(NodeType *)&ClearRoom3State,
		sizeof (EventState),
		0,
		"WCreate",
	},
	WCreate,
	NULL,
	NULL,
	NULL,
};

/*
 * 'WClose' State
 */
FORWARD EventState	PreviousRoomState;
EventState WCloseState = {
	{
		(NodeType *)&PreviousRoomState,
		(NodeType *)&WCreateState,
		sizeof (EventState),
		0,
		"WClose",
	},
	WClose,
	NULL,
	NULL,
	NULL,
};

/*
 * 'PreviousRoom' State
 */
FORWARD EventState	NextRoomState;
EventState PreviousRoomState = {
	{
		(NodeType *)&NextRoomState,
		(NodeType *)&WCloseState,
		sizeof (EventState),
		0,
		"PreviousRoom",
	},
	WPRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'NextRoom' State
 */
FORWARD EventState	FlipPanesState;
EventState NextRoomState = {
	{
		(NodeType *)&FlipPanesState,
		(NodeType *)&PreviousRoomState,
		sizeof (EventState),
		0,
		"NextRoom",
	},
	WNRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'FlipPanes' State
 */
FORWARD EventState	WLockToSourceState;
EventState FlipPanesState = {
	{
		(NodeType *)&WLockToSourceState,
		(NodeType *)&NextRoomState,
		sizeof (EventState),
		0,
		"FlipPanes",
	},
	WFRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'WLockToSource' State
 */
FORWARD EventState	WLockToEditState;
EventState WLockToSourceState = {
	{
		(NodeType *)&WLockToEditState,
		(NodeType *)&FlipPanesState,
		sizeof (EventState),
		0,
		"WLockToSource",
	},
	WLS,
	&WLTS,
	MNU_HAS_RADIO,
	(ListType *)&__WLockToSourceExcList,
};

/*
 * 'WLockToEdit' State
 */
FORWARD EventState	WLockToSameState;
EventState WLockToEditState = {
	{
		(NodeType *)&WLockToSameState,
		(NodeType *)&WLockToSourceState,
		sizeof (EventState),
		0,
		"WLockToEdit",
	},
	WLE,
	&WLTE,
	MNU_HAS_RADIO,
	(ListType *)&__WLockToEditExcList,
};

/*
 * 'WLockToSame' State
 */
FORWARD EventState	WLockClearState;
EventState WLockToSameState = {
	{
		(NodeType *)&WLockClearState,
		(NodeType *)&WLockToEditState,
		sizeof (EventState),
		0,
		"WLockToSame",
	},
	WLC,
	NULL,
	NULL,
	NULL,
};

/*
 * 'WLockClear' State
 */
FORWARD EventState	SpaceToggleState;
EventState WLockClearState = {
	{
		(NodeType *)&SpaceToggleState,
		(NodeType *)&WLockToSameState,
		sizeof (EventState),
		0,
		"WLockClear",
	},
	WLCl,
	&WLC2,
	MNU_HAS_RADIO,
	(ListType *)&__WLockClearExcList,
};

/*
 * 'SpaceToggle' State
 */
FORWARD EventState	RightScrollState;
EventState SpaceToggleState = {
	{
		(NodeType *)&RightScrollState,
		(NodeType *)&WLockClearState,
		sizeof (EventState),
		0,
		"SpaceToggle",
	},
	SpaceToggle,
	&SpaceSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'RightScroll' State
 */
FORWARD EventState	LeftScrollState;
EventState RightScrollState = {
	{
		(NodeType *)&LeftScrollState,
		(NodeType *)&SpaceToggleState,
		sizeof (EventState),
		0,
		"RightScroll",
	},
	RightScroll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LeftScroll' State
 */
FORWARD EventState	UpScrollState;
EventState LeftScrollState = {
	{
		(NodeType *)&UpScrollState,
		(NodeType *)&RightScrollState,
		sizeof (EventState),
		0,
		"LeftScroll",
	},
	LeftScroll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'UpScroll' State
 */
FORWARD EventState	DownScrollState;
EventState UpScrollState = {
	{
		(NodeType *)&DownScrollState,
		(NodeType *)&LeftScrollState,
		sizeof (EventState),
		0,
		"UpScroll",
	},
	UpScroll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'DownScroll' State
 */
FORWARD EventState	RightScrollMultipleState;
EventState DownScrollState = {
	{
		(NodeType *)&RightScrollMultipleState,
		(NodeType *)&UpScrollState,
		sizeof (EventState),
		0,
		"DownScroll",
	},
	DownScroll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'RightScrollMultiple' State
 */
FORWARD EventState	LeftScrollMultipleState;
EventState RightScrollMultipleState = {
	{
		(NodeType *)&LeftScrollMultipleState,
		(NodeType *)&DownScrollState,
		sizeof (EventState),
		0,
		"RightScrollMultiple",
	},
	RightScrollMultiple,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LeftScrollMultiple' State
 */
FORWARD EventState	UpScrollMultipleState;
EventState LeftScrollMultipleState = {
	{
		(NodeType *)&UpScrollMultipleState,
		(NodeType *)&RightScrollMultipleState,
		sizeof (EventState),
		0,
		"LeftScrollMultiple",
	},
	LeftScrollMultiple,
	NULL,
	NULL,
	NULL,
};

/*
 * 'UpScrollMultiple' State
 */
FORWARD EventState	DownScrollMultipleState;
EventState UpScrollMultipleState = {
	{
		(NodeType *)&DownScrollMultipleState,
		(NodeType *)&LeftScrollMultipleState,
		sizeof (EventState),
		0,
		"UpScrollMultiple",
	},
	UpScrollMultiple,
	NULL,
	NULL,
	NULL,
};

/*
 * 'DownScrollMultiple' State
 */
FORWARD EventState	CenterRoomOnCursorState;
EventState DownScrollMultipleState = {
	{
		(NodeType *)&CenterRoomOnCursorState,
		(NodeType *)&UpScrollMultipleState,
		sizeof (EventState),
		0,
		"DownScrollMultiple",
	},
	DownScrollMultiple,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CenterRoomOnCursor' State
 */
FORWARD EventState	FloorUpState;
EventState CenterRoomOnCursorState = {
	{
		(NodeType *)&FloorUpState,
		(NodeType *)&DownScrollMultipleState,
		sizeof (EventState),
		0,
		"CenterRoomOnCursor",
	},
	CenterRoomOnCursor,
	NULL,
	NULL,
	NULL,
};

/*
 * 'FloorUp' State
 */
FORWARD EventState	FloorDownState;
EventState FloorUpState = {
	{
		(NodeType *)&FloorDownState,
		(NodeType *)&CenterRoomOnCursorState,
		sizeof (EventState),
		0,
		"FloorUp",
	},
	FloorUp,
	NULL,
	NULL,
	NULL,
};

/*
 * 'FloorDown' State
 */
FORWARD EventState	OpenLayerState;
EventState FloorDownState = {
	{
		(NodeType *)&OpenLayerState,
		(NodeType *)&FloorUpState,
		sizeof (EventState),
		0,
		"FloorDown",
	},
	FloorDown,
	NULL,
	NULL,
	NULL,
};

/*
 * 'OpenLayer' State
 */
FORWARD EventState	CloseLayerState;
EventState OpenLayerState = {
	{
		(NodeType *)&CloseLayerState,
		(NodeType *)&FloorDownState,
		sizeof (EventState),
		0,
		"OpenLayer",
	},
	OpenLayer,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CloseLayer' State
 */
FORWARD EventState	RoomStatusState;
EventState CloseLayerState = {
	{
		(NodeType *)&RoomStatusState,
		(NodeType *)&OpenLayerState,
		sizeof (EventState),
		0,
		"CloseLayer",
	},
	CloseLayer,
	NULL,
	NULL,
	NULL,
};

/*
 * 'RoomStatus' State
 */
FORWARD EventState	VersionStatusState;
EventState RoomStatusState = {
	{
		(NodeType *)&VersionStatusState,
		(NodeType *)&CloseLayerState,
		sizeof (EventState),
		0,
		"RoomStatus",
	},
	RoomStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'VersionStatus' State
 */
FORWARD EventState	CopyrightStatusState;
EventState VersionStatusState = {
	{
		(NodeType *)&CopyrightStatusState,
		(NodeType *)&RoomStatusState,
		sizeof (EventState),
		0,
		"VersionStatus",
	},
	VersionStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CopyrightStatus' State
 */
FORWARD EventState	CursorStatusState;
EventState CopyrightStatusState = {
	{
		(NodeType *)&CursorStatusState,
		(NodeType *)&VersionStatusState,
		sizeof (EventState),
		0,
		"CopyrightStatus",
	},
	CopyrightStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CursorStatus' State
 */
FORWARD EventState	UserStatusState;
EventState CursorStatusState = {
	{
		(NodeType *)&UserStatusState,
		(NodeType *)&CopyrightStatusState,
		sizeof (EventState),
		0,
		"CursorStatus",
	},
	CursorStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'UserStatus' State
 */
FORWARD EventState	TileStatusState;
EventState UserStatusState = {
	{
		(NodeType *)&TileStatusState,
		(NodeType *)&CursorStatusState,
		sizeof (EventState),
		0,
		"UserStatus",
	},
	UserStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'TileStatus' State
 */
FORWARD EventState	ToggleCycleColorsState;
EventState TileStatusState = {
	{
		(NodeType *)&ToggleCycleColorsState,
		(NodeType *)&UserStatusState,
		sizeof (EventState),
		0,
		"TileStatus",
	},
	TUserStatus,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleCycleColors' State
 */
FORWARD EventState	SetTheColorsState;
EventState ToggleCycleColorsState = {
	{
		(NodeType *)&SetTheColorsState,
		(NodeType *)&TileStatusState,
		sizeof (EventState),
		0,
		"ToggleCycleColors",
	},
	ToggleCycleColors,
	&CycleColorsSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'SetTheColors' State
 */
FORWARD EventState	ToggleSwankyModeState;
EventState SetTheColorsState = {
	{
		(NodeType *)&ToggleSwankyModeState,
		(NodeType *)&ToggleCycleColorsState,
		sizeof (EventState),
		0,
		"SetTheColors",
	},
	SetTheColors,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleSwankyMode' State
 */
FORWARD EventState	ScrollLockState;
EventState ToggleSwankyModeState = {
	{
		(NodeType *)&ScrollLockState,
		(NodeType *)&SetTheColorsState,
		sizeof (EventState),
		0,
		"ToggleSwankyMode",
	},
	ToggleSwankyMode,
	&SwankyModeSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'ScrollLock' State
 */
FORWARD EventState	ScrollLockKEYState;
EventState ScrollLockState = {
	{
		(NodeType *)&ScrollLockKEYState,
		(NodeType *)&ToggleSwankyModeState,
		sizeof (EventState),
		0,
		"ScrollLock",
	},
	ScrollLock,
	&ScrollLockSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'ScrollLockKEY' State
 */
FORWARD EventState	UndoState;
EventState ScrollLockKEYState = {
	{
		(NodeType *)&UndoState,
		(NodeType *)&ScrollLockState,
		sizeof (EventState),
		0,
		"ScrollLockKEY",
	},
	ScrollLockKEY,
	NULL,
	NULL,
	NULL,
};

/*
 * 'Undo' State
 */
FORWARD EventState	RenameRoomState;
EventState UndoState = {
	{
		(NodeType *)&RenameRoomState,
		(NodeType *)&ScrollLockKEYState,
		sizeof (EventState),
		0,
		"Undo",
	},
	Undo,
	NULL,
	NULL,
	NULL,
};

/*
 * 'RenameRoom' State
 */
FORWARD EventState	RoomInfoState;
EventState RenameRoomState = {
	{
		(NodeType *)&RoomInfoState,
		(NodeType *)&UndoState,
		sizeof (EventState),
		0,
		"RenameRoom",
	},
	RenameRoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'RoomInfo' State
 */
FORWARD EventState	TilesetUserState;
EventState RoomInfoState = {
	{
		(NodeType *)&TilesetUserState,
		(NodeType *)&RenameRoomState,
		sizeof (EventState),
		0,
		"RoomInfo",
	},
	RInfo,
	NULL,
	NULL,
	NULL,
};

/*
 * 'TilesetUser' State
 */
FORWARD EventState	RoomUserState;
EventState TilesetUserState = {
	{
		(NodeType *)&RoomUserState,
		(NodeType *)&RoomInfoState,
		sizeof (EventState),
		0,
		"TilesetUser",
	},
	TUser,
	NULL,
	NULL,
	NULL,
};

/*
 * 'RoomUser' State
 */
FORWARD EventState	ToggleLockRoomState;
EventState RoomUserState = {
	{
		(NodeType *)&ToggleLockRoomState,
		(NodeType *)&TilesetUserState,
		sizeof (EventState),
		0,
		"RoomUser",
	},
	URoom,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleLockRoom' State
 */
FORWARD EventState	ToggleLInvisiState;
EventState ToggleLockRoomState = {
	{
		(NodeType *)&ToggleLInvisiState,
		(NodeType *)&RoomUserState,
		sizeof (EventState),
		0,
		"ToggleLockRoom",
	},
	ToggleLockRoom,
	&LockRoomSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'ToggleLInvisi' State
 */
FORWARD EventState	ToggleLLockState;
EventState ToggleLInvisiState = {
	{
		(NodeType *)&ToggleLLockState,
		(NodeType *)&ToggleLockRoomState,
		sizeof (EventState),
		0,
		"ToggleLInvisi",
	},
	ToggleLInvisi,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleLLock' State
 */
FORWARD EventState	ToggleLAllState;
EventState ToggleLLockState = {
	{
		(NodeType *)&ToggleLAllState,
		(NodeType *)&ToggleLInvisiState,
		sizeof (EventState),
		0,
		"ToggleLLock",
	},
	ToggleLLock,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleLAll' State
 */
FORWARD EventState	StripDownBlockCopyState;
EventState ToggleLAllState = {
	{
		(NodeType *)&StripDownBlockCopyState,
		(NodeType *)&ToggleLLockState,
		sizeof (EventState),
		0,
		"ToggleLAll",
	},
	ToggleLAll,
	NULL,
	NULL,
	NULL,
};

/*
 * 'StripDownBlockCopy' State
 */
FORWARD EventState	CopyColors1State;
EventState StripDownBlockCopyState = {
	{
		(NodeType *)&CopyColors1State,
		(NodeType *)&ToggleLAllState,
		sizeof (EventState),
		0,
		"StripDownBlockCopy",
	},
	StripDownBlockCopy,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CopyColors1' State
 */
FORWARD EventState	CopyColors2State;
EventState CopyColors1State = {
	{
		(NodeType *)&CopyColors2State,
		(NodeType *)&StripDownBlockCopyState,
		sizeof (EventState),
		0,
		"CopyColors1",
	},
	CopyColors1,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CopyColors2' State
 */
FORWARD EventState	CopyColors3State;
EventState CopyColors2State = {
	{
		(NodeType *)&CopyColors3State,
		(NodeType *)&CopyColors1State,
		sizeof (EventState),
		0,
		"CopyColors2",
	},
	CopyColors2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'CopyColors3' State
 */
FORWARD EventState	ShowOnlyFloorState;
EventState CopyColors3State = {
	{
		(NodeType *)&ShowOnlyFloorState,
		(NodeType *)&CopyColors2State,
		sizeof (EventState),
		0,
		"CopyColors3",
	},
	CopyColors3,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ShowOnlyFloor' State
 */
FORWARD EventState	NTIMarkState;
EventState ShowOnlyFloorState = {
	{
		(NodeType *)&NTIMarkState,
		(NodeType *)&CopyColors3State,
		sizeof (EventState),
		0,
		"ShowOnlyFloor",
	},
	ShowOnlyFloor,
	&ShowOnlyFloorSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'NTIMark' State
 */
FORWARD EventState	LastMenuEventState;
EventState NTIMarkState = {
	{
		(NodeType *)&LastMenuEventState,
		(NodeType *)&ShowOnlyFloorState,
		sizeof (EventState),
		0,
		"NTIMark",
	},
	NTIMark,
	NULL,
	NULL,
	NULL,
};

/*
 * 'LastMenuEvent' State
 */
FORWARD EventState	IncBackColorState;
EventState LastMenuEventState = {
	{
		(NodeType *)&IncBackColorState,
		(NodeType *)&NTIMarkState,
		sizeof (EventState),
		0,
		"LastMenuEvent",
	},
	LastMenuEvent,
	NULL,
	NULL,
	NULL,
};

/*
 * 'IncBackColor' State
 */
FORWARD EventState	DecBackColorState;
EventState IncBackColorState = {
	{
		(NodeType *)&DecBackColorState,
		(NodeType *)&LastMenuEventState,
		sizeof (EventState),
		0,
		"IncBackColor",
	},
	IncGColor,
	NULL,
	NULL,
	NULL,
};

/*
 * 'DecBackColor' State
 */
FORWARD EventState	ZeroBackColorState;
EventState DecBackColorState = {
	{
		(NodeType *)&ZeroBackColorState,
		(NodeType *)&IncBackColorState,
		sizeof (EventState),
		0,
		"DecBackColor",
	},
	DecGColor,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ZeroBackColor' State
 */
FORWARD EventState	TileComment1State;
EventState ZeroBackColorState = {
	{
		(NodeType *)&TileComment1State,
		(NodeType *)&DecBackColorState,
		sizeof (EventState),
		0,
		"ZeroBackColor",
	},
	ZGColor,
	NULL,
	NULL,
	NULL,
};

/*
 * 'TileComment1' State
 */
FORWARD EventState	TileComment2State;
EventState TileComment1State = {
	{
		(NodeType *)&TileComment2State,
		(NodeType *)&ZeroBackColorState,
		sizeof (EventState),
		0,
		"TileComment1",
	},
	TileComment1,
	NULL,
	NULL,
	NULL,
};

/*
 * 'TileComment2' State
 */
FORWARD EventState	HideCursorState;
EventState TileComment2State = {
	{
		(NodeType *)&HideCursorState,
		(NodeType *)&TileComment1State,
		sizeof (EventState),
		0,
		"TileComment2",
	},
	TileComment2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'HideCursor' State
 */
FORWARD EventState	DebugState;
EventState HideCursorState = {
	{
		(NodeType *)&DebugState,
		(NodeType *)&TileComment2State,
		sizeof (EventState),
		0,
		"HideCursor",
	},
	TE_TogglePointer,
	NULL,
	NULL,
	NULL,
};

/*
 * 'Debug' State
 */
FORWARD EventState	GroupSaveTilesState;
EventState DebugState = {
	{
		(NodeType *)&GroupSaveTilesState,
		(NodeType *)&HideCursorState,
		sizeof (EventState),
		0,
		"Debug",
	},
	TE_Debug,
	NULL,
	NULL,
	NULL,
};

/*
 * 'GroupSaveTiles' State
 */
FORWARD EventState	SaveTilesAsBrushesState;
EventState GroupSaveTilesState = {
	{
		(NodeType *)&SaveTilesAsBrushesState,
		(NodeType *)&DebugState,
		sizeof (EventState),
		0,
		"GroupSaveTiles",
	},
	SMap3,
	NULL,
	NULL,
	NULL,
};

/*
 * 'SaveTilesAsBrushes' State
 */
FORWARD EventState	MakeColor1State;
EventState SaveTilesAsBrushesState = {
	{
		(NodeType *)&MakeColor1State,
		(NodeType *)&GroupSaveTilesState,
		sizeof (EventState),
		0,
		"SaveTilesAsBrushes",
	},
	TE_SaveTilesAsBrushes,
	NULL,
	NULL,
	NULL,
};

/*
 * 'MakeColor1' State
 */
FORWARD EventState	MakeColor2State;
EventState MakeColor1State = {
	{
		(NodeType *)&MakeColor2State,
		(NodeType *)&SaveTilesAsBrushesState,
		sizeof (EventState),
		0,
		"MakeColor1",
	},
	TE_MakeColor1,
	NULL,
	NULL,
	NULL,
};

/*
 * 'MakeColor2' State
 */
FORWARD EventState	MakeColor3State;
EventState MakeColor2State = {
	{
		(NodeType *)&MakeColor3State,
		(NodeType *)&MakeColor1State,
		sizeof (EventState),
		0,
		"MakeColor2",
	},
	TE_MakeColor2,
	NULL,
	NULL,
	NULL,
};

/*
 * 'MakeColor3' State
 */
FORWARD EventState	MakeColor4State;
EventState MakeColor3State = {
	{
		(NodeType *)&MakeColor4State,
		(NodeType *)&MakeColor2State,
		sizeof (EventState),
		0,
		"MakeColor3",
	},
	TE_MakeColor3,
	NULL,
	NULL,
	NULL,
};

/*
 * 'MakeColor4' State
 */
FORWARD EventState	MakeColorRevertState;
EventState MakeColor4State = {
	{
		(NodeType *)&MakeColorRevertState,
		(NodeType *)&MakeColor3State,
		sizeof (EventState),
		0,
		"MakeColor4",
	},
	TE_MakeColor4,
	NULL,
	NULL,
	NULL,
};

/*
 * 'MakeColorRevert' State
 */
FORWARD EventState	ToggleJamPaletteState;
EventState MakeColorRevertState = {
	{
		(NodeType *)&ToggleJamPaletteState,
		(NodeType *)&MakeColor4State,
		sizeof (EventState),
		0,
		"MakeColorRevert",
	},
	TE_MakeColorRevert,
	NULL,
	NULL,
	NULL,
};

/*
 * 'ToggleJamPalette' State
 */
FORWARD EventState	ToggleShowBrushState;
EventState ToggleJamPaletteState = {
	{
		(NodeType *)&ToggleShowBrushState,
		(NodeType *)&MakeColorRevertState,
		sizeof (EventState),
		0,
		"ToggleJamPalette",
	},
	ToggleJamPalette,
	&JamPaletteSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'ToggleShowBrush' State
 */
FORWARD EventState	XFlipBrushState;
EventState ToggleShowBrushState = {
	{
		(NodeType *)&XFlipBrushState,
		(NodeType *)&ToggleJamPaletteState,
		sizeof (EventState),
		0,
		"ToggleShowBrush",
	},
	ToggleShowBrush,
	&ShowBrushSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'XFlipBrush' State
 */
FORWARD EventState	YFlipBrushState;
EventState XFlipBrushState = {
	{
		(NodeType *)&YFlipBrushState,
		(NodeType *)&ToggleShowBrushState,
		sizeof (EventState),
		0,
		"XFlipBrush",
	},
	XFlipBrush,
	&XFlipBrushSTATE,
	MNU_HAS_CHECK,
	NULL,
};

/*
 * 'YFlipBrush' State
 */
EventState YFlipBrushState = {
	{
		(NodeType *)&EventListx.Tail,
		(NodeType *)&XFlipBrushState,
		sizeof (EventState),
		0,
		"YFlipBrush",
	},
	YFlipBrush,
	&YFlipBrushSTATE,
	MNU_HAS_CHECK,
	NULL,
};

ListType EventListx = {
	(NodeType *)&ClearMapState,
	NULL,
	(NodeType *)&YFlipBrushState,
	sizeof (ListType),
	0,
	NULL,
};

ListType *EventList = &EventListx;

