/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * 1floppy.c
 *
 * PROGRAMMER : Dan Chang
 *    VERSION : 00.000
 *    CREATED : 03/28/91
 *   MODIFIED : 02/05/95
 *       TABS : 04 07
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Detect how many floppy drives are in the system.
 *
 * HISTORY
 *		From _Suppressing "Insert Diskette in B:" DOS Messages_ by Chris Howe,
 *		TECH Specialist, April 1991, Volume 2, Number 4, pages 35..37.
 *
 *		10/12/93 (dcc) - Updated for MS-DOS Watcom C32.
 *		02/05/95 (dcc) - #include <echidna/eui.h> to verify function prototypes.
 *
*/

#include <echidna/platform.h>

#include <echidna/etypes.h>
#include <echidna/eui.h>					// verify function prototypes

#include <ctype.h>
#if __TURBOC__
#include <dir.h>
#include <dos.h>
#elif __WATCOMC__
#include "diskp.h"
#endif

#if __MSDOS32X__
#include <echidna/rmdos.h>
#endif

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/


int single_drive_flag;						/* global flag used by SetFloppy() */

/* SingleDrive - returns TRUE if single floppy system, else return FALSE. */

BOOL SingleDrive(void)
{
#if __MSDOS32X__
	char FloppyPresent, FloppyNumber;

	FloppyPresent =  (*((char far32 *) rmMK_FP32(0x40, 0x10))) & 1;
	FloppyNumber = (((*((char far32 *) rmMK_FP32(0x40, 0x10))) & 0xC0) >> 6) + 1;
	return (single_drive_flag = FloppyPresent && (FloppyNumber == 1));
#else
	char FloppyPresent, FloppyNumber;

	FloppyPresent =  (*((char far32 *) MK_FP(0x40, 0x10))) & 1;
	FloppyNumber = (((*((char far32 *) MK_FP(0x40, 0x10))) & 0xC0) >> 6) + 1;
	return (single_drive_flag = FloppyPresent && (FloppyNumber == 1));
#endif
}

/* SetFloppy - in one floppy drive machines, the drive may be referred
	to as drive A: or drive B:. This routine sets an obscure flag in
	the BIOS data area at 0050:0004h, which is the current "name" of
	the floppy drive to be the same as the one specified in the first
	character of the filename. This will suppress the dreaded
	"Insert diskette for drive B: and..." message.
	Note: call SingleDrive() before calling this routine. */

void SetFloppy(char *filename)
{
	char disk_name;

	if (single_drive_flag)
	{
		if (filename[1] == ':')				/* if a drive was specified, */
			disk_name = toupper(filename[0]);		/* get the letter. */
		else										/* Else get default drive */
			disk_name = getdisk()+'A';		/*			...just in case. */
#if __MSDOS32X__
		if (disk_name == 'A')				/* Adjust byte at */
			*((char far32 *) rmMK_FP32(0x50, 4)) = 0;	/* 0050:0004h as needed. */
		else if (disk_name == 'B')
			*((char far32 *) rmMK_FP32(0x50, 4)) = 1;
#else
		if (disk_name == 'A')				/* Adjust byte at */
			*((char far32 *) MK_FP(0x50, 4)) = 0;	/* 0050:0004h as needed. */
		else if (disk_name == 'B')
			*((char far32 *) MK_FP(0x50, 4)) = 1;
#endif
	}
}

