/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * LSTGADG.C
 *
 * PROGRAMMER : Juan M. Alvarado
 *    VERSION : 00.000
 *    CREATED : 02/05/92
 *   MODIFIED : 07/27/94
 *       TABS : 04 08
 *
 *	     \|///-_
 *	     \oO///_
 *	-----w/-w-----
 *	 E C H i D N A
 *	--------------
 *
 * DESCRIPTION
 *		Routines for managing list gadget.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <limits.h>
#include <echidna/etypes.h>
#include <echidna/grafx.h>
#include <echidna/hitareas.h>
#include <echidna/eui.h>
#include <echidna/scic.h>
#include <echidna/elvtgadg.h>
#include <echidna/lstgadg.h>
#include <echidna/mouse.h>
#include <echidna/eerrors.h>
#include <echidna/naming.h>

/**************************** C O N S T A N T S ***************************/
#define FORWARD	extern

enum {iList, iegadH, iegadV};

/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/

/* Value returned must be checked for being in list range */
#define ieTargetFromYOff(plgad, dyha)					\
	((dyha < 0) ? (plgad)->D.ieTop - 1 : 				\
		((plgad)->D.ieTop +  (dyha) / (plgad)->E.cy)	\
	)


/***************************** R O U T I N E S ****************************/

FORWARD void LGAD_UpdateV (void *data);
FORWARD void LGAD_UpdateH (void *data);
FORWARD short LGAD_DrawList (LGAD *plgad);



/* Assumes entry is in display area 
 * fInited (Initialized) should be false the first time you call this so 
 * it can initialize its static variables and TRUE after that until the
 * values of plgad change.
*/
void LGAD_DrawEntry (LGAD *plgad, NOD *pnod, short ie, short fInited)
{
	static short	x;
	static short	y;
	static short	cx;
	static short	cyE;
	static short	xDraw;
	static short	ieDTop;
	static short	White;
	static short	Black;
	short	yDraw;

	if (!plgad->fDrawn) return;

	if (!fInited) {
		x = plgad->D.x;
		y = plgad->D.y;
		cx = plgad->D.cx;
		cyE = plgad->E.cy;
		xDraw = x - plgad->D.ixLeft * plgad->D.dxHPan;
		ieDTop = plgad->D.ieTop;
		White = GetActiveWindow()->White;
		Black = GetActiveWindow()->Black;
	}

	yDraw = y + (ie - ieDTop) * cyE;

	if (!pnod) {
		pnod = Head (plgad->E.plst);
		while (ie) {
			pnod = Next (pnod);
			--ie;
		}
	} 

	BeforeGraphics ();
	PushClipValues (x, yDraw, cx, cyE);
	SetPenColor (White);
	DrawRect (x, yDraw, cx, cyE);
	plgad->E.pfndspnod (pnod, xDraw, yDraw);
	if (pnod->Priority & LGAD_NOD_SELECTED) {
		SetPenColor (White ^ Black);
		DrawXorRect (x, yDraw, cx, cyE);
	}
	if (pnod->Priority & LGAD_NOD_FOCUSED) {
		SetRectPat (rgpubEUIGhostPats[0]);
		DrawPatXorBox (x, yDraw, cx, cyE);
	}

	PopClipValues();
	AfterGraphics ();
}

short ieBotExGet (LGAD *plgad, short ieTop) {
	short ieBotEx = ieTop + plgad->D.ceVMax;
	if (ieBotEx > plgad->E.c) {
		ieBotEx = plgad->E.c;
	}
	return ieBotEx;
}

/*********************************************************************
 *
 * StartHighLightDrag
 *
 * SYNOPSIS
 *		void StartHighLightDrag (LGAD *plgad, short ieTarget)
 *
 * PURPOSE
 *		To initiate a mouse drag over list area.
 *
 * INPUT
 *		ieTarget:	index to list entry that mouse currently points to.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void StartHighLightDrag (LGAD *plgad, short ieTarget)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StartHighLightDrag";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	NOD		*pnod;
	short	 ie;
	short	 fInited;
	short	 fExclusive;
	short	 ieDTop = plgad->D.ieTop;
	short	 ieDBotEx = ieBotExGet (plgad, ieDTop);
	short	 fmTargetState;

	plgad->Sel.ieDragBase = plgad->Sel.ieDragEnd = ieTarget;

	ie = 0;
	fInited = FALSE;
	fExclusive = plgad->Sel.fExclusive;

	pnod = Head (plgad->E.plst);
	while (ie != ieTarget) {
		short fDrawIt = FALSE;
		if (fExclusive && (pnod->Priority & LGAD_NOD_SELECTED)) {
			/* Unhighlight entry */
			pnod->Priority &= ~LGAD_NOD_SELECTED;
			++fDrawIt;
		}
		if (pnod->Priority & LGAD_NOD_FOCUSED) {
			pnod->Priority &= ~LGAD_NOD_FOCUSED;
			++fDrawIt;
		}

		if ((ie >= ieDTop && ie < ieDBotEx) && fDrawIt) {
			LGAD_DrawEntry (plgad, pnod, ie, fInited++);
		}
		pnod = Next (pnod); ++ie;
	}

	pnod->Priority |= LGAD_NOD_FOCUSED;
	plgad->Sel.pnodDragEnd = pnod;
	if (!(pnod->Priority & LGAD_NOD_SELECTED) || fExclusive) {
		fmTargetState = plgad->Sel.fmTargetState = LGAD_NOD_SELECTED;
	} else {
		fmTargetState = plgad->Sel.fmTargetState = 0;
	}
	pnod->Priority = (pnod->Priority & ~LGAD_NOD_SELECTED) | fmTargetState;
	if (ie >= ieDTop && ie < ieDBotEx) {
		LGAD_DrawEntry (plgad, pnod, ie, fInited++);
	}


	pnod = Next (pnod); ++ie;
	while (!IsEOList (pnod)) {
		short fDrawIt = FALSE;
		if (fExclusive && (pnod->Priority & LGAD_NOD_SELECTED)) {
			pnod->Priority &= ~LGAD_NOD_SELECTED;
			++fDrawIt;
		}
		if (pnod->Priority & LGAD_NOD_FOCUSED) {
			pnod->Priority &= ~LGAD_NOD_FOCUSED;
			++fDrawIt;
		}
		if ((ie >= ieDTop && ie < ieDBotEx) && fDrawIt) {
			LGAD_DrawEntry (plgad, pnod, ie, fInited++);
		}
		pnod = Next (pnod); ++ie;
	}
	return;

}} /* StartHighLightDrag */

/*********************************************************************
 *
 * HighLightDrag
 *
 * SYNOPSIS
 *		void HighLightDrag (PLGAD *plgad, short ieTarget)
 *
 * PURPOSE
 *		To drag the highlight to ieTarget.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void HighLightDrag (LGAD *plgad, short ieTarget)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "HighLightDrag";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short	 ieDragEnd = plgad->Sel.ieDragEnd;
	short	ieBotEx = ieBotExGet (plgad, plgad->D.ieTop);

	if (ieTarget >= ieBotEx || ieTarget < 0) {
		return;
	}


	if (ieTarget != ieDragEnd) /* moved into new entry */
	{
		short fInited = FALSE;
		NOD		*pnod = plgad->Sel.pnodDragEnd;
		short	 fBackward = (ieTarget < ieDragEnd);
		short	 fDrawIt;
		short	 ieTop = plgad->D.ieTop;
		short	 ieBotEx = ieBotExGet (plgad, ieTop);

		/* Unfocus original DragEnd */
		pnod->Priority &= ~LGAD_NOD_FOCUSED;
		fDrawIt = TRUE;

		for (;;) {
			short fPriorityOn = (plgad->Sel.fExclusive) ? 
				(ieDragEnd == ieTarget) : plgad->Sel.fmTargetState;
			short mPriority = (fPriorityOn) ? LGAD_NOD_SELECTED : 0;

			if (mPriority != (pnod->Priority & LGAD_NOD_SELECTED)) {
				pnod->Priority = (pnod->Priority & ~LGAD_NOD_SELECTED) | 
					mPriority;
				++fDrawIt;
			}
			if (fDrawIt) {
				if (ieDragEnd >= ieTop  && ieDragEnd < ieBotEx)
				{
					LGAD_DrawEntry (plgad, pnod, ieDragEnd, fInited++);
				}
				fDrawIt = FALSE;
			}

			if (ieTarget == ieDragEnd) break;

			if (fBackward) {
				pnod = Prev (pnod); --ieDragEnd;
			} else {
				pnod = Next (pnod); ++ieDragEnd;
			}

		} 

		plgad->Sel.ieDragEnd = ieDragEnd;
		plgad->Sel.pnodDragEnd = pnod;
		pnod->Priority |= LGAD_NOD_FOCUSED;
		LGAD_DrawEntry (plgad, pnod, ieDragEnd, fInited);

	}

}} /* HighLightDrag */


/*********************************************************************
 *
 * ScrollIfNeedTo
 *
 * SYNOPSIS
 *		void ScrollIfNeedTo (LGAD *plgad, short *pieTarget, short dxha)
 *
 * PURPOSE
 *		Check if target or x offset from hit area start is out of bounds.
 *		If so then scroll list (if possible) and make sure *pieTarget stays
 *		in valid range.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void ScrollIfNeedTo (LGAD *plgad, short *pieTarget, short dxha)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "ScrollIfNeedTo";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short ieTarget = *pieTarget;
	short	 ieTop = plgad->D.ieTop;
	short	 ieBotEx = ieBotExGet (plgad, ieTop);

	if (dxha < 0) { 
		if(plgad->Egad.H.fOn && plgad->D.ixLeft > 0) { 	/* Scroll Left */
			SetFloorPositionEGAD (&plgad->Egad.H.egad, 
				plgad->D.cdxHPanExcess + 1, plgad->D.ixLeft - 1);
		} 
	} else if (dxha > plgad->D.cx) {
		if (plgad->Egad.H.fOn && plgad->D.ixLeft < 
			(FloorsMaxEGAD(&plgad->Egad.H.egad) - 1)) 
		{									/* Scroll Right */
			SetFloorPositionEGAD (&plgad->Egad.H.egad,
				plgad->D.cdxHPanExcess + 1, plgad->D.ixLeft + 1);
		} 
	}
	if (ieTarget < ieTop) {
		if (plgad->Egad.V.fOn && ieTop > 0) {	/* Scroll Up */
			SetFloorPositionEGAD (&plgad->Egad.V.egad, 
				plgad->D.ceVExcess + 1, ieTop - 1);
		}
		ieTarget = plgad->D.ieTop;
	} else if (ieTarget >= ieBotEx) {
		if (plgad->Egad.V.fOn && ieTop < 
			(FloorsMaxEGAD(&plgad->Egad.V.egad) - 1))
		{									/* Scroll Down */
			SetFloorPositionEGAD (&plgad->Egad.V.egad, 
				plgad->D.ceVExcess + 1, ieTop + 1);
		}
		ieTarget = ieBotExGet (plgad, plgad->D.ieTop) - 1;
	} 

	*pieTarget = ieTarget;

}} /* ScrollIfNeedTo */

/*********************************************************************
 *
 * SetLGADListPos
 *
 * SYNOPSIS
 *		void SetLGADListPos (LGAD *plgad, short ieNdx, short fpos)
 *
 * PURPOSE
 *		Sets element 'ieNdx' to be displayed in plgad LGAD
 *		based on fpos.
 *
 * INPUT
 *		ieNdx	: Index to element of list
 *		fpos	: position for element to be displayed
 *
 *		LGAD_POS_TOP	= display at top.
 *		LGAD_POS_MID	= display in middle.
 *		LGAD_POS_BOT	= display at bottom.
 *
 * EFFECTS
 *		Attempts to setup plgad LGAD to display element 'ieNdx' at requested
 *		position if it is possible to do so.
 *
 * RETURN VALUE
 *		None
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		SetLGADnodPos
*/
void SetLGADListPos (LGAD *plgad, short ieNdx, short fpos)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetLGADListPos";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		short	ceVMax  = plgad->D.ceVMax;

		/*
		** Adjust ieNdx based on fpos
		*/
		switch (fpos) {
		case LGAD_POS_MID:
			ieNdx -= ceVMax / 2;
			break;
		case LGAD_POS_BOT:
			ieNdx -= ceVMax - 1;
			break;
		}
		ieNdx = max (0, ieNdx);

		/*
		** If outside num elements set to last element.
		*/
		if (ieNdx > plgad->E.c - plgad->D.ceVMax) {
			ieNdx = plgad->E.c - plgad->D.ceVMax;
			ieNdx = max (0, ieNdx);
		}

		if (plgad->Egad.V.fOn) {
			SetFloorPositionEGAD (&plgad->Egad.V.egad,
				plgad->D.ceVExcess + 1, ieNdx);
		} else {
			LGAD_DrawList (plgad);
		}
	}

} /* SetLGADListPos */

/*********************************************************************
 *
 * SetLGADnodPos
 *
 * SYNOPSIS
 *		void SetLGADnodPos (LGAD *plgad, NOD *pnod, short fpos)
 *
 * PURPOSE
 *		Sets element 'pnod' to be displayed in plgad LGAD
 *		based on fpos.
 *
 * INPUT
 *		pnod	: pointer to node to element of list
 *		fpos	: position for element to be displayed
 *
 *		LGAD_POS_TOP	= display at top.
 *		LGAD_POS_MID	= display in middle.
 *		LGAD_POS_BOT	= display at bottom.
 *
 * EFFECTS
 *		Attempts to setup plgad LGAD to display element 'pnod' at requested
 *		position if it is possible to do so.
 *
 * RETURN VALUE
 *		None
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		SetLGADListPos
 *
*/
void SetLGADnodPos (LGAD *plgad, NodeType *pnod, short fpos)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetLGADnodPos";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	{
		NodeType	*pnodFind;
		short		 count = 0;
		pnodFind = Head (plgad->E.plst);

		while (!EndOfList (pnodFind)) {
			if (pnodFind == pnod) {
				break;
			}
			count++;
			pnodFind = Next (pnodFind);
		}

		SetLGADListPos (plgad, count, fpos);
	}
} /* SetLGADnodPos */

/*********************************************************************
 *
 * LGAD_HighLight
 *
 * SYNOPSIS
 *		short LGAD_HighLight (HitInfo *phi) 
 *
 * PURPOSE
 *		To highlight an entry or entries depending on the select mode
 *		(plgad->Sel.fExclusive).
 *
 * INPUT
 *
 *
 * AFFECTS
 *		Scrolls list if mouse goes off edge of display area.
 *
 * RETURN VALUE
 *		TRUE if first click was on a displayed entry.
 *		FALSE otherwise.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short LGAD_HighLight (HitInfo *phi)
{
	LGAD	*plgad = (LGAD *)phi->HitArea->UserData;
	short	ieDragEnd;
	short	fSelected;

	if (!plgad->E.c || !plgad->fDrawn) return TRUE; /* No list elements, so quit */

	if (phi->inpt == inptFocus) {
		NOD	*pnodDragEnd = plgad->Sel.pnodDragEnd;
		short ieDragEnd, ieTop = plgad->D.ieTop;
		short ieBotEx = ieBotExGet (plgad, ieTop);

		if (phi->fFocus & FOCUS_SPACE_ON) {
			pnodDragEnd->Priority |= LGAD_NOD_FOCUSED;
		} else {
			pnodDragEnd->Priority &= ~LGAD_NOD_FOCUSED;
		}
		if ((ieDragEnd = plgad->Sel.ieDragEnd) >= ieTop &&
			ieDragEnd < ieBotEx)
		{
			LGAD_DrawEntry (plgad, pnodDragEnd, ieDragEnd, FALSE);
		}

		if (plgad->Sel.pfnupdate) {
			/* Call user function */
			(plgad->Sel.pfnupdate)(plgad->Sel.pvUserData, LGAD_FOCUS_CHANGE);
		}
		return TRUE;
	}

	ieDragEnd = plgad->Sel.ieDragEnd;
	fSelected = LGADNodeIsSelected(plgad->Sel.pnodDragEnd);

	switch (phi->inpt) {
	case inptMouse: {
			UWORD Buttons = phi->Buttons;
			short ieTarget = ieTargetFromYOff (plgad, phi->Y);

			ScrollIfNeedTo (plgad, &ieTarget, phi->X);

			if (Buttons & MOUSE_FIRSTDOWN) { 
				StartHighLightDrag (plgad, ieTarget);
			} else { /* mouse held down */
				if (ieTarget != plgad->Sel.ieDragEnd) {
					HighLightDrag (plgad, ieTarget);
				}
			}
		}
		break;
	case inptCursor: {
			UWORD fCursor = phi->fCursor;
			short ieTarget = plgad->D.ieTop;
			short dxha = 0;

			if (fCursor & CURSOR_LFT) {
				dxha = -1;
			} else if (fCursor & CURSOR_RGT) {	
				dxha = SHRT_MAX;	/* huge number to make it scroll right */
			} else { /* Is CURSOR_UP or CURSOR_DN */
				ieTarget = plgad->Sel.ieDragEnd;
				if (plgad->Sel.pnodDragEnd) {	
					ieTarget += ((fCursor & CURSOR_UP) ? -1 : +1);
				}

			}

			ScrollIfNeedTo (plgad, &ieTarget, dxha);

			if (fCursor & (CURSOR_UP | CURSOR_DN)) {
				if (!plgad->Sel.fExclusive && plgad->Sel.pnodDragEnd && 
					phi->Shiftflags & KEY_SHIFT) 
				{
					if (!plgad->Sel.fDragging) {
						StartHighLightDrag (plgad, plgad->Sel.ieDragEnd);
						plgad->Sel.fDragging = TRUE;
					} else {
						HighLightDrag (plgad, ieTarget);
					}
				} else {
					if (!plgad->Sel.fExclusive)	{ 
						/* Make it so only focus changes not highlight */
						NOD	*pnod = plgad->Sel.pnodDragEnd;
						
						/* Find new DragEnd */
						if (ieTarget > plgad->Sel.ieDragEnd) {
							pnod = Next (pnod);
						} else if (ieTarget < plgad->Sel.ieDragEnd) {
							pnod = Prev (pnod);
						}
						pnod->Priority ^= LGAD_NOD_SELECTED;
						plgad->Sel.fDragging = FALSE;
					}
					StartHighLightDrag (plgad, ieTarget);

				}
			}
		}
		break;
	case inptHitKey:
		StartHighLightDrag (plgad, plgad->Sel.ieDragEnd);
		plgad->Sel.fDragging = !plgad->Sel.fExclusive;
		break;
	}

	{ 
		UWORD fWhy = 0;

		if (ieDragEnd != plgad->Sel.ieDragEnd) {
			fWhy |= LGAD_FOCUS_CHANGE;
			if (plgad->Sel.fExclusive) {
				fWhy |= LGAD_HIGHLIGHT_CHANGE;
			} 
		} else if (fSelected != LGADNodeIsSelected(plgad->Sel.pnodDragEnd)) {
			fWhy |= LGAD_HIGHLIGHT_CHANGE;
		}

		if ((phi->Buttons & MOUSE_DOUBLE) && !(fWhy & LGAD_FOCUS_CHANGE)) {
			fWhy |= LGAD_DOUBLE_CLICK;
		}

		if (fWhy && plgad->Sel.pfnupdate) {
			/* Call user function */
			(plgad->Sel.pfnupdate)(plgad->Sel.pvUserData, fWhy);
		}
	 }

	return TRUE;
} /* LGAD_HighLight */

/*********************************************************************
 *
 * DetermineElevatorStatus
 *
 * SYNOPSIS
 *		static void DetermineElevatorStatus (LGAD *plgad)
 *
 * PURPOSE
 *		To determine whether or not elevator bars will be present and
 *		to set up the hit areas appropriately.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static void DetermineElevatorStatus (LGAD *plgad)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DetermineElevatorStatus";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	HitArea		*pha;
	short		x = 0; /*plgad->x */
	short		y = 0; /*plgad->y */
	short		cx = plgad->cx;
	short		cy = plgad->cy;
	short		cpelButtons = plgad->Egad.cpel;
	short		fVEnabled = plgad->Egad.V.fEnabled;
	short		fHEnabled = plgad->Egad.H.fEnabled;
	short		fVOn, fHOn;
	short		fVExcess, fHExcess;
	UWORD		Flags = plgad->Flags;

	do {
		fVOn = fVEnabled && plgad->D.ceVExcess != 0;
		fHOn = fHEnabled && plgad->D.cdxHPanExcess != 0;

		plgad->D.cx = cx - fVOn * (cpelButtons - 1) - 2;
		plgad->D.cy = cy - fHOn * (cpelButtons - 1) - 2;

		plgad->D.ceVMax = plgad->D.cy / plgad->E.cy;

		if ((fVExcess = plgad->E.c > plgad->D.ceVMax) != 0) {
			plgad->D.ceVExcess = plgad->E.c - plgad->D.ceVMax;
		}

		if ((fHExcess = plgad->E.cxMax > plgad->D.cx) != 0) {
			plgad->D.cxeHExcess = plgad->E.cxMax - plgad->D.cx;
			plgad->D.cdxHPanExcess = 
				(plgad->D.cxeHExcess + (plgad->D.dxHPan -1)) / 
				plgad->D.dxHPan;
		}
	} while ((fVEnabled && fVOn != fVExcess) || 
		(fHEnabled && fHOn != fHExcess));


	{
		short		cpelBot = (fHOn) ? cpelButtons : 0;
		short		cpelRgt = (fVOn) ? cpelButtons : 0;
		short		cpelCorner = ((fHOn && fVOn) ? cpelButtons - 1 : 0);

		/* Set up hit areas */
		pha = plgad->rgha;

		/* List area */
		pha->X = x + 1;
		pha->Y = y + 1;
		pha->Width = cx - 1 - cpelRgt - (!cpelRgt);
		pha->Height = cy - 1 - cpelBot - (!cpelBot);
		pha->Flags = HTA_SPACE_FOCUS | HTA_CURSOR |  HTA_DOWN | HTA_REPEAT | HTA_NO_UP;
		pha->Data = LGAD_HighLight;
		pha->UserData = plgad;
		++pha;

		/* Vertical elevator */
		if (fVOn) {
			short xE = (x + cx - cpelButtons);

			pha->X = xE;
			pha->Y = y;
 			pha->Width = cpelButtons;
			pha->Height = cy - cpelCorner;
			pha->Flags = HTA_SUBAREAS;
			pha->Data = phaEGAD (&plgad->Egad.V.egad);
			++pha;

			plgad->Egad.V.fOn = InitEGAD (
				&plgad->Egad.V.egad, VERTICAL, 
				cy - cpelCorner, cpelButtons, 
				plgad->D.ceVExcess + 1, 
				((Flags & LGAD_V_REALTIME_SLIDER)? EGAD_REALTIME_SLIDER : 0),
				plgad->Egad.V.cDelayUntilRepeat, 
				LGAD_UpdateV, plgad); 
		}

#if USE_H_ELEVATORS
		/* Horizontal elevator */
		if (fHOn) {
			short yE = (y + cy - cpelButtons);

			pha->X = x;
			pha->Y = yE;
			pha->Width = cx - cpelCorner;
			pha->Height = cpelButtons;
			pha->Flags = HTA_SUBAREAS;
			pha->Data = phaEGAD (&plgad->Egad.H.egad);
			++pha;

			plgad->Egad.H.fOn = InitEGAD (
				&plgad->Egad.H.egad, HORIZONTAL, 
				cx - cpelCorner, cpelButtons, 
				plgad->D.cdxHPanExcess + 1, 
				((Flags & LGAD_H_REALTIME_SLIDER)? EGAD_REALTIME_SLIDER : 0), 
				plgad->Egad.H.cDelayUntilRepeat, 
				LGAD_UpdateH, plgad); 
		}
#endif
	}
	/* end area */

	MAKE_END_HITAREA (pha);

}
} /* DetermineElevatorStatus */

static void LGAD_DisplayText (
	NOD *pnod,
	short x, short y
)	
{
	BeforeGraphics ();
	if (pnod) {
		DrawString (GetActiveWindow()->Font, x + 1, y + 1, NodeName(pnod));
	}
	AfterGraphics ();
}

PFNDSPNOD	DisplayTextLGAD = LGAD_DisplayText;

short LGAD_DrawList (LGAD *plgad)
{
	NOD		*pnod;
	short	fInited = FALSE;
	short	ie = 0;
	short	ce = plgad->D.ieTop;
	short	yDraw;

	if (!plgad->fDrawn) return FALSE;

	pnod = Head (plgad->E.plst);
	while (ce) {
		pnod = Next (pnod); ++ie;
		--ce;
	}

	while (ce < plgad->D.ceVMax && !IsEOList (pnod)) {
		LGAD_DrawEntry (plgad, pnod, ie, fInited++);
		pnod = Next (pnod); ++ie;
		++ce;
	}

	/* Erase to bottom of display area */
	yDraw = plgad->D.y + (ie - plgad->D.ieTop) * plgad->E.cy;

	BeforeGraphics ();
	SetPenColor (GetActiveWindow()->White);
	if (yDraw <= plgad->D.yMaxin) {
		DrawRect (plgad->D.x, yDraw, 
			plgad->D.cx, plgad->D.yMaxin - yDraw + 1);
	}
	AfterGraphics ();

	return TRUE;

} /* LGAD_DrawList */


void LGAD_UpdateV (void *data) {
	LGAD *plgad = (LGAD *)data;
	plgad->D.ieTop = iFloorCrntEGAD(&plgad->Egad.V.egad);
	LGAD_DrawList (plgad);
		
}

#if USE_H_ELEVATORS
void LGAD_UpdateH (void *data) {
	LGAD *plgad = (LGAD *)data;

	plgad->D.ixLeft = iFloorCrntEGAD(&plgad->Egad.H.egad);
	LGAD_DrawList (plgad);

}
#endif

/*********************************************************************
 *
 * DrawLGAD
 *
 * SYNOPSIS
 *		void DrawLGAD (LGAD *plgad, short x, short y)
 *
 * PURPOSE
 *		To draw the list gadget.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DrawLGAD (LGAD *plgad, short x, short y)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawLGAD";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short cx = plgad->cx;
	short cy = plgad->cy;
	short cpelBtn = plgad->Egad.cpel;

	plgad->D.x = (plgad->x = x) + 1;
	plgad->D.y = (plgad->y = y) + 1;
	plgad->D.xMaxin = (plgad->xMaxin = x + cx - 1) - 
		plgad->Egad.V.fOn * (cpelBtn - 1) - 1;
	plgad->D.yMaxin = (plgad->yMaxin = y + cy - 1) -
		plgad->Egad.H.fOn * (cpelBtn - 1) - 1;

	BeforeGraphics ();
	/* background */
	SetPenColor (GetActiveWindow()->White);
	DrawRect (x+1, y+1, cx-2, cy-2);

	/* outline */
	SetPenColor (GetActiveWindow()->Black);
	DrawBox (x, y, cx, cy);

	plgad->fDrawn = TRUE;

	LGAD_DrawList (plgad); 

	{ 
		short cegad = 0;
		/* Draw vertical elevator if any */
		if (plgad->Egad.V.fOn) {
			DrawEGAD (&plgad->Egad.V.egad, 
				(x + cx - cpelBtn), y);
			++cegad;
		} 

	#if USE_H_ELEVATORS
		/* Draw horizontal elevator if any */
		if (plgad->Egad.H.fOn) {
			DrawEGAD (&plgad->Egad.H.egad,
				x, (y + cy - cpelBtn));
			++cegad;
		}  
		if (cegad == 2) {	/* Draw corner section */
			short cpelCorner = plgad->Egad.cpel - 2;
			SetPenColor (GetActiveWindow()->LightGray);
			DrawRect (plgad->xMaxin - cpelCorner,	
				plgad->yMaxin - cpelCorner,
				cpelCorner, cpelCorner);
		}
	#endif
	}

	
	AfterGraphics ();

}
} /* DrawLGAD */

/*********************************************************************
 *
 * InitLGAD
 *
 * SYNOPSIS
 *		short InitLGAD (
 *			LGAD	*plgad,
 *			short	 cx, short cy,
 *			LST		*plst,
 *			short	 Flags,
 *			short	 cpelButtons,
 *			short	 cDelayUntilRepeatH,
 *			short	 cDelayUntilRepeatV,
 *			short	cxElementMax,
 *			short	 cyElement,
 *			short	dxHPan,
 *			PFNDSPNOD	 pfndspnod,
 *			PFNUPDATE	 pfnupdate,
 *			void		*pvUserData
 *		)
 *
 * PURPOSE
 *		To set up a list gadget structure.
 *
 * INPUT
 *		plgad: 	pointer to LGAD struct (must be non-NULL)
 *		cpelButtons: x and y dimensions of area to use for elevator and buttons.
 *		cDelayUntilRepeatH: Delay until horizontal arrow buttons repeat.
 *		cDelayUntilRepeatV: Delay until vertical arrow buttons repeat.
 *		cxElementMax:	Maximum width of any list element. If this is bigger
 *			than cx and you have set the LGAD_H_ELEVATOR flag then your list
 *			will have a horizontal elevator to pan the list left and right.
 *		cyElement:	Height of a single list element.
 *		dxHPan: Amount to move left or right with each horizontal 
 *			elevator notch.
 *		pfndspnod:	Pointer to function that displays an element.	
 *		pfnupdate:	Pointer to function to call when something changes
 *					in the list (highlighting or focus) or when user double
 *					clicks on entry.
 *		pvUserDate:	Pointer to data to pass as first parameter to pfnupdate.
 *	
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		TRUE if succesful.
 *		FALSE if size is too small.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short InitLGAD (
	LGAD	*plgad,
	short	 cx, short cy,
	LST		*plst,
	UWORD	 Flags,
	short	 cpelButtons,	
	short	 cDelayUntilRepeatH,
	short	 cDelayUntilRepeatV,
	short	 cxElementMax,
	short	 cyElement,
	short	 dxHPan,
	PFNDSPNOD	 pfndspnod,
	PFNUPDATE	 pfnupdate,
	void		*pvUserData
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "InitLGAD";
#endif
#if REQUIRE
	if (!pfndspnod) {
		SetGlobalErr (ERR_REQUIREMENTS);
		GEprintf ("NULL passed for display function to InitLGAD\n");
		return FALSE;
	}
#endif
/*-------------------------------------------------------------------------*/
{
	short		fVE = Flags & LGAD_V_ELEVATOR;
	short		fHE = Flags & LGAD_H_ELEVATOR;
	short		cpelBot = (fVE) ? cpelButtons : 0;
	short		cpelRgt = (fHE) ? cpelButtons : 0;
	short		cE, cxD, cyD, ceDVMax;

	/* The area must be big enough for buttons and have room for arrow icon */
	if (
		((fVE || fHE) && (cpelButtons < cpelEGADButtonMin)) ||
		((cx - 2 - cpelRgt) < cyElement) ||
		((cy - 2 - cpelBot) < cyElement)
	)
	{	
		return FALSE;
	}

	memset (plgad, 0, sizeof (LGAD));

	plgad->Sel.pfnupdate = pfnupdate;
	plgad->Sel.pvUserData = pvUserData;
	
	plgad->Flags = Flags;
	plgad->D.cx = cxD = (plgad->cx = cx) - 2;
	plgad->D.cy = cyD = (plgad->cy = cy) - 2;

	plgad->E.pfndspnod = pfndspnod;
	plgad->E.plst = plst;
	plgad->E.c = cE = (short) OpOnList (plst, NULL);
	plgad->E.cxMax = cxElementMax;
	plgad->E.cy = cyElement;

	plgad->D.ceVMax = ceDVMax = cyD / cyElement;
	if (cE > ceDVMax) {
		plgad->D.ceVExcess = cE - ceDVMax;
	}
	if (cxElementMax > cxD) {
		plgad->D.cxeHExcess = cxElementMax - cxD;
	}
	plgad->D.dxHPan = dxHPan;
	plgad->D.cdxHPanExcess = (plgad->D.cxeHExcess + (dxHPan -1)) / dxHPan;

	plgad->Sel.fExclusive = Flags & LGAD_EXCLUSIVE;
	plgad->Sel.pnodDragEnd = Head (plst);

	plgad->Egad.cpel = cpelButtons;
#if USE_H_ELEVATORS
	plgad->Egad.H.fEnabled = fHE;
	plgad->Egad.H.cDelayUntilRepeat = cDelayUntilRepeatH;
#endif
	plgad->Egad.V.fEnabled = fVE;
	plgad->Egad.V.cDelayUntilRepeat = cDelayUntilRepeatV;

	DetermineElevatorStatus(plgad);

	return TRUE;
}
} /* InitLGAD */

/*********************************************************************
 *
 * SetLGADList
 *
 * SYNOPSIS
 *		void SetLGADList (
 *			LGAD *plgad,
 *			LST  *plst,
 *			short cxElementMax
 *		)
 *
 * PURPOSE
 *		To update the element list of a list gadget.
 *
 * INPUT
 *		plgad: List Gadget pointer to update.
 *		plst:	New (changed) element list pointer.
 *		cxElementMax:	Widest element in list. May pass 0 or less to leave
 *			setting unchanged from initialization value.
 *
 * AFFECTS
 *		Updates paremeters in LGAD struct. Redraws List Gadget.
 *		If cxElementMax changes then the horizontal elvator gadget may
 *		appear or disappear (if enabled).
 *
 * RETURN VALUE
 *		NONE.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetLGADList (
	LGAD *plgad,
	LST  *plst,
	short cxElementMax
)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetLGADList";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
{
	short iFloorH = SHRT_MAX, iFloorV = SHRT_MAX;
	short cFloorH, cFloorV;
	short x = plgad->x;
	short y = plgad->y;

	if (plgad->Egad.H.fOn) {
		iFloorH = iFloorCrntEGAD (&plgad->Egad.H.egad);

	}
	if (plgad->Egad.V.fOn) {
		iFloorV = iFloorCrntEGAD (&plgad->Egad.V.egad);
	}
	InitLGAD (	
		plgad,
		plgad->cx, plgad->cy,
		plst,
		plgad->Flags,
		plgad->Egad.cpel,
		plgad->Egad.H.cDelayUntilRepeat,
		plgad->Egad.V.cDelayUntilRepeat,
		cxElementMax, /*plgad->E.cxMax,*/
		plgad->E.cy,
		plgad->D.dxHPan,
		plgad->E.pfndspnod,
		plgad->Sel.pfnupdate,
		plgad->Sel.pvUserData
	);

	if (plgad->Egad.V.fOn) {
		cFloorV = FloorsMaxEGAD (&plgad->Egad.V.egad);
		if (iFloorV >= cFloorV) iFloorV = cFloorV -1;
		SetFloorPositionEGAD (&plgad->Egad.V.egad,
			cFloorV,
			iFloorV
		);
	}

	if (plgad->Egad.H.fOn) {
		cFloorH = FloorsMaxEGAD (&plgad->Egad.H.egad);
		if (iFloorH >= cFloorH) iFloorH = cFloorH -1;
		SetFloorPositionEGAD (&plgad->Egad.H.egad,
			cFloorH,
			iFloorH
		);

	}


	DrawLGAD (plgad, x, y);

}} /* SetLGADList */

/*********************************************************************
 *
 * pnodLGADFindSelected
 *
 * SYNOPSIS
 *		NOD *pnodLGADFindSelected (NOD *pnod)
 *
 * PURPOSE
 *		Return first selected node starting with this node.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		Pointer to selected node (possibly node passed in).
 *		NULL if no selected nodes (after and including this node).
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
NOD *pnodLGADFindSelected (NOD *pnod)
{
	while (!IsEOList(pnod)) {
		if (LGADNodeIsSelected (pnod)) {
			return pnod;
		}
		pnod = Next (pnod);
	}
	return NULL;

} /* pnodLGADFindSelected */

/*********************************************************************
 *
 * pnodLGADFocused
 *
 * SYNOPSIS
 *		NOD *pnodLGADFocused (LGAD *plgad)
 *
 * PURPOSE
 *		Find node which is currently focused.
 *
 * INPUT
 *		plgad:	Pointer to list gadget.
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *		Pointer to node that is focused.
 *		NULL if no node is focused.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
NOD *pnodLGADFocused (LGAD *plgad)
{
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "pnodLGADFocused";
#endif
#if REQUIRE
	if (!plgad) {
		SetGlobalErr (ERR_REQUIREMENTS);
		GEprintf ("NULL passed to pnodLGADFocused()\n");
		return NULL;
	}
#endif
/*-------------------------------------------------------------------------*/
{
	NOD *pnod = Head (plgad->E.plst);

	while (!IsEOList(pnod)) {
		if (LGADNodeIsFocused (pnod)) {
			return pnod;
		}
		pnod = Next (pnod);
	}
	return NULL;

}} /* pnodLGADFocused */

