/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * APalette.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 01/28/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 * DESCRIPTION
 *		Routines for setting the Amiga palette.
 *
 * HISTORY
 *	06/01/91 Saturday (RGM) - Added support for SAS/Lattice C 5.10a.
 *
*/
#include <echidna/platform.h>
#include "switches.h"

#include <echidna/etypes.h>
#include <intuition/intuition.h>
#include <echidna/graphics.h>
#if AZTEC_C
#include <functions.h>
#elif LATTICE
#include <proto/graphics.h>>
#endif
#include "easydbuf.h"

/******************************** CONSTANTS *******************************/
#define MAX_PALETTE		(32)

/*************************** External Variables ***************************/
extern struct Screen *AmigaScreen;

/*********************************************************************
 *
 * LowSetPalette
 *
 * SYNOPSIS
 *		void	LowSetPalette (
 *			short		regnum,
 *			short		numcolors,
 *			UBYTE	*colortable
 *		)
 *
 * USAGE
 *
 *
 * PURPOSE
 *		Set palette.
 *
 * INPUTS
 *
 *
 * RESULTS
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void LowSetPalette (
	ColorType	*colortable
) {
	short numcolors;

	for (numcolors = 0; numcolors < MAX_PALETTE; numcolors++) {
		SetRGB (numcolors, colortable->Color);
		colortable++;
	}
} /* LowSetPalette */

/*********************************************************************
 *
 * SetPalette
 *
 * SYNOPSIS
 *		void	SetPalette (
 *			short		regnum,
 *			short		numcolors,
 *			PaletteType	*palette
 *		)
 *
 * USAGE
 *
 *
 * PURPOSE
 *		Set palette.
 *
 * INPUTS
 *
 *
 * RESULTS
 *
 *
 * BUGS
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetPalette (
	PaletteType	*palette
) {
	CurrentPalette = palette;
	LowSetPalette (palette->Data.RGB);
} /* SetPalette */

/*********************************************************************
 *
 * FadePalette
 *
 * SYNOPSIS
 *		void	FadePalette (
 *			short		 regnum,
 *			short		 numcolors,
 *			ColorType	*colortable,
 *			short		 frames,
 *			short		 in
 *		)
 *
 * PURPOSE
 *		To fade a palette in or out.
 *
 * INPUT
 *		in:		True means fade in. FALSE means fade out (to black).
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void	FadePalette (
	PaletteType	*palette,
	short		 frames,
	short 		 in
) {
	short 		 numcolors  = palette->NumColors;
	ColorType	*colortable = palette->Data.RGB;
	ColorType	 palette2[MAX_PALETTE];
	ColorType	*ct;
	UWORD		 temp[MAX_PALETTE * 3];
	UWORD		*tp;
	 WORD		 delta[MAX_PALETTE * 3];
	 WORD		*dl;
	short		 i;
	short		 j;
	ColorType	*ctable;

	CurrentPalette = palette;

	/* Compute Increment */
	/* Fill in temp palette */
	ctable = colortable;
	dl     = delta;
	tp     = temp;
	for (i = 0; i < numcolors; i++) {
		*tp++ = (in) ? 0 : (((UWORD)((ctable->Color & 0x0F00) >> 8)) * 256);
		*dl   = (((UWORD)((ctable->Color & 0x0F00) >> 8)) * 256) / frames;
		*dl  += (*delta && in);
		*dl  *= (in ? 1 : -1);
		*dl++;
		*tp++ = (in) ? 0 : (((UWORD)((ctable->Color & 0x00F0) >> 4)) * 256);
		*dl   = (((UWORD)((ctable->Color & 0x00F0) >> 4)) * 256) / frames;
		*dl  += (*delta && in);
		*dl  *= (in ? 1 : -1);
		*dl++;
		*tp++ = (in) ? 0 : (((UWORD)((ctable->Color & 0x000F) >> 0)) * 256);
		*dl   = (((UWORD)((ctable->Color & 0x000F) >> 0)) * 256) / frames;
		*dl  += (*delta && in);
		*dl  *= (in ? 1 : -1);
		*dl++;
		ctable++;
	}

	ResetVTimer (1);
	for (j = 0; j < frames; j++) {

		/* Inc or Dec temp */
		dl     = delta;
		tp     = temp;
		for (i = 0; i < numcolors * 3; i++) {
			*tp += *dl;
			dl++;
			tp++;
		}
		
	 	/* copy temp to palette */
		dl     = delta;
		tp     = temp;
		ct     = palette2;
		for (i = 0; i < numcolors; i++) {
			ct->Color =  (tp[0] & 0x0F00) |
						((tp[1] & 0x0F00) >> 4) |
						((tp[2] & 0x0F00) >> 8);
			ct++;
			tp += 3;
		}

		LowSetPalette (palette2);

		WaitVTimer ();
	}

	if (AmigaScreen) {
		ct = palette2;
		for (i = 0; i < numcolors; i++) {
			SetRGB4 (&AmigaScreen->ViewPort, i,
				(ct->Color & 0x0F00) >> 8,
				(ct->Color & 0x00F0) >> 4,
				(ct->Color & 0x000F));
			ct++;
		}
	}

} /* FadePalette */

