/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * EDBUF.c
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 03/05/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		EGA specific DBUF routines.
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <echidna/dbuf.h>
#include "dbufarea.h"

/******************** E X T E R N A L   R O U T I N E S *******************/
extern volatile WORD far		 EGA_RetraceBegun;

/**************************** C O N S T A N T S ***************************/
#define GHEADERSIZE	(GSHAPESIZE + EGASDATASIZE)

/*********************************************************************
 *
 * EGA_OpenDBufGraphics
 *
 * SYNOPSIS
 *		short EGA_OpenDBufGraphics (short mode, size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short EGA_OpenDBufGraphics (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EGA_OpenDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	*WorkBuffer = 0xA200;
	*BackBuffer = 0xA400;

	return TRUE;

} /* EGA_OpenDBufGraphics */

/*********************************************************************
 *
 * EGA_CloseDBufGraphics
 *
 * SYNOPSIS
 *		void EGA_CloseDBufGraphics (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_CloseDBufGraphics (void)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "EGA_CloseDBufGraphics";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

} /* EGA_CloseDBufGraphics */

/*********************************************************************
 *
 * EGA_DBufPeepRect
 *
 * SYNOPSIS
 *		void EGA_DBufPeepRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufPeepRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= PEEP_FLAG_MASK; /* Erase next frame */    

	DrawRect (x, y, width, height);

} /* EGA_DBufPeepRect */

/*********************************************************************
 *
 * EGA_DBufPeepShape
 *
 * SYNOPSIS
 *		void EGA_DBufPeepShape (
 *			GShape	*shapeptr,
 *			short x,
 *			short y
 *		) 
 *
 * PURPOSE
 *		To record the rectangular area where the shape will be drawn in the
 *		shape area draw list.  
 *		Then to draw the shape via call to 'DrawGShape'. This shape will
 *		stay on the display screen for only one frame.
 *
 * INPUT
 *		shapeptr: 	Far pointer to shape data.
 *		x, y:		Screen coordinates for drawing shape.	
 *		SAdrawPtr:	(GLOBAL) Pointer to first free record in Shape Area Draw
 *					List.  
 *  
 * AFFECTS
 *		SAdrawPtr:	(GLOBAL) Incremented to point to next free record.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufPeepShape (
	GShape	*shapeptr,
	short x,
	short y
) 
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/
	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)&shapeptr->Data.EGA.XOffset;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 3;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= PEEP_FLAG_MASK;		/* Erase next frame */

	DrawGShape (shapeptr, x, y);

} /* EGA_DBufPeepShape */

/*********************************************************************
 *
 * EGA_DBufStampShape
 *
 * SYNOPSIS
 *		void EGA_DBufStampShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a shape onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufStampShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	UWORD	*saptr		= (UWORD *)SAdrawPtr++;
	UWORD	*dataptr	= (UWORD *)&shapeptr->Data.EGA.XOffset;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x + (*dataptr++);		/* Area->x = x + shape->xoffset */
	*(saptr++)	= (*dataptr++) << 3;	/* Area->width = shape->width   */
	*(saptr++) 	= y + *dataptr++;		/* Area->y = y + shape->yoffset */
	*(saptr++) 	= *dataptr;				/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;		/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShape (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShape (shapeptr, x, y);	/* in work */

} /* EGA_DBufStampShape */

/*********************************************************************
 *
 * EGA_DBufStampShapeNoTrans
 *
 * SYNOPSIS
 *		void EGA_DBufStampShapeNoTrans (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *                   without transparency.
 *		To draw a shape ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufStampShapeNoTrans (
	GShape	*shapeptr,
	short	 x,
	short	 y
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= shapeptr->Width;	/* Area->width = shape->Width   */
	*(saptr++) 	= y;
	*(saptr++) 	= shapeptr->Height;	/* Area->height = shape->Height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in back */
	
	SetWorkPage (WorkBuffer);
	DrawGShapeNoTrans (shapeptr, x, y);	/* in work */

} /* EGA_DBufStampShapeNoTrans */

/*********************************************************************
 *
 * EGA_DBufStampClearShape
 *
 * SYNOPSIS
 *		void EGA_DBufStampClearShape (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufStampClearShape (
	GShape	*shapeptr,
	short	 x,
	short	 y
) {
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	
	/* dataptr:			Data Pointer. Points to the succesive fields of the
	 *					shape data header: <x offset>, <width>,
	 *					<y offset>, <height>. 
	*/

	WORD	*saptr 		= (WORD *)SAdrawPtr++;
	WORD	width;
	WORD	height;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= (width=shapeptr->Width); /* Area->width = shape->width */
	*(saptr++) 	= y;
	*(saptr++) 	= (height=shapeptr->Height);/* Area->height = shape->height */
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	
	SetPaperColor (255);
#if !USE_SHAPE_TPAT
	EvenPatternTbl[255] = shapeptr->TColor;
	OddPatternTbl[255]  = shapeptr->TColor >> 8;
#else
	EvenPatternTbl[255] = shapeptr->Data.EGA.TPatEven;
	OddPatternTbl[255]  = shapeptr->Data.EGA.TPatOdd;
#endif
	DrawRect (x, y, width, height);
	
	SetWorkPage (WorkBuffer);
	DrawRect (x, y, width, height);

} /* EGA_DBufStampClearShape */

/*********************************************************************
 *
 * EGA_DBufStampRect
 *
 * SYNOPSIS
 *		void EGA_DBufStampRect (
 *			GShape	*shapeptr,
 *			short	 x,
 *			short	 y,
 *			short	 width,
 *			short	 height,
 *		)
 *
 * PURPOSE
 *		To draw a rect in shape's size ^ onto the background buffer, and work buffer and
 *		and enter the area into the shape area list so it will be copied
 *		to the display buffer on the next call to DBufFlipScreen().
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void EGA_DBufStampRect (
	short	 x,
	short	 y,
	short	 width,
	short	 height
)
{
	/* saptr:			Points to the succesive fields 
	 *					of the current 'AreaRecType' struct pointed to
	 *					by 'SAdrawPtr'. The fields are: 'x', 'width',  
	 *					'y', 'height' and 'backflag'.
	*/	

	WORD	*saptr 		= (WORD *)SAdrawPtr++;

	/** Record in draw list the area rectangle where shape will be drawn **/
	*(saptr++) 	= x;
	*(saptr++)	= width;
	*(saptr++) 	= y;
	*(saptr++) 	= height;
  	*saptr		= STAMP_FLAG_MASK;	/* Do not Erase next frame */    

	SetWorkPage (BackBuffer);
	DrawRect (x, y, width, height);

	SetWorkPage (WorkBuffer);
	DrawRect (x, y, width, height);

} /* EGA_DBufStampRect */

/*********************************************************************
 *
 * RestorePeepAreas
 *
 * SYNOPSIS
 *		void RestorePeepAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular peep areas in 'arealist' from the Back screen
 *		to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *		
 * AFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/
static void RestorePeepAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
			if (arealist->flags & PEEP_FLAG_MASK) {
				Restore (arealist->x, arealist->y, 
							arealist->width, al_height);		
			}
	}

} /* RestorePeepAreas */
	
/*********************************************************************
 *
 * RestoreStampAreas
 *
 * SYNOPSIS
 *		void RestoreStampAreas (
 *			AreaRecType	*arealist
 *		)
 *
 * PURPOSE
 *		To copy the rectangular stamp areas in 'arealist' from the Background
 *		screen to the Work screen via call to 'Restore'.  
 *
 * INPUT
 *		arealist:	Array of AreaRecType structures with end of list marked
 *					by entry with height field of 0.
 *
 * AFFECTS
 *		Copies areas from background buffer to work buffer. 
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		Restore()
 *
*/
static void RestoreStampAreas (
	AreaRecType	*arealist
)
{
	WORD al_height;

	for ( ; (al_height = arealist->height) != 0; arealist++) {
		if (arealist->flags & STAMP_FLAG_MASK) {
			Restore (arealist->x, arealist->y, 
						arealist->width, al_height);		
		}
	}

} /* RestoreStampAreas */

/*********************************************************************
 *
 * EGA_DBufFlipScreen
 *
 * SYNOPSIS
 *		void EGA_DBufFlipScreen (void)
 *
 * PURPOSE
 *		To ShapeFlip double buffering screens and update the display.
 *
 * INPUT
 *		(GLOBALS)
 *		SAList1, SAList2:	Shape Area lists (arrays).
 *		SAeraseList:		List from which to restore areas of Display  
 *							screen where old shapes are.
 *		SAdrawList:			List from which to update areas of Display
 *							screen where new shapes are. List from which
 *							to restore areas of Work screen where new shapes
 *							were drawn.
 *		SAerasePtr:			Pointer to end of SAerasList.
 *		SAdrawPtr:			Pointer to end of SAdrawList.
 *
 * AFFECTS
 *		(GLOBALS)
 *		'SAeraseList' and 'SAdrawList' are swapped.
 *		'SAerasePtr' is set to final value of 'SAdrawPtr'.
 *		'SAdrawPtr' is set to start of new 'SAdrawList'.
 *
 * RETURN VALUE
 *		None.
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *		DisplayWorkAreas (), RestoreBackAreas (),
 *		SWAP_RESET_SALISTS ()
 *
*/
void EGA_DBufFlipScreen (void)
{
	UWORD	temp;

	/* Mark end of area lists */
		SAerasePtr->height = 0;
		SAdrawPtr->height = 0; 

	/* Flip Display and Work Buffers */
		temp           = *WorkBuffer;
		*WorkBuffer    = *DisplayBuffer;
		*DisplayBuffer = temp;
		SetWorkPage (WorkBuffer);

	/* Display new display buffer */
		outport (0x03D4, (temp << 4) | 0x0C);

	/* Wait for Display to flip */
		EGA_RetraceBegun = FALSE;
		while (!EGA_RetraceBegun) {}

	/* Copy Old Shape areas from Work to Display */
		RestoreStampAreas (SAdrawList);

	/* Copy New Shape areas from Work to Display */
		RestorePeepAreas (SAeraseList);

	/* Reset shape save areas */
		SWAP_RESET_SALISTS ();
} /* EGA_DBufFlipScreen */

typedef void (far *RoutinePtr)();

static RoutinePtr EGADBufRoutineTable[] = {
	(RoutinePtr)EGA_OpenDBufGraphics,
	(RoutinePtr)EGA_CloseDBufGraphics,
	(RoutinePtr)EGA_DBufPeepShape,
	(RoutinePtr)EGA_DBufStampShape,
	(RoutinePtr)EGA_DBufStampShapeNoTrans,
	(RoutinePtr)EGA_DBufStampClearShape, 
	(RoutinePtr)EGA_DBufStampRect,
	(RoutinePtr)EGA_DBufPeepRect,
	(RoutinePtr)EGA_DBufFlipScreen,
};


/*********************************************************************
 *
 * GetEGADBufRoutineTable
 *
 * SYNOPSIS
 *		RoutinePtr *GetEGADBufRoutineTable (void)
 *
 * PURPOSE
 *		Returns pointer to EGADBufRoutineTable.  Calling this
 *		routine should force loading if EGA_DBufRoutines as an
 *		overlay.
 *
 * INPUT
 *
 *
 * AFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
RoutinePtr *GetEGADBufRoutineTable (void)
{
	return EGADBufRoutineTable;

} /* GetEGADBufRoutineTable */

