;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** ERECT.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 03/05/90
;**   MODIFIED : 01/21/92
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	Routine to draw a rectangle in current color on EGA screen.
;**
;**
;** HISTORY
;**
;**

	IDEAL

;=====================================================================
;**
;** SmallRect
;**
;** PURPOSE
;**	Draw Even or Odd lines of a rectangle
;**
;** INPUT
;**	ES:DI	First Byte of rectangle
;**	DL	Mask for start of line
;**	DH	Mask for end of line
;**	SI	Width in bytes
;**	AL	Even pattern for lines being drawn
;**	AH	Odd pattern for lines being drawn
;**	Modulo	Modulo for rect
;**	Height	Height in lines (half?)
;**	Odd	Starting line odd = 1, even = 0
;**
;** 	NOTE: DS must = CS
;**
;** REGISTER USED
;**	ALL of them
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
Modulo	dw	0
RHeight	dw	0
Line	db	0
Odd	db	0
CEP	dw	0
COP	dw	0
ELEP	dw	0
ELOP	dw	0
OLEP	dw	0
OLOP	dw	0
SMask	db	0
EMask	db	0

SMasks	DB	0FFh
	DB	07Fh
	DB	03Fh
	DB	01Fh
	DB	00Fh
	DB	007h
	DB	003h
	DB	001h
EMasks	DB	080h
	DB	0C0h
	DB	0E0h
	DB	0F0h
	DB	0F8h
	DB	0FCh
	DB	0FEh
	DB	0FFh

	MPROC SmallRect NEAR

	cld		; Set Store Direction.

	push	ax	; Save/Separate pattern
	mov	bl,00h
	mov	cl,4
	shr	ax,cl
	and	ax,0F0Fh
	mov	bh,al
	mov	[ELEP],bx
	mov	bh,ah
	mov	[OLEP],bx
	pop	ax
	and	ax,0F0Fh
	mov	bh,al
	mov	[ELOP],bx
	mov	bh,ah
	mov	[OLOP],bx

	mov	[SMask],dl	; Save Start Mask
	mov	[EMask],dh	; Save End Mask

	mov	dx,03C4h	; write to all planes
	mov	ax,0F02h
	out	dx,ax

	mov	dx,03CEh
	mov	ax,0005h	; Select EGA RMD 0/WMD 0
	out	dx,ax

	mov	ax,0F01h	; Set Enable Set/Reset
	out	dx,ax

@@drawnextrow:
	cmp	[Odd],0	; did we just do an odd line?
	jne	@@doodd

	mov	ax,[ELEP]	; Get Even Pattern
	mov	[CEP],ax
	mov	ax,[ELOP]
	mov	[COP],ax
	jmp	short @@nextline

@@doodd:
	mov	ax,[OLEP]	; Get Odd Pattern
	mov	[CEP],ax
	mov	ax,[OLOP]
	mov	[COP],ax

@@nextline:
	mov	cx,si	; restore width
	cmp	cx,1
	jne	@@normalrect

;*
;* both first and last bytes are the same so do them here.
;*
;
; Do even pixels of first/last byte.
;
	mov	ax,0AA08h	; Set EGA BitMask.
	and	ah,[SMask]
	and	ah,[EMask]
	out	dx,ax

	mov	ax,[CEP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

;
; Do odd pixels of first byte.
;
	mov	ax,05508h	; Set EGA BitMask.
	and	ah,[SMask]
	and	ah,[EMask]
	out	dx,ax

	mov	ax,[COP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

	jmp	short @@nolstbytehere
@@normalrect:
	cmp	[SMask],0FFh	; do first byte if necessary.
	je	@@no1stbyte

;
; Do even pixels of first byte.
;
	mov	ax,0AA08h	; Set EGA BitMask.
	and	ah,[SMask]
	out	dx,ax

	mov	ax,[CEP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

;
; Do odd pixels of first byte.
;
	mov	ax,05508h	; Set EGA BitMask.
	and	ah,[SMask]
	out	dx,ax

	mov	ax,[COP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

;
; move to next byte.
;
	inc	di
	dec	cx

@@no1stbyte:
	cmp	[EMask],0FFh	; is there a last byte?
	je	@@nolstbyte
	dec	cx
	jz	@@not_odd

@@nolstbyte:
	push	cx	; save low cl

;
; Do Even Pixels of Row.
;
	mov	ax,0AA08h	; Set EGA BitMask
	out	dx,ax

	mov	ax,[CEP]	; Set EGA Set/Reset (color)
	out	dx,ax

	rep	stosb	; Do Even Pixels

	pop	cx	; Backup
	push	cx
	sub	di,cx

;
; Do Odd Pixels of Row.
;
	mov	ax,05508h	; Set EGA BitMask
	out	dx,ax

	mov	ax,[COP]	; Set EGA Set/Reset (color)
	out	dx,ax

@@oloop:
	or	[es:di],al	; Do Odd Pixels.
	inc	di
	loop	@@oloop

	pop	cx
@@not_odd:
	cmp	[EMask],0FFh
	je	@@nolstbytestl	; is there a last byte?

;
; Do even pixels of last byte.
;
	mov	ax,0AA08h	; Set EGA BitMask.
	and	ah,[EMask]
	out	dx,ax

	mov	ax,[CEP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

;
; Do odd pixels of last byte.
;
	mov	ax,05508h	; Set EGA BitMask.
	and	ah,[EMask]
	out	dx,ax

	mov	ax,[COP]	; Set EGA Set/Reset
	out	dx,ax

	or	[es:di],al	; Set Pixels

@@nolstbytehere:
	inc	di
	dec	cx

@@nolstbytestl:
	add	di,[Modulo]	; move to next line.

@@realnext:
	xor	[Odd],01h
	dec	[RHeight]
	jle	short @@alldone
	jmp	@@drawnextrow
@@alldone:
	ret

	ENDP

;=====================================================================
;**
;** EGA_DrawRect
;**
;** PURPOSE
;**	Draws a rectangle on the specified rastport in the current color
;**
;** INPUT
;**	DS:SI RastPort			:Not Implemented
;**	AX = starting x pos
;**	BX = starting y pos
;**	CX = width
;**	DX = height
;**
;** EFFECTS
;**	AX,BX,CX,DX,ES,DI,SI,DS
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC EGA_DrawRect

	if 1
	cmp	dx,0	; Skip if height = 0
	je	@@no_rect
	cmp	cx,0	; Skip if width = 0
	je	@@no_rect
	endif

	mov	[cs:RHeight],dx	; Save Height

	push	ds
	mov	dx,cs
	mov	ds,dx

	test	bx,01h	; Set up Odd flag.
	jnz	@@startodd
	mov	[Odd],0
	jmp	short @@setstart
@@startodd:
	mov	[Odd],1
@@setstart:
	shl	bx,1
	mov	di,[LineSt+BX]	; Get Starting address of y row
	mov	bx,ax
	shr	bx,1
	shr	bx,1
	shr	bx,1
	add	di,bx	; add x column.
	push	bx	; save start byte.

	mov	si,ax	; save xpos

	and	al,07h	; get start pixel num (0-7)
	xor	bh,bh
	mov	bl,al
	mov	dl,[SMasks+bx]	; get start mask
	mov	bx,si	; restore xpos.
	add	bx,cx	; add width
	dec	bx	; comp last pixel
	mov	cx,bx	; save end pixel
	and	bl,07h	; get end pixel num (0-7)
	xor	bh,bh
	mov	dh,[EMasks+bx]	; get end mask

	pop	ax	; get start byte
	shr	cx,1	; comp last byte
	shr	cx,1
	shr	cx,1
	sub	cx,ax	; comp width in bytes
	inc	cx

	mov	ax,320/8	; compute modulo of rect.
	sub	ax,cx
	mov	[Modulo],ax

	mov	ax,[EGrSeg]	; Get Segment of current page.
	mov	es,ax

	mov	si,cx	; save byte width.

	mov	bl,[EPenColor]	; Get Pen Color.
	mov	ah,[_EGAOddPatTbl+bx]	; Get Odd pattern image
	mov	al,[_EGAEvenPatTbl+bx]; Get Even pattern image

	call	SmallRect

	pop	ds

@@no_rect:
	ret

	ENDP EGA_DrawRect

;=====================================================================
;**
;** _EGA_DrawRect(x, y, width, height)
;**
;** PURPOSE
;**	Draws a rectangle on the current workpage in the current color
;**
;** INPUT
;**	From stack using C conventions.
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	PUBLIC _EGA_DrawRectAsm
	MPROC	_EGA_DrawRectAsm
	ARG	_LEFT:WORD, _TOP:WORD, _WIDTH:WORD, _HEIGHT:WORD

	push	bp
	mov	bp,sp

	push	di	; Save SI and DI
	push	si

	mov	dx,[_HEIGHT]	; Get C Parms
	mov	cx,[_WIDTH]
	mov	bx,[_TOP]
	mov	ax,[_LEFT]

	call	ClipBox
	jnc	@@no_rect
	call	EGA_DrawRect	; Draw the Rect
@@no_rect:

	pop	si	; Restore SI and DI
	pop	di

	pop	bp
	ret

	ENDP _EGA_DrawRectAsm

