/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * FONTS.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 09/10/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Font Routines
 *
 * HISTORY
 *		03/02/93 Tuesday (dcc) - added wFitString().
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#include <string.h>
#include <stdlib.h>
#include <echidna/etypes.h>
#include <echidna/graphics.h>
#include <echidna/fonts.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*------------------------------------------------------------------------*/
/**# MODULE:FONTS_DrawStringN                                             */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * DrawStringN
 *
 * SYNOPSIS
 *		short DrawStringN (
 *				Font	*font,
 *				short	 x,
 *				short	 y,
 *				char	*string,
 *				size_t	 len
 *			)
 *
 * PURPOSE
 *		Draw a string with the specified font and return the final X pos.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawStringN (
	Font	*font,
	short	 x,
	short	 y,
	char	*string,
	size_t	 len
)
{
	UBYTE	 c;
	GShape	*shape;
	GShape	**letters	= font->Letters;
	UBYTE	 first		= font->FirstLetter;
	UBYTE	 last		= font->LastLetter;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawStringN";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (len) {
		c = (UBYTE)(*string++);		/* is this an OK substitution?? */
		if (c >= first && c <= last) {
			shape = letters[c - first];
			DrawGShape (shape, x, y);
			x += shape->Width;
		}
		len--;
	}

	return x;

} /* DrawStringN */

/*------------------------------------------------------------------------*/
/**# MODULE:FONTS_DrawString                                              */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * DrawString
 *
 * SYNOPSIS
 *		short DrawString (
 *			Font	*font,
 *			short	 x,
 *			short	 y,
 *			char	*string
 *		)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short DrawString (
	Font	*font,
	short	 x,
	short	 y,
	char	*string
)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DrawString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	return DrawStringN (font, x, y, string, strlen (string));

} /* DrawString */

/*------------------------------------------------------------------------*/
/**# MODULE:FONTS_StringLenN                                              */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * StringLenN
 *
 * SYNOPSIS
 *		short StringLenN (Font *font, char *string, size_t len)
 *
 * PURPOSE
 *		Compute and return the width in pixels of the <len> bytes of
 *		<string> when printed using font <font>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short StringLenN (Font *font, char *string, size_t len)
{
	UBYTE	 c;
	GShape	**letters	= font->Letters;
	UBYTE	 first		= font->FirstLetter;
	UBYTE	 last		= font->LastLetter;
	short	 width		= 0;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StringLenN";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while (len) {
		c = (UBYTE)(*string++);		/* is this an OK substitution?? */
		if (c >= first && c <= last) {
			width += letters[c - first]->Width;
		}
		len--;
	}

	return width;

} /* StringLenN */

/*------------------------------------------------------------------------*/
/**# MODULE:FONTS_StringLen                                               */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * StringLen
 *
 * SYNOPSIS
 *		short StringLen  (Font *font, char *string)
 *
 * PURPOSE
 *		Compute and return the width in pixels of
 *		<string> when printed using font <font>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short StringLen  (Font *font, char *string)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "StringLen";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	return StringLenN (font, string, strlen (string));

} /* StringLen */

/*------------------------------------------------------------------------*/
/**# MODULE:FONTS_wFitString                                              */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * wFitString
 *
 * SYNOPSIS
 *		short wFitString(Font *pFont, char *sz, short wWidth)
 *
 * PURPOSE
 *		Return how many characters of string <sz> will fit
 *		in pixel width <wWidth> if printed using <pFont>.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short wFitString(Font *pFont, char *sz, short wWidth)
{

	UBYTE	 c;
	GShape	**letters	= pFont->Letters;
	UBYTE	 first		= pFont->FirstLetter;
	UBYTE	 last		= pFont->LastLetter;
	short	 n			= 0;
	short	 w			= 0;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "wFitString";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while ((c = (UBYTE)(*(sz+n))) != 0)
	{
		if (c >= first && c <= last)
		{
			w += letters[c - first]->Width;
			if (w > wWidth)
				return n;
		}
		n++;
	}
	return n;

} /* wFitString */

