;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** SETORG.ASM
;**
;** PROGRAMMER : Juan M. Alvarado
;**    VERSION : 00.000
;**    CREATED : 05/25/91
;**   MODIFIED : 05/25/91
;**       TABS : 05 09
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**			Routine for setting the origin of the screen so you can pan
;**			up, down, left and right one pixel at a time.
;**			Modification of a General purpose routine for scrolling in
;**			the book "Programmer's Guide To PC & PS/2 Video Systems"
;**			pages 387 to 390.
;**
;** HISTORY
;**
;**


; Name:			ScreenOrigin
;
; Function:		Set screen origin on EGA and VGA
; 
; Caller:		Microsoft C:
;
;					void ScreenOrigin (void);
;
;							
;
;

ADDR_6845	EQU	63h

_TEXT		SEGMENT byte public 'CODE'
			ASSUME	cs:_TEXT

			PUBLIC	_ScreenOrigin
			PUBLIC	_BytesPerRow
			PUBLIC	_SetOriginEnableFlag
			PUBLIC	_SetOriginX
			PUBLIC	_SetOriginY

_SetOriginX				dw	?		
_SetOriginY				dw	?
_SetOriginEnableFlag	dw	?		;do work or not
_BytesPerRow			dw	80		; bytes per row MCGA 320x200x256		

_ScreenOrigin	PROC	far 
			mov		ax, [cs:_SetOriginEnableFlag]
			or		ax, ax
			jz		@@OutDone		;=> nothing to do!

			push	si
			push	di
				
			mov		ax,40h
			mov		es,ax		;ES -> video BIOS data area

			mov		ax,[cs:_SetOriginY]		; AX := pixel y-coordinate
			mov		bx,[cs:_SetOriginX]		; BX := pixel x-coordinate

; setup for graphics modes (4 pixels per byte)
			mul		word ptr _BytesPerRow
								; AX := byte offset of start row

			xchg	ax,bx		; AX := column offset
								; BX := byte offset of start of row
			mov		cx,4		; 4 displayed pixels per byte.
			div		cl			; AH := Bit offset in byte.
								; AL :=  byte offset in pixel row
			mov		cl,ah		; CL := bit offset 

			shl		cl,1		; 2 bits per pixel (for Horizontal Pel Pan)
			xor		ah,ah		
			add		bx,ax		; BX := buffer offset

; update Start Address High and Low Registers
			mov		dx,es:[ADDR_6845]	;CRTC I/O port (3B4H or 3D4H)

			mov		ah,bh		; AH := value for Start Address High
			mov		al,0Ch		; AL := Start Address High reg number
			out		dx,ax		; update this register

			mov		ah,bl		; AH := value for Start Address Low
			inc		al			; AL := Start Address Low reg number
			out		dx,ax		; update this register

; update Horizontal Pel Pan
			add		dl,6		; DX := video status port
			in		al,dx		; Reset Attribute Controller flip-flop.

			mov		dl,0C0h		; DX := 3C0h (Attribute Controller port)
			mov		al,13h OR 20h	; AL bit 0-4 := Horiz Pel Pan reg num
									; AL bit 5 := 1 ; screen enabled, palette
									;				 cannot be modified.
			out		dx,al		; write Attribute Controller Address reg
								; 	(The Attribute Controller address
								;	flip-flop has been reset by the
								;	IN at L22.)
			jmp		$+2			; Waste a few cycles so Attribute Controller
								; 	can respond.
			mov		al,cl		; AL := value for Horiz Pel Pan reg
			and		al,7
			out		dx,al		; update this register


; Clear work flag and get out.
			mov		[cs:_SetOriginEnableFlag], 0	;= 0

			pop		di			; restore registers and exit
			pop		si
@@OutDone:
			ret

_ScreenOrigin	ENDP


_TEXT		ENDS

			END

