;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** MSHAPE.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 12/15/89
;**   MODIFIED : 10/02/92
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	MCGA Shape routine adapted from routine by John Reego.
;**
;** HISTORY
;**	01/19/92 Sunday (JMA) - Made clipping routines work properly with 
;**		boundries other than the screen edges.
;**	
;**

	IDEAL

	PUBLIC	_SPRx1
	PUBLIC	_SPRx2
	PUBLIC	_SPRy1
	PUBLIC	_SPRy2

; DRAW_SPRITE (sprite #, Xw, Yw)  {SI, BX, CX}
;
;     sprite #  - address of sprite structure found in sprite_addr_table
;     Xw        - X position in world coordinates (w)
;     Yw        - Y position in world coordinates (w)

; sprite_addr_table
;        DD      sprite address 0
;        DD      sprite address 1
;        DD      sprite address 2
;        DD      sprite address .
;        DD      sprite address n
;
; sprite_structure
;       DW       x adjustment
;       DW       x length
;       DW       y adjustment
;       DW       y length
;       DB       SPRITE DATA  ...
;
; CS:MClipLeft 	- least    x pixel address of screen in world coordinates
; CS:MClipRight	- greatest x pixel address of screen in world coordinates
; CS:MClipTop 	- least    y pixel address of screen in world coordinates
; CS:MClipBottom	- greatest y pixel address of screen in world coordinates
;
; SPRx1 - least    x pixel address of sprite in world coordinates
; SPRx2 - greatest x pixel address of sprite in world coordinates
; SPRy1 - least    y pixel address of sprite in world coordinates
; SPRy2 - greatest y pixel address of sprite in world coordinates
;
; ABS x1, x2, y1, y2 should already be computed

CLIP_LEFT	dw ?
CLIP_RIGHT	dw ?
CLIP_TOP		dw ?
CLIP_BOTTOM	dw ?
START_X		dw ?
START_Y		dw ?
CLIP_VALUE	dw ?

X_LENGTH		dw 0	; fail RESTORE_BGND
REAL_X_LENGTH	dw ?
LINES_TO_GO	dw ?
LINE_ADDR	dw ?

NextABSx1	dw ?
NextABSy1	dw ?
Screen_Up	db ?	; screen moving up
Screen_Down	db ?	; screen moving down flag
Screen_Right	db ?
Screen_Left	db ?

_SPRx1:
SPRx1		dw ?
_SPRx2:
SPRx2		dw ?
_SPRy1:
SPRy1		dw ?
_SPRy2:
SPRy2		dw ?

ds_save		dw ?
si_save		dw ?

LABEL   CASE_TABLE  WORD
		dw DRAW_BEGIN	; 0
		dw CL_BEGIN	; 1
		dw CR_BEGIN	; 2
		dw CT_BEGIN	; 3
		dw CTL_BEGIN	; 4
		dw CTR_BEGIN	; 5
		dw DRAW_BEGIN	; 6
		dw CL_BEGIN 	; 7
		dw CR_BEGIN	; 8

which_case	db ?

;******************************** TEST DATA ********************************
	IF 0
PUBLIC _test_sprite1
PUBLIC _test_sprite2
Label   _sprite_addr_table DWORD
	dw      _test_sprite1
	dw      @CODE
	dw      _test_sprite2
	dw      @CODE
	DD 256 DUP (?)	          ; space for sprite address table

LABEL   _test_sprite1 byte
	dw      0, 7, 0, 3      ; x_adj, x_len, y_adj, y_len
	db      3, 1, 9, 5, 3, 9, 9, 9, 5, 1, 9, 3

LABEL   _test_sprite2 byte
	dw	0, 24, 0, 15             ; x_adj, x_len, y_adj, y_len
	db      5, 6,6,6,6,6,6,6, 17
	db      8,6,6,6,6,6,6,6,6, 15
	db      10,6,6,6,6,6,6,6,6,6,6, 12
	db      14,2,2,6,6,6,6,6,6,6,6,6,6,2,2, 3, 3,4,4,4, 3
	db      16,2,2,2,6,6,6,6,6,6,6,6,6,6,2,2,2, 2, 4,4,4,4,4, 2
	db      16,2,2,2,2,6,6,6,6,6,6,6,6,2,2,2,2, 1, 5,4,4,4,4,4, 3
	db      14,2,2,2,2,2,6,6,6,6,2,2,2,2,2, 2, 4,4,4,4,4, 5
	db      13,2,2,2,2,2,2,2,2,2,2,2,2,2, 1, 4,4,4,4,4, 6
	db      3,2,2,2, 1, 5,2,2,2,2,2, 1, 3,2,2,2, 1, 3,4,4,4, 7
	db      3,2,2,2, 2, 1,8, 1, 1,8, 2, 3,2,2,2, 1, 1,4, 9
	db      5,2,12,12,12,12, 5, 4,2,2,2,4, 9
	db      6,12,12,12,12,12,7, 6, 2,6,4, 10
	db      5,7,12,12,12,7, 6, 3,6,4,6, 11
	db      3,7,12,7, 7, 1,4, 14
	db      1,7,20

	db      1024 dup (?)

	ENDIF


	PROC DRAW_SPRITEx NEAR
off_screen:
	sub   bx,bx
	ret

early_exit:
	sub   bx,bx	; set X_LENGTH to zero
	ret

DRAW_SPRITE1:
	cld
	mov   ax,cs
	mov   es,ax
;--------------------------------------------------------------------------
;       COMPUTE EDGES OF SPRITE IN WORLD COORDINATES
;
;          INPUT:  bx = x_addr of sprite in world coordinates
;                  cx = y_addr of sprite in world coordinates
;                  ds:si = sprite number
;--------------------------------------------------------------------------
	LODSW		; compute SPRx1
	add   bx,ax	;
	mov   [cs:SPRx1],bx	;
	LODSW		; compute SPRx2
	mov   [cs:X_LENGTH],ax	; save X_LENGTH
	add   bx, ax	;
	dec   bx		;
	mov   [cs:SPRx2], bx	;
	LODSW		; compute SPRy1
	add   cx, ax	;
	mov   [cs:SPRy1], cx	;
	LODSW		; compute SPRy2
	mov   [cs:LINES_TO_GO], ax	; save Y_LENGTH
	add   cx, ax	;
	dec   cx		;
	mov   [cs:SPRy2], cx	;
;--------------------------------------------------------------------------
;       IS ANY PART OF THE SPRITE ON THE VIEWING SCREEN ?
;--------------------------------------------------------------------------
	mov   [cs:ds_save], ds	; save ds {sprite_data}
	mov   ax, cs	; restore ds
	mov   ds, ax	;
	mov   [si_save], si	; save si {sprite data}

	mov   [which_case], 0	; assume no clipping
	mov   ax, [SPRy1]	; SPRITE BELOW SCREEN ?
	cmp   ax, [CS:MClipBottom]	;
	jg    early_exit	;
	mov   ax, [SPRy2]	; SPRITE ABOVE SCREEN ?
	cmp   ax, [CS:MClipTop]	;
	jl    early_exit	;
	mov   ax, [SPRx1]	; SPRITE RIGHT OF SCREEN ?
	cmp   ax, [CS:MClipRight]	;
	jg    earlyexit	;
	mov   ax, [SPRx2]	; SPRITE LEFT OF SCREEN ?
	cmp   ax, [CS:MClipLeft]	;
	jl    earlyexit	;
	jmp   SHORT keep_trucking
earlyexit:
	jmp   early_exit
offscrn:
	jmp   off_screen
keep_trucking:

;--------------------------------------------------------------------------
;       COMPUTE CLIPPING COMPONENTS (IF ANY)
;--------------------------------------------------------------------------
	sub   ax, ax	;
	lea   di, [CLIP_LEFT]	;
	mov   cx, 6	;
	REP   STOSW	; zero out clip variables
	
	mov   ax,[SPRx1]	; clip the left edge.
	mov   [START_X],ax
	neg   ax
	add   ax,[CS:MClipLeft]
	js    do_right	;
	jz    do_top	;
	mov   [CLIP_LEFT], ax	;
	mov   ax,[CS:MClipLeft]
	mov   [START_X],ax
	add   [which_case], 1	;
	jmp SHORT   do_top	;

do_right:
	mov   ax, [SPRx2]	; clip the right edge ?
	sub   ax, [CS:MClipRight]	;
	js    do_top	;
	je    do_top	;

	mov   [CLIP_RIGHT], ax	;
	add   [which_case], 2	;

do_top:
	mov   ax, [SPRy1]	; clip the top.
	mov   [START_Y],ax
	neg   ax
	add   ax, [CS:MClipTop]
	js    do_bottom	;
	jz    not_neg_d
	mov   [CLIP_TOP], ax	;
	mov   ax,[CS:MClipTop]
	mov   [START_Y],ax
	add   [which_case], 3	;
	jmp SHORT   not_neg_d	;

do_bottom:

	mov   ax, [SPRy2]	; clip the bottom ?
	sub   ax, [CS:MClipBottom]	;
	js    not_neg_d	;
	jz    not_neg_d	;
	mov   [CLIP_BOTTOM], ax	;
	add   [which_case], 6	;
not_neg_d:		;

;--------------------------------------------------------------------------
;     CLIPPING ALGORITHM    ah must = 0
;	                   bx = pixels remaining to DRAW / CLIP
; CASE:                     cx = # data bits DATA / TRANS
;     4--3--5               dx = LINES_TO_GO
;     1  0  2               bp = LINE_ADDR  (where DRAW starts for each line)
;     7--6--8               ds:si --> sprite data
;                           es:di --> screen address in DRAW modes
;--------------------------------------------------------------------------

	mov   ax, [X_LENGTH]	;
	mov   [REAL_X_LENGTH], ax	;
	sub   ax, [CLIP_RIGHT]	;
	sub   ax, [CLIP_LEFT]	;
	mov   [X_LENGTH], ax	;

	mov   ax, [LINES_TO_GO]	;
	sub   ax, [CLIP_TOP]	;
	sub   ax, [CLIP_BOTTOM]	;
	mov   [LINES_TO_GO], ax	;
	mov   bx, 1	; set not clipped flag
	ret

DRAW_SPRITE2:		;
	cld
	mov   bx, [START_Y]	;
	shl   bx, 1	;
	mov   bx, [LINEST+bx]	; multiply by 320
	add   bx, [START_X]	;
	mov   [LINE_ADDR], bx	;

	mov   dx, [LINES_TO_GO]	; dx = LINES_TO_GO
	mov   ax, [CS:MGrSeg]	;
	mov   es, ax	;
	mov   si, [si_save]	; restore si --> sprite data

	mov   bl, [which_case]	;
	sub   bh, bh	;
	shl   bx, 1	;
	jmp   [case_table+bx]	;





;--------------------------------------------------------------------------
;    ENTRY POINTS
;--------------------------------------------------------------------------

CTL_BEGIN:		; CASES (1, 7)

	mov   bp, [LINE_ADDR]	;
	sub   bp, 320	; point back 1 line (use cr rtn)
	mov   bx, [CLIP_LEFT]	;
	mov   [CLIP_VALUE], bx	;
	push  bx	;
; ---------------------------------------------------------------------
	mov   bx, [CLIP_TOP]
	mov   ax, [REAL_X_LENGTH]
	mul   bl
	mov   bx, ax
	mov   ds, [ds_save]
	sub   ah, ah
	LODSB
CTL_TRAN:
	cmp   ax, bx
	ja    CTL_TRAN_gr
	je    CTL_TRAN_eq
	sub   bx, ax
	LODSB
	jmp SHORT   CTL_DATA

CTL_TRAN_eq:
	LODSB
	pop   bx
	jmp   CR_CLIP_DATA

CTL_TRAN_gr:
	sub   ax, bx
	pop   bx
	jmp   CR_CLIP_TRANS

CTL_DATA:
	cmp   ax, bx
	ja    CTL_DATA_gr
	je    CTL_DATA_eq
	sub   bx, ax
	add   si, ax
	LODSB
	jmp SHORT   CTL_TRAN

CTL_DATA_eq:
	add   si, ax
	LODSB
	pop   bx
	jmp   CR_CLIP_TRANS

CTL_DATA_gr:
	sub   ax, bx
	add   si, bx
	pop   bx
	jmp   CR_CLIP_DATA
; ---------------------------------------------------------------------

CTR_BEGIN:		; CASES (2, 8)
	mov   ax, [CLIP_RIGHT]	;
	mov   [CLIP_VALUE], ax	;
	mov   bx, [X_LENGTH]	;
	push  bx
	mov   bp, [LINE_ADDR]	;
	mov   di, bp	;
; ---------------------------------------------------------------------
	mov   bx, [CLIP_TOP]
	mov   ax, [REAL_X_LENGTH]
	mul   bl
	mov   bx, ax
	mov   ds, [ds_save]
	sub   ah, ah
	LODSB
CTR_TRAN:
	cmp   ax, bx
	ja    CTR_TRAN_gr
	je    CTR_TRAN_eq
	sub   bx, ax
	LODSB
	jmp SHORT   CTR_DATA

CTR_TRAN_eq:
	LODSB
	pop   bx
	jmp   CR_DRAW_DATA

CTR_TRAN_gr:
	sub   ax, bx
	pop   bx
	jmp   CR_DRAW_TRANS

CTR_DATA:
	cmp   ax, bx
	ja    CTR_DATA_gr
	je    CTR_DATA_eq
	sub   bx, ax
	add   si, ax
	LODSB
	jmp SHORT   CTR_TRAN

CTR_DATA_eq:
	add   si, ax
	LODSB
	pop   bx
	jmp   CR_DRAW_TRANS

CTR_DATA_gr:
	sub   ax, bx
	add   si, bx
	pop   bx
	jmp   CR_DRAW_DATA
; ---------------------------------------------------------------------

CT_BEGIN:		; CASES (0, 6)
	mov   [CLIP_VALUE], 0	;
	mov   bx, [X_LENGTH]	;
	mov   bp, [LINE_ADDR]	;
	mov   di, bp	;
	push  bx
; ---------------------------------------------------------------------
	mov   bx, [CLIP_TOP]
	mov   ax, [REAL_X_LENGTH]
	mul   bl
	mov   bx, ax
	mov   ds, [ds_save]
	sub   ah, ah
	LODSB
CT_TRAN:
	cmp   ax, bx
	ja    CT_TRAN_gr
	je    CT_TRAN_eq
	sub   bx, ax
	LODSB
	jmp SHORT   CT_DATA

CT_TRAN_eq:
	LODSB
	pop   bx
	jmp   DRAW_DATA

CT_TRAN_gr:
	sub   ax, bx
	pop   bx
	jmp SHORT   DRAW_TRANS

CT_DATA:
	cmp   ax, bx
	ja    CT_DATA_gr
	je    CT_DATA_eq
	sub   bx, ax
	add   si, ax
	LODSB
	jmp SHORT   CT_TRAN

CT_DATA_eq:
	add   si, ax
	LODSB
	pop   bx
	jmp SHORT   DRAW_TRANS

CT_DATA_gr:
	sub   ax, bx
	add   si, bx
	pop   bx
	jmp   DRAW_DATA
; ---------------------------------------------------------------------


CL_BEGIN:		; CASES (1, 7)

	mov   bp, [LINE_ADDR]	;
	sub   bp, 320	; point back 1 line (use cr rtn)
	mov   bx, [CLIP_LEFT]	;
	mov   [CLIP_VALUE], bx	;
	mov   ds, [ds_save]	; CLIP_LEFT
	sub   ah, ah	;
	LODSB		;
	jmp   CR_CLIP_TRANS	;


CR_BEGIN:		; CASES (2, 8)
	mov   ax, [CLIP_RIGHT]	;
	mov   [CLIP_VALUE], ax	;
	mov   bx, [X_LENGTH]	;
	mov   bp, [LINE_ADDR]	;
	mov   di, bp	;
	mov   ds, [ds_SAVE]	;
	sub   ah, ah	;
	LODSB		;
	jmp   CR_DRAW_TRANS	;


DRAW_BEGIN:		; CASES (0, 6)
	mov   [CLIP_VALUE], 0	;
	mov   bx, [X_LENGTH]	;
	mov   bp, [LINE_ADDR]	;
	mov   di, bp	;
	mov   ds, [ds_save]	;
	sub   ah, ah	;
	LODSB		;
	jmp SHORT   DRAW_TRANS	;


;--------------------------------------------------------------------------
;    DRAW SPRITE WITHOUT CLIPPING
;--------------------------------------------------------------------------

DRAW_TRANS:
	cmp   ax, bx
	ja    DT_gr
	je    DT_eq
	add   di, ax
	sub   bx, ax
	LODSB
	jmp SHORT   DRAW_DATA

zxc_ALL_DONE2:
	jmp   ALL_DONE2

DT_eq:
	dec   dx
	jz    zxc_ALL_DONE2
	add   bp, 320
	mov   di, bp
	lodsb
	mov   bx, [cs:X_LENGTH]
	jmp SHORT   DRAW_DATA

DT_gr:
	dec   dx
	jz    zxc_ALL_DONE2
	sub   ax, bx
	add   bp, 320
	mov   di, bp
	mov   bx, [cs:X_LENGTH]
	jmp SHORT   DRAW_TRANS

will_wrap1:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd1	;

DRAW_DATA:
	cmp   ax, bx
	ja    DD_gr
	je    DD_eq
	sub   bx, ax
	mov   cx, ax
	add   di, cx
	jc    will_wrap1
	sub   di, cx
	shr   cx, 1
	REP   MOVSW
	jnc   not_odd1
	MOVSB
not_odd1:
	LODSB
	jmp SHORT   DRAW_TRANS

will_wrap2:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd2	;

DD_eq:
	mov   cx, ax
	add   di, cx
	jc    will_wrap2
	sub   di, cx
	shr   cx, 1
	REP   MOVSW
	jnc   not_odd2
	MOVSB
not_odd2:
	dec   dx
	jz    ALL_DONE2
	LODSB
	add   bp, 320
	mov   di, bp
	mov   bx, [cs:X_LENGTH]
	jmp SHORT   DRAW_TRANS

will_wrap3:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd3	;

DD_gr:
	sub   ax, bx
	xchg  bx, ax
	mov   cx, ax
	add   di, cx
	jc    will_wrap3
	sub   di, cx
	shr   cx, 1
	REP   MOVSW
	jnc   not_odd3
	MOVSB
not_odd3:
	dec   dx
	jz    ALL_DONE2
	mov   ax, bx
	add   bp, 320
	mov   di, bp
	mov   bx, [cs:X_LENGTH]
	jmp SHORT   DRAW_DATA

ALL_DONE2:
	mov   ax, cs
	mov   ds, ax
	mov   ax, [START_X]
	add   ax, [CS:MClipLeft]
	mov   bx, [X_LENGTH]
	mov   cx, [START_Y]
	add   cx, [CS:MClipTop]
	mov   dx, [LINES_TO_GO]
	ret


;--------------------------------------------------------------------------
;    DRAW SPRITE WITH CLIPPING	  CLIP RIGHT & CLIP LEFT
;--------------------------------------------------------------------------

CR_DRAW_TRANS:		; (CASE 5, 2, 8)
	cmp   ax, bx	; trans bits, bits to go
	ja    CR_DT_gr	;
	je    CR_DT_eq	;
	add   di, ax	; add trans bits
	sub   bx, ax	; subtract bits to go
	LODSB		;
	jmp SHORT   CR_DRAW_DATA	; go to data mode

CR_DT_eq:
	dec   dx		; LINES_TO_GO
	jz    ALL_DONE2	;
	LODSB		;
	mov   bx, [cs:CLIP_VALUE]	;
	jmp   CR_CLIP_DATA	;

CR_DT_gr:		;
	dec   dx		; LINES_TO_GO
	jz    ALL_DONE2	;
	sub   ax, bx	; subtract from trans bit count
	mov   bx, [cs:CLIP_VALUE]	;
	jmp SHORT   CR_CLIP_TRANS	;


will_wrap4:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd4	;

CR_DRAW_DATA:		;  (CASE 5, 2, 8)
	cmp   ax, bx	;
	ja    CR_DD_gr	;
	je    CR_DD_eq	;
	sub   bx, ax	; draw area will not be filled
	mov   cx, ax
	add   di, cx
	jc    will_wrap4
	sub   di, cx
	shr   cx, 1
	REP   MOVSW	; draw the sprite data
	jnc   not_odd4
	MOVSB
not_odd4:
	LODSB		;
	jmp SHORT   CR_DRAW_TRANS	; draw the transparent bytes

will_wrap5:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd5	;

CR_DD_eq:		;
	mov   cx, ax
	add   di, cx
	jc    will_wrap5
	sub   di, cx
	shr   cx, 1
	REP   MOVSW	;
	jnc   not_odd5
	MOVSB
not_odd5:
	dec   dx		; LINES_TO_GO
	jz    ALL_DONE2	;
	LODSB		;
	mov   bx, [cs:CLIP_VALUE]	; establish clip_right length
	jmp SHORT   CR_CLIP_TRANS	; goto clipright - transparent

will_wrap6:		;
	sub   di, cx	;
	REP   MOVSB	;
	jmp   not_odd6	;

CR_DD_gr:		;
	xchg  ax, bx	; bx = data count, ax = draw bits left
	sub   bx, ax	; bx = count left after draw
	mov   cx, ax
	add   di, cx
	jc    will_wrap6
	sub   di, cx
	shr   cx, 1
	REP   MOVSW	; draw the data to clip edge
	jnc   not_odd6
	MOVSB
not_odd6:
	dec   dx		; LINES_TO_GO
	jz    uio_ALL_DONE2	;
	mov   ax, bx	; ax = bits of data left
	mov   bx, [cs:CLIP_VALUE]	; bx = clip right length
	jmp SHORT   CR_CLIP_DATA	;

uio_ALL_DONE2:
	jmp   ALL_DONE2

CR_CLIP_TRANS:	;
		; ah = 0
		; bx = clip length remaining
		; ax = # of transparent bits remaining
		; ds:si --> data count, data bits ...
		; es:di --> doesn't matter, we are not drawing here
		; dx = LINES_TO_GO
		; bp = LINE_ADDRESS (for data)

	cmp   ax, bx	;
	ja    CR_CT_gr	;
	je    CR_CT_eq	;
	sub   bx, ax	; decriment clip length remaining
	LODSB		; get data count
	jmp SHORT   CR_CLIP_DATA	; more clipping to do

CR_CT_eq:
	LODSB		;
	mov   bx, [cs:X_LENGTH]	; # of bits to draw
	add   bp, 320	; LINE_ADDRESS
	mov   di, bp	;
	jmp SHORT   CR_DRAW_DATA	; DATA MODE/DRAW

CR_CT_gr:
	sub   ax, bx	; dec trans bit count
	mov   bx, [cs:X_LENGTH]	; new draw count
	add   bp, 320	; update LINE_ADDRESS
	mov   di, bp	;
	jmp   CR_DRAW_TRANS	;

CR_CLIP_DATA:
		; ah = 0
		; bx = clip length remaining
		; ax = # of data bits remaining
		; ds:si --> transparent count
		; es:di --> doesn't matter, we are not drawing here
		; dx = LINES_TO_GO
		; bp = LINE_ADDRESS (for data)

	cmp   ax, bx	; # data bits input stream, # bits to clip
	ja    CR_CD_gr	; more data bits in input stream
	je    CR_CD_eq	; data bits input stream = clip bits
	sub   bx, ax	; decriment clip bits remaining
	add   si, ax	; point past data
	LODSB		; load transparent count
	jmp SHORT   CR_CLIP_TRANS	; go to clipping trans mode

CR_CD_eq:		;
	add   si, ax	; point past clipped data
	LODSB		; get transparent length
	add   bp, 320	; update LINE_ADDRESS
	mov   di, bp	; point to next line
	mov   bx, [cs:X_LENGTH]	; reset # of bits to draw
	jmp   CR_DRAW_TRANS	; switch to trans mode, now drawing

CR_CD_gr:		;
	add   si, bx	; point past clipped data
	sub   ax, bx	; more data in input stream
	add   bp, 320	; update LINE_ADDRESS
	mov   di, bp	; point to next line to draw
	mov   bx, [CS:X_LENGTH]	; reset # of bits to draw
	jmp   CR_DRAW_DATA	; still in data mode, now drawing

ENDP DRAW_SPRITEx

;=====================================================================
;**
;** MCGA_DrawShape
;**
;** PURPOSE
;**	Draw a Shape on an MCGA screen.
;**
;** INPUT
;**	DS:SI	far pointer to Shape data.
;**	BX	xpos.
;**	CX	ypos.
;**
;** REGISTER USED
;**	All of them
;**
;** EFFECTS
;**	??
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	May have problems if Shape Data crosses the end of the
;**	specified SEGMENT, so, NORMALIZE DS:SI so that SI is a
;**	small value (0-15).
;**
;** HISTORY
;**	
;**

	MPROC	MCGA_DrawShape
	CALL	DRAW_SPRITE1
	or	bx,bx
	jz	@@cliped
	CALL	DRAW_SPRITE2
@@cliped:
	ret
	ENDP

;=====================================================================
;**
;** _MCGA_DrawShape
;**
;** PURPOSE
;**	C glue for DrawShape.  Draws a shape on an MCGA screen.
;**
;** INPUT
;**	C style parms: DrawShape (far *Shape, int x, int y)
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	May have problems if Shape Data crosses the end of the
;**	specified SEGMENT, so, NORMALIZE DS:SI so that SI is a
;**	small value (0-15).
;**
;** HISTORY
;**	
;**

	MPROC _MCGA_DrawShape
	ARG	SpOff:WORD,SpSeg:WORD,XPOS:WORD,YPOS:WORD

	push	bp
	mov	bp,sp

	push	ds
	push	si
	push	di

	mov	ds,[SpSeg]
	mov	si,[SpOff]
	add	si,8	; Skip Comment Shape Header.
	mov	bx,[XPOS]
	mov	cx,[YPOS]
	call	MCGA_DrawShape

	pop	di
	pop	si
	pop	ds

	pop	bp
	ret
	ENDP

