;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** TGFX.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 12/15/89
;**   MODIFIED : 10/02/92
;**       TABS : 09 15 40 44
;**
;**    	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**    Various Graphics Routines for TGA mode.
;**    
;** HISTORY
;**
;**

	IDEAL

;********************************* EXTERNS *********************************
	EXTRN _TGA_OpenGraphics:PROC
	EXTRN _TGA_CloseGraphics:PROC
	EXTRN _TGA_ConvertShape:PROC
	EXTRN _TGA_FreeShape:PROC
	EXTRN _TGA_DrawShapeNoTrans:PROC
	EXTRN _TGA_DrawRect:PROC
	EXTRN _TGA_GetGShapeLoadBuf:PROC
	EXTRN _TGA_UsePaletteN:PROC
	EXTRN _TGA_FadeToPaletteN:PROC
	EXTRN _TGA_UseColorCycleSet:PROC   
	EXTRN _TGA_FadeToColorCycleSet:PROC
	EXTRN _TGA_UsePaintBoxN:PROC
	EXTRN _TGA_FadeToPaintBoxN:PROC
	EXTRN _TGA_OpenPainter:PROC
	EXTRN _TGA_ClosePainter:PROC       

;********************************* SEGMENT *********************************
	INCLUDE "GFXMDL.inc"
	WSEG	TGFX_TEXT

;**************************** GLOBAL VARIABLES *****************************
	PUBLIC _TGARoutineTable
	PUBLIC _TGADataTable
	PUBLIC _TDMemTbl
	PUBLIC _TDAcsTbl
	PUBLIC _TDispVal

;***************************** PUBLIC ROUTINES *****************************
	PUBLIC _TGA_ASMOpenGraphics
	PUBLIC _TGA_ASMCloseGraphics
	PUBLIC _TGA_SetColorReg
	PUBLIC _TGA_ASMSetPalette

;******************************** VARIABLES ********************************
TGrSeg		 DW ?
TPenColor	 DB ?
TPaperColor	 DB ?
TWorkBuffer	 DW 2
TBackBuffer	 DW 4
TDisplayBuffer	 DW 0
_TDispVal	DB 11000110b
_TDMemTbl	DW 0B800h	;Display
		DW 0B800h	;Work
		DW 0	;Back
_TDAcsTbl	DW 00110000b	;Display
		DW 00100000b	;Work
		DW 00000000b	;Back
_TGAEvenPatTbl DB 000h ; 0
		DB 011h ; 1
		DB 022h ; 2
		DB 033h ; 3
		DB 044h ; 4
		DB 055h ; 5
		DB 066h ; 6
		DB 077h ; 7
		DB 088h ; 8
		DB 099h ; 9
		DB 0AAh ;10
		DB 0BBh ;11
		DB 0CCh ;12
		DB 0DDh ;13
		DB 0EEh ;14
		DB 0FFh ;15
		DB 240 DUP (?)
_TGAOddPatTbl	DB 000h ; 0
		DB 011h ; 1
		DB 022h ; 2
		DB 033h ; 3
		DB 044h ; 4
		DB 055h ; 5
		DB 066h ; 6
		DB 077h ; 7
		DB 088h ; 8
		DB 099h ; 9
		DB 0AAh ;10
		DB 0BBh ;11
		DB 0CCh ;12
		DB 0DDh ;13
		DB 0EEh ;14
		DB 0FFh ;15
		DB 240 DUP (?)

	MACRO Rptr Routine

	DW	OFFSET Routine, SEG Routine

	ENDM

_TGADataTable:
	RPtr	TPenColor
	RPtr	TPaperColor
	Rptr	TGrSeg
	RPtr	TWorkBuffer
	RPtr	TBackBuffer
	RPtr	TDisplayBuffer
	RPtr	_TGAEvenPatTbl
	RPtr	_TGAOddPatTbl

_TGARoutineTable:
	Rptr	_TGA_OpenGraphics
	Rptr	_TGA_CloseGraphics
	Rptr	_TGA_DrawPixel
	Rptr	_TGA_DrawRect
	Rptr	_TGA_ClearScreen
	Rptr	_TGA_DrawShape
	Rptr	_TGA_Restore
	Rptr	_TGA_UnRestore
	Rptr	_TGA_ShowRect
	Rptr	_TGA_ConvertShape
	Rptr	_TGA_FreeShape
	Rptr	_TGA_DrawShapeNoTrans
	Rptr	_TGA_GetGShapeLoadBuf
	Rptr	_TGA_UsePaletteN
	Rptr	_TGA_FadeToPaletteN
	Rptr	_TGA_UseColorCycleSet   
	Rptr	_TGA_FadeToColorCycleSet
	Rptr	_TGA_UsePaintBoxN
	Rptr	_TGA_FadeToPaintBoxN
	Rptr	_TGA_OpenPainter
	Rptr	_TGA_ClosePainter       
	Rptr	_TGA_SetClipArea       
	Rptr	0

	LABEL LineSt  WORD              ; Lookup table for line start adresses
x       =     0
        REPT 50
        dw  x*160+0000h
        dw  x*160+2000h
        dw  x*160+4000h
        dw  x*160+6000h
x       =   x + 1
        ENDM


PMasks	DB	0F0h
	DB	00Fh

;=====================================================================
;**
;** _TGA_ASMCloseGraphics
;**
;** PURPOSE
;**	Free TGA resources and return to text mode.
;**
;** INPUT
;**	NONE
;**
;** REGISTERS USED
;**	AX,BX,ES
;**
;** EFFECTS
;**	OBuffer and BBuffer 64000 bytes * 2 freed.
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC _TGA_ASMCloseGraphics
	MPROC TGA_ASMCloseGraphics

	mov	ax,03h	; Return to Text mode
	int	10h

@@no_obuf:
	mov	ax,0	; Return 0 for OpenGraphics Failure.
	ret
	ENDP
	ENDP

;=====================================================================
;**
;** TGA_ASMOpenGraphics
;** _TGA_ASMOpenGraphics
;**
;** PURPOSE
;**	Initialize TGA Graphics Routines.
;**
;** INPUT
;**	NONE
;**
;** REGISTERS USED
;**	AX, BX, ES
;**
;** EFFECTS
;**	Allocates 2 64000 byte buffers and enters TGA mode
;**
;** RETURN VALUE
;**	AX = 0  failed
;**	   = !0 success
;**
;** LIMITS/BUGS
;**		
;**
;** HISTORY
;**		
;**

	MPROC _TGA_ASMOpenGraphics
	MPROC TGA_ASMOpenGraphics

	mov	ax,09h	; Start TGA mode.
	int	10h

	ret
	ENDP
	ENDP


;=====================================================================
;**
;** TGA_DrawPixel
;**
;** PURPOSE
;**	Draw one Pixel on the current work page.
;**
;** INPUT
;**	AX       = xpos
;**	BX       = ypos
;**	PenColor = color
;**
;** REGISTERS USED
;**	AX, BX, CX, DX, DI, ES
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC TGA_DrawPixel

	mov	cl,bl	; Save Ypos
	shl	bx,1
	mov	di,[CS:LineSt+BX]	; Get Starting address of y row
	mov	bx,ax
	shr	bx,1
	add	di,bx	; add x column.
	and	al,01h	; get pixel num (0-1);
	mov	bl,al
	xor	bh,bh
	mov	al,[CS:PMasks+bx]

	push	ax
	mov	bx,[CS:TGrSeg]	; Point Hardware to current page
	mov	ax,[CS:_TDAcsTbl+bx]	;
	or	al,[CS:_TDispVal]	; Or in current display
	mov	dx,03DFh
	out	dx,al
	mov	bx,[CS:_TDMemTbl+bx]	; Get current page's segment
	mov	es,bx
	pop	ax

	xor	bh,bh
	mov	bl,[CS:TPenColor]	; Get Pen Color.
	test	cl,01h	; odd or even line?
	jnz	@@oddline
	mov	dl,[CS:_TGAEvenPatTbl+bx]
	jmp	short @@gotit
@@oddline:
	mov	dl,[CS:_TGAOddPatTbl+bx]
@@gotit:
	mov	bl,dl	; Get old screen data.
	xor	bl,[es:di]
	and	bl,al
	xor	bl,[es:di]
	mov	[es:di],bl

	ret

	ENDP


;=====================================================================
;**
;** _TGA_DrawPixel
;**
;** PURPOSE
;**	C Glue to TGA_DrawPixel
;**
;** INPUT
;**	C parms: TGA_DrawPixel (int x, int y)
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	NONE
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC _TGA_DrawPixel
	ARG  XPOS:WORD, YPOS:WORD

	push	bp
	mov	bp,sp

	push	di

	mov	bx,[YPOS]
	mov	ax,[XPOS]
	call	TGA_DrawPixel

	pop	di

	pop	bp
	ret

	ENDP

;=====================================================================
;**
;** _TGA_SetColorReg (short regnum, UBYTE color)
;**
;** PURPOSE
;**	
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC _TGA_SetColorReg
	ARG  REGNUM:WORD, COLOR:BYTE

	push	bp
	mov	bp,sp

	mov	dx,03DAh	; Point to correct palette reg
	mov	ax,[REGNUM]
	or	al,010h
	out	dx,al

	mov	dx,03DEh	; Set Color
	mov	al,[COLOR]
	out	dx,al

	pop	bp
	ret

	ENDP


;=====================================================================
;**
;** _TGA_ASMSetPaletteg (UBYTE *color)
;**
;** PURPOSE
;**	
;**
;** INPUT
;**	
;**
;** REGISTER USED
;**	
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC _TGA_ASMSetPalette
	ARG	SpOff:WORD,SpSeg:WORD

	push	bp
	mov	bp,sp

	push	si
	push	ds

	cld

	mov	ds,[SpSeg]
	mov	si,[SpOff]

	mov	cx,10h
	mov	bx,10h
@@covloop:
	mov	dx,03DAh	; Point to correct palette reg
	mov	al,bl
	out	dx,al
	inc	bl

	mov	dx,03DEh
	lodsb
	out	dx,al

	loop	@@covloop

	pop	ds
	pop	si
	pop	bp
	ret

	ENDP


;******************************** INCLUDES *********************************
	INCLUDE "TSHAPE.asm"
	INCLUDE "TRESTORE.asm"
	INCLUDE "TRECT.asm"
	INCLUDE "TCLEAR.asm"
	INCLUDE "TWRTCREG.asm"

	ENDS
	END

