;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** TRESTORE.asm
;**
;** PROGRAMMER : Gregg A. Tavares
;**    VERSION : 00.000
;**    CREATED : 03/06/90
;**   MODIFIED : 10/02/92
;**       TABS : 10 18 40 44
;**
;**	     \|///-_
;**	     \oo///_
;**	-----w/-w------
;**	 E C H I D N A
;**	---------------
;**
;** DESCRIPTION
;**	ShowRect/Restore for TGA mode.
;**
;** HISTORY
;**
;**
	IDEAL

;=====================================================================
;**
;** ClipBox
;**
;** PURPOSE
;**	Clip x, y, width, height to screen.
;**
;** INPUT
;**	AX = x
;**	BX = y
;**	CX = width
;**	DX = height
;**
;** REGISTER USED
;**	SI, DI
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	AX,BX,CX,DX = new clipped box
;**	carry clear = box was entirely clipped
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**

	MPROC ClipBox NEAR

	mov	si,cx	; Save Width
	cmp	ax,0	; if (x < 0)
	jge	@@xnottoosmall
	mov	cx,ax	; width = inwidth - (0 - x);
	neg	cx
	sub	si,cx
	mov	cx,si
	sub	ax,ax	; x = 0;
	jmp	short @@xclipped

@@xnottoosmall:
	mov	di,320
	sub	di,si
	cmp	ax,di	; if (x > 0 + 320 - inwidth)
	jle	@@xclipped
	mov	cx,320	; width = (0 + 320) - x;
	sub	cx,ax

@@xclipped:

	mov	si,dx	; Save Height
	cmp	bx,0	; if (y < 0)
	jge	@@ynottoosmall
	mov	dx,bx	; height = inheight - (0 - y)
	neg	dx
	sub	si,dx
	mov	dx,si
	sub	bx,bx	; y = 0;
	jmp	short @@yclipped

@@ynottoosmall:
	mov	di,200
	sub	di,si
	cmp	bx,di	; if (y > 0 + 200 - inheight)
	jle	@@yclipped
	mov	dx,200	; height = (0 + 200) - y;
	sub	dx,bx

@@yclipped:

	cmp	dx,0	; if (x > 0 && y > 0)
	jg	@@xokay	; return carry set otherwise
	clc		; return carry clear
	ret

@@xokay:
	cmp	cx,0
	jg	@@yokay
	clc
	ret

@@yokay:
	stc
	ret

	ENDP

;=====================================================================
;**
;** TGA_CopyInPlace
;**
;** PURPOSE
;**	Copy a byte aligned rectangle of graphics from an one buffer
;**	to another.  (an area in one buffer to the SAME area in another)
;**
;** INPUT
;**	DS = source buffer segment
;**	ES = dest   buffer segment
;**	AX = xpos
;**	BX = ypos
;**	CX = width
;**	DX = height
;**
;** REGISTER USED
;**	ALL of them
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
	MPROC TGA_CopyInPlace NEAR

	call	ClipBox
	jc	@@coordsokay
	ret

@@coordsokay:
	cld

	shl	bx,1

	add	cx,ax	; compute last pixel
	dec	cx
	shr	cx,1	; compute last byte
	shr	ax,1	; compute first byte
	sub	cx,ax	; compute width in bytes
	inc	cx

	mov	bp,cx	; save width in bytes

@@copyrow:
	mov	cx,bp	; restore width
	mov	di,[cs:LineSt+bx]	; compute starting addr of source
	add	di,ax
	mov	si,[cs:LineSt+bx]	; compute starting addr of dest
	add	si,ax

	shr	cx,1	; compute width in words
	jz	@@nowords

	rep	movsw	; copy the words.

@@nowords:
	test	bp,001h	; is there a trailing byte?
	jz	@@nobyte

	movsb

@@nobyte:
	add	bx,2	; move index to next line
	dec	dx	; next line?
	jg	@@copyrow
	ret

	ENDP

;
; C PROTOTYPE:
;
; void far TGA_Restore(int xmin, int ymin, int xmax, int ymax);
;
; _Restore - restores from background buffer to the current work page.
; segment of background buffer is in [cs:Pages+4]
; segment of current work page is in [cs:GrPage]
; by Daniel Chang, 07/23/89.

	MPROC	_TGA_Restore
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	bx,[CS:TBackBuffer]	;source segment
	mov	bx,[CS:_TDMemTbl+bx]
	mov	ds,bx

	mov	bx,[CS:TWorkBuffer]	; Point Hardware to current page
	mov	ax,[CS:_TDAcsTbl+bx]	;
	or	al,[CS:_TDispVal]	; Or in current display
	mov	dx,03DFh
	out	dx,al
	mov	bx,[CS:_TDMemTbl+bx]	; Get current page's segment
	mov	es,bx

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	call	TGA_CopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_TGA_Restore


;
; C PROTOTYPE:
;
; void far TGA_UnRestore(int xmin, int ymin, int xmax, int ymax);
;
; _UnRestore - 'unrestores' from the current work page to background buffer.
; segment of background buffer is in [cs:Pages+4]
; segment of current work page is in [cs:GrPage]
; by Daniel Chang, 07/23/89.
; modified from TGA_Restore 07/08/91 (RGM).

	MPROC	_TGA_UnRestore
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	bx,[CS:TBackBuffer]	;source segment
	mov	bx,[CS:_TDMemTbl+bx]
;	mov	ds,bx
	mov	es,bx

	mov	bx,[CS:TWorkBuffer]	; Point Hardware to current page
	mov	ax,[CS:_TDAcsTbl+bx]	;
	or	al,[CS:_TDispVal]	; Or in current display
	mov	dx,03DFh
	out	dx,al
	mov	bx,[CS:_TDMemTbl+bx]	; Get current page's segment
;	mov	es,bx
	mov	ds,bx

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	call	TGA_CopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_TGA_UnRestore


;************************************  *************************************
;********************** Special Tandy Work -> Display **********************
;************************************  *************************************

;=====================================================================
;**
;** TGA_CopyInPlace
;**
;** PURPOSE
;**	Copy a byte aligned rectangle of graphics from an one buffer
;**	to another.  (an area in one buffer to the SAME area in another)
;**
;** INPUT
;**	DS = source buffer segment
;**	ES = dest   buffer segment
;**	AX = xpos
;**	BX = ypos
;**	CX = width
;**	DX = height
;**
;** REGISTER USED
;**	ALL of them
;**
;** EFFECTS
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	
;**
TSCPLins	DW	0
TSCPOff		DW	0
TSCPSrc		DB	0
TSCPDest	DB	0

	MPROC TGA_SpecialCopyInPlace NEAR

	call	ClipBox
	jc	@@coordsokay
	ret

@@coordsokay:
	cld

	mov	[CS:TSCPLins],dx	; Save No of Lines.

	shl	bx,1

	add	cx,ax	; compute last pixel
	dec	cx
	shr	cx,1	; compute last byte
	shr	ax,1	; compute first byte
	sub	cx,ax	; compute width in bytes
	inc	cx

	mov	[CS:TSCPOff],ax	; Save Line Offset
	mov	bp,cx	; save width in bytes

	mov	al,[BYTE PTR CS:_TDAcsTbl]
	or	al,[CS:_TDispVal]
	mov	[CS:TSCPDest],al
	mov	al,[BYTE PTR CS:_TDAcsTbl+2]
	or	al,[CS:_TDispVal]
	mov	[CS:TSCPSrc],al
	mov	dx,03DFh

@@copyrow:
	mov	cx,bp	; restore width
	mov	di,[cs:LineSt+bx]	; compute starting addr of source
	add	di,[cs:TSCPOff]
	mov	si,[cs:LineSt+bx]	; compute starting addr of dest
	add	si,[cs:TSCPOff]

	shr	cx,1	; compute width in words
	jz	@@nowords

@@loop:
	mov	al,[CS:TSCPSrc]	; Point To Work
	out	dx,al
	lodsw
	push	ax
	mov	al,[CS:TSCPDest]	; Point To Display
	out	dx,al
	pop	ax
	stosw
	loop	@@loop

@@nowords:
	test	bp,001h	; is there a trailing byte?
	jz	@@nobyte

	mov	al,[CS:TSCPSrc]	; Point To Work
	out	dx,al
	lodsb
	mov	ah,al
	mov	al,[CS:TSCPDest]	; Point To Display
	out	dx,al
	mov	al,ah
	stosb

@@nobyte:
	add	bx,2	; move index to next line
	dec	[CS:TSCPLins]	; next line?
	jg	@@copyrow
	ret

	ENDP

;************************************  *************************************
;
; C PROTOTYPE:
;
; void far TGA_ShowRect(int xmin, int ymin, int width, int height);
;
; _Restore - copies from work page to the display page.
; segment of current work page is in [cs:GrPage]
; segment of display buffer is in [cs:Pages]
; by Daniel Chang, 07/23/89.

	MPROC	_TGA_ShowRect
	ARG	xmin:WORD, ymin:WORD, rwidth:WORD, height:WORD=AUTO_SIZE

	push	bp
	mov	bp, sp
	sub	sp, AUTO_SIZE

	push	ds
	push	es
	push	di
	push	si

	mov	ax, 0b800h
	mov	ds, ax	; source segment
	mov	es, ax	; destination segment

	mov	ax, [xmin]
	mov	bx, [ymin]
	mov	cx, [rwidth]
	mov	dx, [height]

	call	TGA_SpecialCopyInPlace

	pop	si
	pop	di
	pop	es
	pop	ds

	add	sp, AUTO_SIZE
	pop	bp
	ret
	ENDP	_TGA_ShowRect

