/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * GMEMORY.c
 *
 * PROGRAMMER : Gregg A. Tavares (with suggestions by Juan M. Alavardo)
 *    VERSION : 00.000
 *    CREATED : 03/15/90
 *   MODIFIED : 07/27/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		Memory allocation routines for (games?)
 *
 * HISTORY
 *
*/

#include <echidna/platform.h>
#include "switches.h"

#define PMESS	0
#define POMESS	0

#include <echidna/etypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#if __TURBOC__
#include <dos.h>
#endif
#if __AMIGAOS__
#include <exec/memory.h>
#include <echidna/listfunc.h>
#if AZTEC_C
#include <functions.h>
#endif
#endif
#include <echidna/emacros.h>
#include <echidna/eerrors.h>
#include <echidna/memory.h>

#if LATTICE
extern int atexit (int (*) ());
#endif

/******************************** CONSTANTS *******************************/

#define MEM_STACK_SIZE	8

#if __AMIGAOS__
#define GRAPHIC_MEM_NEEDED	200000
#define REGULAR_MEM_NEEDED	15000
#endif

/********************************* MACROS *********************************/

#if __AMIGAOS__
#define RoundUp(val)	(((val) & 0x01) ? ((val) + 1) : (val))
#define RoundDown(val)	((val) & 0xFFFFFFFE)
#endif

/********************************** TYPES *********************************/

#if __AMIGAOS__
typedef struct {
	NodeType	 Node;
	ULONG		 Size;
	UBYTE		*LoMem;
	UBYTE		*HiMem;
	UBYTE		*LoMemStack[MEM_STACK_SIZE];
	UWORD		 Graphic;
} MemTracker;
#endif

/****************************** TRACKING VARS *****************************/

#if __MSDOS__
static UBYTE huge32	*LoMem = NULL;
static UBYTE huge32	*HiMem = NULL;

static UBYTE huge32	*LoMemStack[MEM_STACK_SIZE];
static UWORD		 LoMemStackPtr = 0;

static AllocMemFuncPtr	ExtraMemFunc  = NULL;
static FreeSomeFuncPtr	ExtraFSFunc = NULL;

static UBYTE huge32	*ExtraMem;
static ULONG		 ExtraSize;
static UBYTE		 ExtraLast;
#endif

#if __AMIGAOS__
static ListType		 MemList;
static MemTracker	*LastMem;
static UWORD		 LoMemStackPtr = 0;
static MemTracker	*TempMem;
#endif

/******************************** ROUTINES ********************************/

#if __AMIGAOS__
/**************************************************************************
 *
 * CleanupMemory
 *
 * SYNOPSIS
 *		
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static void CleanupMemory (void)
{
	MemTracker	*mem;

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "CleanupMemory";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	while ((mem = RemTail (&MemList))) {
		FreeMem (mem, mem->Size);
	}

} /* CleanupMemory */

static short GetTheMem (ULONG needed, ULONG typeflags)
{
	ULONG			 biggest;
	ULONG			 used;
	MemTracker		*mem;
	short			 status		= FALSE;
	static ULONG	 largest	= 0;

	Forbid ();

	while (needed > 0) {
		biggest = AvailMem (MEMF_LARGEST | typeflags);
		used = min (needed + sizeof (MemTracker), biggest - sizeof (MemTracker));
		if (used <= 0) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("OOM:initializing malloc");
			goto gtmcleanup;
		}
		mem = AllocMem (used, typeflags | MEMF_CLEAR);
		if (!mem) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("bug in GetTheMem (memory.c)\n");
			goto gtmcleanup;
		}
		AddTail (&MemList, mem);
		mem->Size    = used;
		mem->LoMem   = (UBYTE *)mem + sizeof (MemTracker);
		mem->Graphic = typeflags & MEMF_CHIP;
		used        -= sizeof (MemTracker);
		mem->HiMem   = mem->LoMem + used;
		needed      -= used;
		if (used > largest) {
			largest = used;
			TempMem = mem;
		}
	}

	status = TRUE;

gtmcleanup:
	Permit ();

	return status;
}

/********************************* AMIGAOS ********************************/
static short initmem (void)
{
	static UBYTE	 init = TRUE;

	if (init) {
		init = FALSE;

		InitList (&MemList);

		if (!GetTheMem (GRAPHIC_MEM_NEEDED, MEMF_CHIP)) {
			return FALSE;
		}

#if AZTEC_C
		{
			extern void (*_cln)(void);

			_cln = CleanupMemory;
		}
#elif LATTICE
		{
			if (atexit ((int (*)(void)) CleanupMemory)) {
				/* return FALSE; */
			}
		}
#else
#error Need to free mem on exit from this C compiler
#endif

		if (!GetTheMem (REGULAR_MEM_NEEDED, NULL)) {
			return FALSE;
		}

	}
	return TRUE;
}

/********************************* AMIGAOS ********************************/
static void *FindSomeMem (size_t size, short graphic)
{
	MemTracker	*mem;
	void		*data;

	mem = Head (&MemList);
	while (!IsTail (mem)) {
		if ((mem->Graphic != 0) == (graphic != 0)) {
			if (mem->HiMem - mem->LoMem >= size) {
				data        = mem->LoMem;
				mem->LoMem += size;
				LastMem     = mem;
				return data;
			}
		}
		mem = Next (mem);
	}

	return NULL;
}

#endif

#if __MSDOS__
static short initmem (void)
{
	static UBYTE	 init = TRUE;
	int				 biggest;
	unsigned		 seg;

	if (init) {
		init = FALSE;
		biggest = allocmem (65535U, &seg);
		if (allocmem (biggest, &seg) != -1) {
			SetGlobalErr (ERR_OUT_OF_MEMORY);
			GEprintf ("OOM: Initializing malloc");
			return FALSE;
		}
		LoMem = (UBYTE huge32 *)MK_FP(seg, 0x0000);
		HiMem = LoMem + biggest * 16L;
	}

	return TRUE;
}
#endif

ULONG memfree (void)
{
#if __MSDOS__
	return (ULONG)(HiMem - LoMem);
#elif __AMIGAOS__
	{
		ULONG		 total = 0;
		MemTracker	*mem;

		mem = Head (&MemList);
		while (!IsTail (mem)) {
			total += (ULONG)(mem->HiMem - mem->LoMem);
			mem    = Next (mem);
		}

		return total;
	}
#else
#error Need 'memfree' for this computer
#endif
}

void far32 *farmalloc (unsigned long size)
{
#if __MSDOS__
	{
		UBYTE huge32		*mem = NULL;

		if (size) {
			if (ExtraMemFunc) {
				mem = ExtraMemFunc (size);
			}
			if (initmem ()) {
				if (!mem) {
					if (HiMem - LoMem >= size) {
						mem       = LoMem;
						LoMem    += size;
						ExtraLast = FALSE;
					}
				}
			}

#if PMESS
		printf ("  farmalloc (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif
		}

		return (void far32 *)mem;
	}
#elif __AMIGAOS__
	{
		UBYTE 		*mem = NULL;

		if (size) {
			size = RoundUp (size);
			if (!initmem ()) {
				return NULL;
			}
			mem = FindSomeMem (size, FALSE);
			if (!mem) {
				mem = FindSomeMem (size, TRUE);
			}

#if PMESS
		printf ("  farmalloc (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif
		}

		return (void *)mem;
	}
#else
#error Need 'farmalloc' for this computer.
#endif
}

void far32 *malloc (size_t size)
{
	return farmalloc (size);
}

void farfree (void far32 *mem)
{
#if PMESS
	printf ("free ()\n");
#endif
	mem = mem;
}

void free (void far32 *mem) {
#if PMESS
	printf ("free ()\n");
#endif
	mem = mem;
}

void far32 *farcalloc (unsigned long num, unsigned long size)
{
	UBYTE far32	*mem;
	long		 rsize;

	rsize = num * size;
	mem = farmalloc (rsize);
	if (mem) {
		UBYTE huge32	*clr = (UBYTE huge32 *)mem;

		while (rsize) {
			size = min (rsize, 32000);

			memset (clr, 0, (short)size);
			clr   += size;
			rsize -= size;
		}
	}

	return mem;
}

void far32 *calloc (size_t num, size_t size)
{
	return farcalloc (num, size);
}

void freesome (size_t size)
{
#if __MSDOS__
	if (ExtraLast) {
		if (ExtraFSFunc) {
			ExtraFSFunc (size);
		}
	} else {
		LoMem -= size;
#if PMESS
	printf ("freesome (%6ld); memfree = %7lu\n", (long)size, memfree());
#endif
	}
#elif __AMIGAOS__
	size = RoundDown (size);
	LastMem->LoMem -= size;
#if PMESS
	printf ("freesome (%6ld); memfree = %7lu\n", (long)size, memfree());
#endif
#else
#error Need 'freesome' for this computer
#endif
}

/*********************************************************************
 *
 * OpenMemory
 *
 * SYNOPSIS
 *		short OpenMemory (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
short OpenMemory (void)
{
#if __MSDOS__
	{
#if REQUIRE
		if (ExtraMemFunc) {
			SetGlobalErr (ERR_GENERIC);
			GEprintf ("OpenMemory: Shouldn't be called with ExtraMemFunc is set\n");
			return FALSE;
		}
#endif
		if (LoMemStackPtr == MEM_STACK_SIZE) {
			SetGlobalErr (ERR_GENERIC);
			GEprintf ("OpenMemory: Stack full");
			return FALSE;
		}

		LoMemStack[LoMemStackPtr++] = LoMem;
		return TRUE;
	}
#elif __AMIGAOS__
	{
		MemTracker	*mem;

		if (!initmem ()) {
			return FALSE;
		}

		if (LoMemStackPtr == MEM_STACK_SIZE) {
			SetGlobalErr (ERR_GENERIC);
			GEprintf ("OpenMemory: Stack full");
			return FALSE;
		}

		mem = Head (&MemList);
		while (!IsTail (mem)) {
			mem->LoMemStack[LoMemStackPtr] = mem->LoMem;
			mem = Next (mem);
		}

#if POMESS
		printf (" OpenMemory(); memfree = %7lu\n", memfree());
#endif

		LoMemStackPtr++;
		return TRUE;

	}
#else
#error Need 'OpenMemory' for this platform
#endif

} /* OpenMemory */

/*********************************************************************
 *
 * CloseMemory
 *
 * SYNOPSIS
 *		void CloseMemory (void)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *		ExtraMemFunc
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void CloseMemory (void)
{
#if __MSDOS__
	{
		ExtraMemFunc = NULL;
		LoMem = LoMemStack[--LoMemStackPtr];
	}
#elif __AMIGAOS__
	{
		MemTracker	*mem;

#if ERRORS
		if (LoMemStackPtr <= 0) {
			SetGlobalErr (ERR_GENERIC);
			GEprintf ("CloseMemory: Call more than OpenMemory");
			return;
		}
#endif
		LoMemStackPtr--;
		mem = Head (&MemList);
		while (!IsTail (mem)) {
			mem->LoMem = mem->LoMemStack[LoMemStackPtr];
			mem = Next (mem);
		}

#if POMESS
		printf ("CloseMemory(); memfree = %7lu\n", memfree());
#endif
	}
#else
#error Need 'CloseMemory' for this platform.
#endif
} /* CloseMemory */


/*********************************************************************
 *
 * CreateTempBuffer
 *
 * SYNOPSIS
 *		void far32 *CreateTempBuffer (size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void far32 *CreateTempBuffer (size_t size)
{
#if __MSDOS__
	if (initmem ()) {
		if (HiMem - LoMem >= size) {
			HiMem -= size;

#if PMESS
		printf ("CreateTB (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif

			return (void far32 *)HiMem;
		}
	}

	return NULL;
#elif __AMIGAOS__
	if (!initmem ()) {
		return NULL;
	}
	size = RoundUp (size);
	if (TempMem->HiMem - TempMem->LoMem >= size) {
		TempMem->HiMem -= size;

#if PMESS
	printf ("CreateTB (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif

		return (void *)TempMem->HiMem;
	}

	return NULL;
#else
#error Need 'CreateTempBuffer' for this platform
#endif

} /* CreateTempBuffer */

/*********************************************************************
 *
 * DestroyTempBuffer
 *
 * SYNOPSIS
 *		void DestroyTempBuffer (void *mem, size_t size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void DestroyTempBuffer (void *mem, size_t size)
{
#if __MSDOS__
	mem    = mem;
	HiMem += size;

#if PMESS
	printf ("DestryTB (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif
#elif __AMIGAOS__
	mem  = mem;
	size = RoundUp (size);
	TempMem->HiMem += size;

#if PMESS
	printf ("DestryTB (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif

#else
#error Need 'DestroyTempBuffer' for this platform.'
#endif

} /* DestroyTempBuffer */

/************************************  ************************************/
/********************************** MSDOS *********************************/
/************************************  ************************************/

#if __MSDOS__
/*********************************************************************
 *
 * FreeMemOnPara
 *
 * SYNOPSIS
 *		void FreeMemOnPara (UWORD seg)
 *
 * PURPOSE
 *		Free memory allocated by AllocMemOnPara()
 *
 * INPUT
 *		seg = SEGMENT of memory as returned by AllocMemOnPara()
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreeMemOnPara (void *mem)
{
	mem = mem;
} /* FreeMemOnPara */

/*********************************************************************
 *
 * AllocMemOnPara
 *
 * SYNOPSIS
 *		UWORD AllocMemOnPara (ULONG size)
 *
 * PURPOSE
 *		Allocate memory for graphics on paragraph boundary because
 *		Turbo C is poo!
 *
 * INPUT
 *		Size of Mem you need in bytes.
 *		BUGBUG: Note that even though it now takes a ULONG (for
 *		compatibility with MEMORY.C), you are limited to allocating
 *		blocks of memory < 64K-18 under Borland C.
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *		Segment of Mem or 0
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void *AllocMemOnPara (ULONG size)
{
	UBYTE huge32 *mem;

	if (size+18 >= 65536UL)
		return NULL;

	mem = (UBYTE huge32 *)malloc ((unsigned) size+18);
	if (!mem) {
		return NULL;
	}
	while (FP_OFF(mem)) {
		mem++;
	}
	return (void *)mem;

} /* AllocMemOnPara */

/*********************************************************************
 *
 * SetExtraMemFuncs
 *
 * SYNOPSIS
 *		void SetExtraMemFuncs (AllocMemFuncPtr memfunc)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void SetExtraMemFuncs (AllocMemFuncPtr memfunc, FreeSomeFuncPtr fsfunc)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "SetExtraMemFunc";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	ExtraMemFunc = memfunc;
	ExtraFSFunc  = fsfunc;

} /* SetExtraMemFuncs */

/*********************************************************************
 *
 * DistrubuteMem
 *
 * SYNOPSIS
 *		static void *DistrubuteMem (size_t size)
 *
 * PURPOSE
 *		Use 'simple' Extra memory specified by UseExtraMemory ();
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
static void *DistrubuteMem (unsigned long size)
{
	UBYTE huge32 *mem = NULL;
/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "DistrubuteMem";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	if (size <= ExtraSize) {
		ExtraSize -= size;
		mem        = ExtraMem;
		ExtraMem  += size;
		ExtraLast  = TRUE;

#if PMESS
	printf ("DstrbtMm (%6lu),                     XtraFre = %7lu\n", (ULONG)size, (ULONG)ExtraSize);
#endif

	}

	return (void far32 *)mem;

} /* DistrubuteMem */

/*********************************************************************
 *
 * FreeSomeExtra
 *
 * SYNOPSIS
 *		void FreeSomeExtra (size_t size)
 *
 * PURPOSE
 *		Free some extra memory?
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void FreeSomeExtra (unsigned long size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "FreeSomeExtra";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/
	ExtraMem  -= size;
	ExtraSize += size;

#if PMESS
	printf (" FSExtra (%6ld);                     XtraFre = %7lu\n", (long)size, ExtraSize);
#endif
} /* FreeSomeExtra */

/*********************************************************************
 *
 * UseExtraMemory
 *
 * SYNOPSIS
 *		void UseExtraMemory (void *mem, size_t size)
 *
 * PURPOSE
 *		Specify a 'simple' piece of memory for use as extra memory.
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
void UseExtraMemory (void *mem, size_t size)
{

/*-------------------------------------------------------------------------*/
#if FUNC_NAMES
 CurrentFuncName = "UseExtraMemory";
#endif
#if REQUIRE

#endif
/*-------------------------------------------------------------------------*/

	ExtraMem  = (void huge32 *)mem;
	ExtraSize = size;
	SetExtraMemFuncs (DistrubuteMem, FreeSomeExtra);

} /* UseExtraMemory */

#endif

/************************************  ************************************/
/********************************* AMIGAOS ********************************/
/************************************  ************************************/

#if __AMIGAOS__
void *mallocChipMem (size_t size)
{
	UBYTE 		*mem = NULL;

	if (size) {
		size = RoundUp (size);
		if (!initmem ()) {
			return NULL;
		}
		mem = FindSomeMem (size, TRUE);

#if PMESS
	printf ("mallocCM (%6ld); memfree = %7ld\n", (long)size, memfree());
#endif
	}

	return (void *)mem;
}

/********************************* AMIGAOS ********************************/
void *callocChipMem (size_t num, size_t size)
{
	UBYTE 		*mem;
	size_t		 rsize;

	rsize = num * size;
	mem   = mallocChipMem (rsize);
	if (mem) {
		memset (mem, 0, rsize);
	}

	return mem;
}

#endif

