/* 
 * The contents of this file are subject to the Mozilla Public 
 * License Version 1.1 (the "License"); you may not use this file 
 * except in compliance with the License. You may obtain a copy of 
 * the License at http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS 
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * rights and limitations under the License. 
 * 
 * The Original Code is tUME: the Universal Map Editor, released 
 * June 18, 2000. 
 * 
 * The Initial Developer of the Original Code is Echidna. 
 * Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
 * All Rights Reserved. 
 * 
 * Contributor(s): 
 */

/*
 * CHECKGLU.C
 *
 * PROGRAMMER : Gregg A. Tavares
 *    VERSION : 00.000
 *    CREATED : 11/03/93
 *   MODIFIED : 10/06/94
 *       TABS : 05 09
 *
 *	     \|///-_
 *	     \oo///_
 *	-----w/-w------
 *	 E C H I D N A
 *	---------------
 *
 * DESCRIPTION
 *		All kinds of routines that exit on error.
 *
 * HISTORY
 *		10/06/94 (dcc) - updated to use <echidna/alloc.h>
 *
*/

#include <echidna/platform.h>
#include "switches.h"
#include <echidna/proc.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <echidna/alloc.h>
#include <echidna/eio.h>
#include <echidna/listfunc.h>
#include <echidna/xtramem.h>
#include <echidna/memory.h>
#include <echidna/checkglu.h>

/**************************** C O N S T A N T S ***************************/


/******************************** T Y P E S *******************************/


/****************************** G L O B A L S *****************************/


/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Open                                                      */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Open
 *
 * SYNOPSIS
 *		int  CHK_Open (char *name, int access)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (int , CHK_Open, (char *name, int access))
{
	int	fh;

	fh = EIO_Open (name, access);
	if (fh == (-1))
	{
		printf ("Couldn't open %s\n", name);
		exit (EXIT_FAILURE);
	}

	return fh;
}
PROC_END // CHK_Open

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Read                                                      */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Read
 *
 * SYNOPSIS
 *		long  CHK_Read (int fh, void *buf, long size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (long , CHK_Read, (int fh, void *buf, long size))
{
	long	len;

	len = EIO_Read (fh, buf, size);
	if (len == (-1))
	{
		printf ("Error reading from file\n");
		exit (EXIT_FAILURE);
	}

	return len;

}
PROC_END // CHK_Read

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Write                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Write
 *
 * SYNOPSIS
 *		long  CHK_Write (int fh, void *buf, long size)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (long , CHK_Write, (int fh, void *buf, long size))
{

	long	len;

	len = EIO_Write (fh, buf, size);
	if (len == (-1))
	{
		printf ("Error writing to file\n");
		exit (EXIT_FAILURE);
	}

	return len;


}
PROC_END // CHK_Write

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Close                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Close
 *
 * SYNOPSIS
 *		void  CHK_Close (int fh)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_Close, (int fh))
{

	if (EIO_Close (fh) == (-1))
	{
		printf ("Error closing file\n");
		exit (EXIT_FAILURE);
	}

}
PROC_END // CHK_Close

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Seek                                                      */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Seek
 *
 * SYNOPSIS
 *		long  CHK_Seek (int fh, long off, int orig)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (long , CHK_Seek, (int fh, long off, int orig))
{
	long	pos;

	pos = EIO_Seek (fh, off, orig);
	if (pos == (-1))
	{
		printf ("Error seeking in file\n");
		exit (EXIT_FAILURE);
	}

	return pos;
}
PROC_END // CHK_Seek

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_Tell                                                      */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_Tell
 *
 * SYNOPSIS
 *		long  CHK_Tell (int fh)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (long , CHK_Tell, (int fh))
{
	long	pos;

	pos = EIO_Tell (fh);
	if (pos == (-1))
	{
		printf ("Error getting current position in file\n");
		exit (EXIT_FAILURE);
	}

	return pos;
}
PROC_END // CHK_Tell

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_FileLength                                                */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_FileLength
 *
 * SYNOPSIS
 *		long  CHK_FileLength (int fh)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (long , CHK_FileLength, (int fh))
{

	long	pos;

	pos = EIO_FileLength (fh);
	if (pos == (-1))
	{
		printf ("Error getting file length\n");
		exit (EXIT_FAILURE);
	}

	return pos;

}
PROC_END // CHK_FileLength

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_FindFile                                                  */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_FindFile
 *
 * SYNOPSIS
 *		void  CHK_FindFile (char *filename, char *guidename, char *pathbuff)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_FindFile, (char *filename, char *guidename, char *pathbuff))
{

	if (!EIO_FindFile (filename, guidename, pathbuff))
	{
		printf ("Couldn't find file %s in path\n", filename);
		exit (EXIT_FAILURE);
	}

}
PROC_END // CHK_FindFile

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fopen                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fopen
 *
 * SYNOPSIS
 *		FILE * CHK_fopen (char *filename, char *mode)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (FILE *, CHK_fopen, (char *filename, char *mode))
{

	FILE	*fp;

	fp = fopen (filename, mode);
	if (!fp)
	{
		printf ("Couldn't open file %s\n", filename);
		exit (EXIT_FAILURE);
	}

	return fp;

}
PROC_END // CHK_fopen

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fclose                                                    */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fclose
 *
 * SYNOPSIS
 *		void  CHK_fclose (FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_fclose, (FILE *fp))
{

	fclose (fp);

	#if 0
	if (!fclose (fp))
	{
		printf ("Error closing file\n");
		exit (EXIT_FAILURE);
	}
	#endif

}
PROC_END // CHK_fclose

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fread                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fread
 *
 * SYNOPSIS
 *		size_t  CHK_fread (void *buf, size_t size, size_t n, FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (size_t , CHK_fread, (void *buf, size_t size, size_t n, FILE *fp))
{
	size_t	res;

	res = fread (buf, size, n, fp);
	if (!res)
	{
		printf ("Error reading from file\n");
		exit (EXIT_FAILURE);
	}

	return res;
}
PROC_END // CHK_fread

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fwrite                                                    */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fwrite
 *
 * SYNOPSIS
 *		size_t  CHK_fwrite (void *buf, size_t size, size_t n, FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (size_t , CHK_fwrite, (void *buf, size_t size, size_t n, FILE *fp))
{
	size_t	res;

	res = fwrite (buf, size, n, fp);
	if (!res)
	{
		printf ("Error writing to file\n");
		exit (EXIT_FAILURE);
	}

	return res;
}
PROC_END // CHK_fwrite

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fprintf                                                   */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fprintf
 *
 * SYNOPSIS
 *		int  CHK_fprintf (FILE *fp, char *fmt, ...)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (int , CHK_fprintf, (FILE *fp, char *fmt, ...))
{
	int	res;
	va_list ap;	/* points to each unnamed arg in turn */

	va_start (ap,fmt); /* make ap point to 1st unnamed arg */
	res = vfprintf (fp, fmt, ap);
	va_end (ap);	/* clean up when done */

	if (res < 0)
	{
		printf ("Error fprintf to file\n");
		exit (EXIT_FAILURE);
	}

	return res;
}
PROC_END // CHK_fprintf

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fputc                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fputc
 *
 * SYNOPSIS
 *		void  CHK_fputc (int c, FILE *fp)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_fputc, (int c, FILE *fp))
{
	if (fputc (c, fp) == EOF)
	{
		printf ("Error in CHK_fputc writing to file\n");
		exit (EXIT_FAILURE);
	}
}
PROC_END // CHK_fputc

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_fputs                                                     */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_fputs
 *
 * SYNOPSIS
 *		void  CHK_fputs (FILE *fp, char *s)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_fputs, (FILE *fp, char *s))
{
	if (fputs (s, fp))
	{
		printf ("Error in CHK_fputs writing to file\n");
		exit (EXIT_FAILURE);
	}
}
PROC_END // CHK_fputs


/*------------------------------------------------------------------------*/
/**# MODULE:CHK_CreateList                                                */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_CreateList
 *
 * SYNOPSIS
 *		ListType * CHK_CreateList (char *name, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (ListType *, CHK_CreateList, (char *name, char *id))
{

	ListType	*list;

	list = CreateList (name);
	if (!list)
	{
		printf ("Out of Memory: Could allocate list %s\n", id);
		exit (EXIT_FAILURE);
	}

	return list;
}
PROC_END // CHK_CreateList

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_CreateNode                                                */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_CreateNode
 *
 * SYNOPSIS
 *		void * CHK_CreateNode (size_t size, char *name, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void *, CHK_CreateNode, (size_t size, char *name, char *id))
{
	NodeType	*nd;

	nd = CreateNode (size, name);
	if (!nd)
	{
		printf ("Out of memory: couldn't allocate node %s\n", id);
		exit (EXIT_FAILURE);
	}

	return (void *)nd;

}
PROC_END // CHK_CreateNode


/*------------------------------------------------------------------------*/
/**# MODULE:CHK_AllocateMemory                                            */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_AllocateMemory
 *
 * SYNOPSIS
 *		void * CHK_AllocateMemory (long size, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void *, CHK_AllocateMemory, (long size, char *id))
{
	void	*buf;

	buf = MEM_farmalloc (size);
	if (!buf)
	{
		printf ("Out of memory: Couldn't allocate %s\n", id);
		exit (EXIT_FAILURE);
	}

	return buf;
}
PROC_END // CHK_AllocateMemory

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_CallocateMemory                                           */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_CallocateMemory
 *
 * SYNOPSIS
 *		void * CHK_CallocateMemory (long size, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void *, CHK_CallocateMemory, (long size, char *id))
{
	void	*buf;

	buf = CHK_AllocateMemory (size, id);
	memset (buf, 0, (size_t) size);

	return buf;
}
PROC_END // CHK_CallocateMemory

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_DeallocateMemory                                          */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_DeallocateMemory
 *
 * SYNOPSIS
 *		void  CHK_DeallocateMemory (void *ptr, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_DeallocateMemory, (void *ptr, char *id))
{
	id = id;
	MEM_farfree (ptr);

}
PROC_END // CHK_DeallocateMemory

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_AllocateXTRA                                              */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_AllocateXTRA
 *
 * SYNOPSIS
 *		XTRAPntr  CHK_AllocateXTRA (size_t size, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (XTRAPntr , CHK_AllocateXTRA, (size_t size, char *id))
{
	XTRAPntr	xtra;

	xtra = AllocXTRA (size);
	if (!xtra)
	{
		printf ("Out of memory: couldn't allocate %s\n", id);
		exit (EXIT_FAILURE);
	}

	return xtra;
}
PROC_END // CHK_AllocateXTRA

/*------------------------------------------------------------------------*/
/**# MODULE:CHK_DeallocateXTRA                                            */
/*------------------------------------------------------------------------*/

/*********************************************************************
 *
 * CHK_DeallocateXTRA
 *
 * SYNOPSIS
 *		void  CHK_DeallocateXTRA (XTRAPntr xtra, char *id)
 *
 * PURPOSE
 *		
 *
 * INPUT
 *
 *
 * EFFECTS
 *
 *
 * RETURN VALUE
 *
 *
 * HISTORY
 *
 *
 * SEE ALSO
 *
*/
PROC (void , CHK_DeallocateXTRA, (XTRAPntr xtra, char *id))
{

	id = id;
	FreeXTRA (xtra);

}
PROC_END // CHK_DeallocateXTRA

