;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** TIMERP.ASM
;**
;** PROGRAMMER : Dan Chang
;**    VERSION : 00.000
;**    CREATED : 09/13/93
;**   MODIFIED : 03/20/94
;**       TABS : 09 17
;**
;** DESCRIPTION
;**	Protected-mode 8253 Programmer interval timer routines.
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created (based on real-mode VBLANK.ASM).
;**	10/12/93 Tuesday (dcc) - added switch to our own stack.
;**	03/18/94 Friday (dcc) - revise Reset_Vint() to be more safe.
;**
	.386

	IDEAL

	; Only "PUBLIC" so that INTERUPT.C can access these routines:

	PUBLIC	_OpenTimerInterrupt_
	PUBLIC	_CloseTimerInterrupt_

	; Only "PUBLIC" so that INTERUPT.C can access these variables:

	public	__TimerRecord
	public	__uwFreeHead

	PUBLIC	__TimerCount
	PUBLIC	__NewTimerCount

	public	__VBlankAddress
	public	__VBlankActive
	public	__LastVBlankActive

	; "User modifiable" constants

MAXVBLANKS	equ	8		;number of vblanks you may install
MAXTIMERRECORDS	equ	256

STACKSIZE	equ	512

	; "Program" constants

ACTIVE	equ	(10000000b)
MODIFY	equ	(01000000b)
COUNTUP	equ	(00001000b)
ROUTINE	equ	(00000100b)

;32bits	equ	(00000010b)
;16bits equ	(00000001b)
;08bits	equ	(00000000b)

vb60Hz	equ	19886	;(1193180/60)

	STRUC	TimerType
uwNext	dw	?
uwPrev	dw	?
ulAddress	dd	?
ulCounter	dd	?
ulTrigger	dw	?
		db	?
ubFlags		db	?
	ENDS

	SEGMENT _DATA DWORD PUBLIC USE32 'DATA'

	; Public variables

__TimerRecord	TimerType	MAXTIMERRECORDS dup (<>)
__uwFreeHead	dw	?

__TimerCount	dw	?		; current timer count value loaded
__NewTimerCount	dw	?		; new timer count value to load

__VBlankAddress	dd	MAXVBLANKS dup (?)	; address of vblank routine to call

__VBlankActive	db	MAXVBLANKS dup (?)	; set TRUE if VBlankDisp is valid

__LastVBlankActive	dw	?	; last vblank active in list

	; Private variables

old_timer_off	dd	0		; old int 08H interrupt vector
old_timer_sel	dw	0		; must immediately follow old_timer_off

VblankCount	dw	?		; keep track of counts between calls
					; to vertical blanking routine
VblankTrigger	dw	?		; if (VblankCount >= VblankTrigger) do vblank


OldTimerCount	dw	?		; keep track of counts between calls
OldTimerCountH	dw	?		; to old timer interrupt routine

	ALIGN	4
InterruptStack	dd	(STACKSIZE/4) dup (?)
	ENDS	_DATA

	GROUP	DGROUP _DATA

	SEGMENT _TEXT DWORD PUBLIC USE32 'CODE'

	ASSUME	cs:_TEXT
	ASSUME	ds:DGROUP

;=====================================================================
;**
;** LOCAL pmhandler
;**
;** PURPOSE
;**	This is the actual int 08H interrupt routine. It gets called
;**	60 times per second to begin with, but may increase in
;**	frequency as needed. This routine calls all other routines
;**	that depend on the timer interrupt.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	NOTHING.
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created (based on real-mode Vint).
;**
	PROC	pmhandler

	; Save registers:

 	push	eax
	push	ebx
	push	ecx
	push	edx
	push	ebp
	push	esi
	push	edi

	push	ds
	push	es

	mov	ax, DGROUP
	mov	ds, ax
	mov	es, ax

	; Switch to our stack:

	mov	bx, ss
	mov	ecx, esp
	mov	ss, ax
	lea	esp, [InterruptStack+STACKSIZE]
	push	ebx			;old SS
	push	ecx			;old ESP

	mov	ax, [__TimerCount]
	cmp	ax, [__NewTimerCount]
	jne	@@FastEnough		;=> still running fast enough

	; Need to speed up interrupt rate?

	mov	bx, [__NewTimerCount]
        mov   al, 36h                   ; set new interrupt rate
        out   43h, al                   ;
        jmp   $+2                       ;
        mov   al, bl                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
        mov   al, bh                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
@@FastEnough:
	mov	ax, [__TimerCount]
	add	[VblankCount], ax
	mov	ax, [VblankCount]
	cmp	ax, [VblankTrigger]
	jb	@@notVblank

	; Handle vertical blank routines

	sub	ax, vb60Hz
	mov	[VblankCount], ax

;!!	; Vertical blank should occur within this interrupt period.
;!!	; Wait for it to start...

;!!	mov	ah, 1000b
;!!	mov	dx, 3dah

;!!	in	al, dx			;check vertical retrace
;!!	test	al, ah
;!!	jnz	@@retrace		;=> already in vertical retrace
;!!@@waitretrace:
;!!	in	al, dx
;!!	test	al, ah
;!!	jz	@@waitretrace		;=> wait for retrace to happen
;!!
;!!	mov	bx, [__TimerCount]	; DCC load current timer value
;!!
;!!        mov   al, 36h                   ; set new interrupt rate
;!!        out   43h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bl                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bh                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!
;!!@@retrace:
	sub	esi, esi
	sub	edi, edi
@@nextVBL:
	cmp	[__VBlankActive+esi], 0
	jz	@@noVBL

	call	[__VBlankAddress+edi]
@@noVBL:
	add	di, 4
	inc	si
	cmp	si, [__LastVBlankActive]
	jb	@@nextVBL

;!!	sub	ax, ax			; reset VblankCount
@@notVblank:
;!!	mov	[VblankCount], ax

	sub	esi, esi
	jmp	SHORT @@updateLoop

	; Remove timer/interrupt from active list.

@@remove:
	; First, unlink entry from active list.

	mov	ax, [esi+__TimerRecord.uwNext]
	mov	di, [esi+__TimerRecord.uwPrev]
	mov	[edi+__TimerRecord.uwNext], ax

	mov	ax, [esi+__TimerRecord.uwPrev]
	mov	di, [esi+__TimerRecord.uwNext]
	mov	[edi+__TimerRecord.uwPrev], ax

	; Then add node to head of free list.

	mov	di, [__uwFreeHead]
	mov	[edi+__TimerRecord.uwPrev], si
	mov	[esi+__TimerRecord.uwNext], di
	mov	[esi+__TimerRecord.uwPrev], 0ffffh
	mov	[__uwFreeHead], si

	; Set offset index to previous active record in list
	; (so when @@updateLoop gets next record it'll be correct).

	mov	si, ax

	; Top of loop: go through all active timer records.

@@updateLoop:
	mov	si, [esi+__TimerRecord.uwNext]	; get next record offset

	; See if last record reached (_TimerRecourd.uwNext == 0ffffh)?

	mov	ax, [esi+__TimerRecord.uwNext]
	inc	ax
	jz	@@done			;=> last record reached

	; See if record is being modified? If so, skip it.

	mov	al, [esi+__TimerRecord.ubFlags]
	test	al, MODIFY
	jnz	@@updateLoop		;=> this record is being updated

	; See if record is active? If not, delete it.

	test	al, ACTIVE
	jz	@@remove		;=> remove record

	; See if ulTrigger is zero (timer suspended)?

	sub	ah, ah
	mov	al, [byte ptr esi+2+__TimerRecord.ulTrigger]
	or	ax, [word ptr esi+__TimerRecord.ulTrigger]
	jz	@@updateLoop		;=> this timer is suspended

	; Add time elapsed since last interrupt to total time elapsed for this record.

	mov	ax, [__TimerCount]
	add	[word ptr esi+__TimerRecord.ulCounter], ax
	adc	[word ptr esi+2+__TimerRecord.ulCounter], 0

	; Compare total time elapsed with the trigger value (3 byte quantity).

	sub	bh, bh			;zero out 4th byte of trigger
	mov	bl, [byte ptr esi+2+__TimerRecord.ulTrigger]	;get 3rd byte

	mov	ax, [word ptr esi+__TimerRecord.ulCounter]
	sub	ax, [word ptr esi+__TimerRecord.ulTrigger]
	mov	dx, [word ptr esi+2+__TimerRecord.ulCounter]
	sbb	dx, bx
	jb	@@updateLoop		;=> not time to inc timer/call routine yet

	; Subtract out Trigger value and save new total time elapsed.

	mov	[word ptr esi+__TimerRecord.ulCounter], ax
	mov	[word ptr esi+2+__TimerRecord.ulCounter], dx

	; Increment/decrement timer or call routine?

	mov	al, [esi+__TimerRecord.ubFlags]
	test	al, ROUTINE
	jz	@@IncOrDec		;=> ROUTINE bit not set, inc/dec timer

	; Call user install routine.

	call	[esi+__TimerRecord.ulAddress]
	jmp	@@updateLoop

	; Increment or decrement memory location

@@IncOrDec:
	mov	edi, [dword ptr esi+__TimerRecord.ulAddress]

	test	al, COUNTUP
	jz	@@countDN		;=> decrement memory location

	; Increment memory location.

	and	al, 11b
	jnz	@@notBup

	inc	[byte ptr edi]
	jmp	@@updateLoop
@@notBup:
	dec	al
	jnz	@@notWup

	inc	[word ptr edi]
	jmp	@@updateLoop
@@notWup:
	inc	[dword ptr edi]
@@nextEntry:
	jmp	@@updateLoop

	; Decrement memory location.
@@countDN:
	and	al, 11b
	jnz	@@notBdn

	mov	al, [byte ptr edi]
	or	al, al
	jz	@@updateLoop		;=> byte counter already at zero
	dec	al
	mov	[byte ptr edi], al
	jmp	@@updateLoop
@@notBdn:
	dec	al
	jnz	@@notWdn

	mov	ax, [word ptr edi]
	or	ax, ax
	jz	@@updateLoop		;=> word counter already at zero
	dec	ax
	mov	[word ptr edi], ax
	jmp	@@updateLoop
@@notWdn:
	mov	eax, [dword ptr edi]
	or	eax, eax
	jz	@@updateLoop		;=> long word counter already at zero
	dec	eax
	mov	[dword ptr edi], eax
	jmp	@@updateLoop
@@done:
	mov	ax, [__TimerCount]

	mov	bx, [__NewTimerCount]
	mov	[__TimerCount], bx	; __TimerCount = __NewTimerCount

	add	[OldTimerCount], ax
	adc	[OldTimerCountH], 0
	jnz	@@CallOld		; => time to call old timer interrupt

	mov	al, 20h			;eoi
	out	20h, al

	; Switch back to interrupt stack:

	pop	ebx			;old ESP
	pop	eax			;old SS
	mov	ss, ax
	mov	esp, ebx

	; Restore registers:

	pop	es
	pop	ds

	pop	edi
	pop	esi
	pop	ebp
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	iretd

	; Call old timer interrupt routine.

@@CallOld:
	dec	[OldTimerCountH]	; update OldTimerCount

	; Switch back to interrupt stack:

	pop	ebx			;old ESP
	pop	eax			;old SS
	mov	ss, ax
	mov	esp, ebx

	; This may be a total KLUDGE! By empirical hacking, we've
	; determined that we need to do a PUSHFD before the CALL.
	; We need to look in a 386 manual to verify what's going on.

	pushfd
	call	[PWORD PTR old_timer_off]

	; Restore registers:

	pop	es
	pop	ds

	pop	edi
	pop	esi
	pop	ebp
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	iretd

	ENDP	pmhandler


;=====================================================================
;**
;** LOCAL SaveVector
;**
;** PURPOSE
;**	Save the old protected-mode int 08H handler address.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created.
;**
	PROC	SaveVector

	push	eax
	push	ebx
	push	es
	push	ds

	sub	ax,ax
	mov	ds,ax
	mov	es,ax
	mov	eax, 3508h
	int	21h

	pop	ds

	mov	[old_timer_sel], es
	mov	[old_timer_off], ebx

	pop	es
	pop	ebx
	pop	eax
	ret

	ENDP	SaveVector


;=====================================================================
;**
;** LOCAL InitVblank
;**
;** PURPOSE
;**	Initialize vertical blanking interrupt rate. Set timer
;**	rate to vb60Hz. The following variables are set:
;**
;**	
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	The old time rate was 65536 (about 18.2 times per second).
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created (based on real-mode Read_VBlank).
;**
	PROC	InitVblank

	push	ax
	push	bx

	sub	ax, ax
	mov	[VblankCount], ax	; reset vblank timer count
	mov	[OldTimerCount], ax	; reset old timer count
	mov	[OldTimerCountH], ax

	mov	bx, vb60Hz
	mov	[__TimerCount], bx	; set current timer value
	mov	[__NewTimerCount], bx	; set new timer value
	mov	[VblankTrigger], bx	; before we need to call vbl routine

	CLI

        mov   al, 36h                   ; set new interrupt rate
        out   43h, al                   ;
        jmp   $+2                       ;
        mov   al, bl                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
        mov   al, bh                    ;
        out   40h, al                   ;
        jmp   $+2                       ;

	STI

	pop	bx
	pop	ax

	ret

	ENDP	InitVblank


;=====================================================================
;**
;** LOCAL InstallHandler
;**
;** PURPOSE
;**	Install our int 08H handler.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created.
;**
	PROC	InstallHandler

	push	eax
	push	edx
	push	ds
	push	es

	sub	ax, ax
	mov	es, ax
	mov	ax, seg pmhandler
	mov	ds, ax
	mov	edx, offset pmhandler
	mov	eax, 2508h
	int	21h

	pop	es
	pop	ds
	pop	edx
	pop	eax
	ret

	ENDP	InstallHandler


;=====================================================================
;**
;** LOCAL RemoveHandler
;**
;** PURPOSE
;**	Restore the old timer interrupt 08H handler.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created.
;**	03/18/94 Friday (dcc) - go paranoid - reorder, disable int
;**
	PROC	RemoveHandler

	push	eax
	push	edx
	push	es
	pushf

	cli

        mov   al,36H               ; Timer mode 3, load rate Low/High
        out   43H,al               ;
        jmp   $+2                  ;
        sub   eax, eax             ;
        out   40H,al               ;
        jmp   $+2                  ;
        out   40H,al               ; 65536 for 18.2 times per second.

	push	ds

	mov	es, ax			;ES = 0
	mov	edx, [old_timer_off]
	mov	ax, [old_timer_sel]
	mov	ds, ax
	mov	eax, 2508h
	int	21h			; set int 08H to old_timer vector

	pop	ds

	sub	eax, eax
	mov	[old_timer_sel], ax	; set flag: routine restored
	mov	[old_timer_off], eax

	popf
	pop	es
	pop	edx
	pop	eax
	ret

	ENDP	RemoveHandler


;=====================================================================
;**
;** _OpenTimerInterrupt_
;**
;** PURPOSE
;**	Install an interrupt routine that runs at 60Hz. This routine
;**	will call your interrupt routines and update your TIMERs as
;**	appropriate.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	NOTHING.
;**
;** REGISTER USED
;**	
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created (based on real-mode OpenTimerInterrupt).
;**
	PROC	_OpenTimerInterrupt_

	push	eax

	sub	eax, eax
	mov	ax, [old_timer_sel]
	or	eax, [old_timer_off]	;see if already installed
	jnz	@@installed		;=> already installed

	call	SaveVector		;save old timer interrupt vector
	call	InitVBlank		;initialize vertical blank handler
	call	InstallHandler		;start our timer interrupt handler
@@installed:
	pop	eax
	ret

	ENDP	_OpenTimerInterrupt_


;=====================================================================
;**
;** _CloseTimerInterrupt_
;**
;** PURPOSE
;**	Remove the our int 08H handler, and restore the timer to
;**	normal speed.
;**
;** INPUT
;**	
;**
;** ASSUMES
;**	The old time rate was 65536 (about 18.2 times per second).
;**
;** REGISTER USED
;**	
;**
;** RETURN VALUE
;**	
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 Monday (dcc) - created (based on real-mode CloseTimerInterrupt).
;**
	PROC	_CloseTimerInterrupt_

	push	eax

	; See if there is anything to restore.

	sub	eax, eax
	mov	ax, [old_timer_sel]	;see if already restored or
	or	eax, [old_timer_off]	;    if never installed
	jz	@@notinstalled		;=> don't restore

	call	RemoveHandler
@@notinstalled:
	pop	eax
	ret

	ENDP	_CloseTimerInterrupt_

	ENDS	_TEXT
    END

