;* 
;* The contents of this file are subject to the Mozilla Public 
;* License Version 1.1 (the "License"); you may not use this file 
;* except in compliance with the License. You may obtain a copy of 
;* the License at http://www.mozilla.org/MPL/ 
;* 
;* Software distributed under the License is distributed on an "AS 
;* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
;* implied. See the License for the specific language governing 
;* rights and limitations under the License. 
;* 
;* The Original Code is tUME: the Universal Map Editor, released 
;* June 18, 2000. 
;* 
;* The Initial Developer of the Original Code is Echidna. 
;* Portions created by Echidna are Copyright (c) 1989-2000 Echidna. 
;* All Rights Reserved. 
;* 
;* Contributor(s): 
;* 

;**
;** TIMERPB.ASM
;**
;** PROGRAMMER : Dan Chang
;**    VERSION : 00.000
;**    CREATED : 09/13/93
;**   MODIFIED : 10/27/94
;**       TABS : 09 17
;**
;** DESCRIPTION
;**	Protected-mode 8253 Programmer interval timer routines.
;**
;** HISTORY
;**	09/13/93 (dcc) - created (based on real-mode VBLANK.ASM).
;**	10/12/93 (dcc) - added switch to our own stack.
;**	03/18/94 (dcc) - revise Reset_Vint() to be more safe.
;**	10/27/94 (dcc) - Updated for Borland C++ 4.02 32-bit protected-mode.
;**
	.386
	.model flat,c

	; "User modifiable" constants

MAXVBLANKS	equ	8		;number of vblanks you may install
MAXTIMERRECORDS	equ	256

	; "Program" constants

ACTIVE	equ	(10000000b)
MODIFY	equ	(01000000b)
COUNTUP	equ	(00001000b)
ROUTINE	equ	(00000100b)

vb60Hz	equ	19886	;(1193180/60)

	.data

	extrn  __TIM_dataseg:word
	extrn  __TIM_oldhandler:fword

	TimerType	STRUC
uwNext	dw	?
uwPrev	dw	?
ulAddress	dd	?
ulCounter	dd	?
ulTrigger	dw	?
		db	?
ubFlags		db	?
	ENDS

	; Public variables

	extrn	__TimerRecord:TimerType
	extrn	__uwFreeHead:word

	extrn	__TimerCount:word	; current timer count value loaded
	extrn	__NewTimerCount:word	; new timer count value to load

	extrn	__VBlankAddress:dword	; address of vblank routine to call

	extrn	__VBlankActive:byte	; set TRUE if VBlankDisp is valid

	extrn	__LastVBlankActive:word	; last vblank active in list

	; Private variables

VblankCount	dw	?		; keep track of counts between calls
					; to vertical blanking routine
VblankTrigger	dw	?		; if (VblankCount >= VblankTrigger) do vblank

OldTimerCount	dw	?		; keep track of counts between calls
OldTimerCountH	dw	?		; to old timer interrupt routine

	.code


;=====================================================================
;**
;** __TIM_pmhandler
;**
;** PURPOSE
;**	This is the actual int 08H interrupt routine. It gets called
;**	60 times per second to begin with, but may increase in
;**	frequency as needed. This routine calls all other routines
;**	that depend on the timer interrupt.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	NOTHING.
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	10/27/94 (dcc) - Created (based on Watcom protected-mode pmhandler).
;**
	PROC	__TIM_pmhandler
	public	__TIM_pmhandler

	; Save registers:

 	push	eax
	push	ebx
	push	ecx
	push	edx
	push	ebp
	push	esi
	push	edi

	push	ds
	push	es

	mov	ds,cs:[__TIM_dataseg]
	mov	ax, [__TimerCount]
	cmp	ax, [__NewTimerCount]
	jne	@@FastEnough		;=> still running fast enough

	; Need to speed up interrupt rate?

	mov	bx, [__NewTimerCount]
        mov   al, 36h                   ; set new interrupt rate
        out   43h, al                   ;
        jmp   $+2                       ;
        mov   al, bl                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
        mov   al, bh                    ;
        out   40h, al                   ;
        jmp   $+2                       ;
@@FastEnough:
	mov	ax, [__TimerCount]
	add	[VblankCount], ax
	mov	ax, [VblankCount]
	cmp	ax, [VblankTrigger]
	jb	@@notVblank

	; Handle vertical blank routines

	sub	ax, vb60Hz
	mov	[VblankCount], ax

;!!	; Vertical blank should occur within this interrupt period.
;!!	; Wait for it to start...

;!!	mov	ah, 1000b
;!!	mov	dx, 3dah

;!!	in	al, dx			;check vertical retrace
;!!	test	al, ah
;!!	jnz	@@retrace		;=> already in vertical retrace
;!!@@waitretrace:
;!!	in	al, dx
;!!	test	al, ah
;!!	jz	@@waitretrace		;=> wait for retrace to happen
;!!
;!!	mov	bx, [__TimerCount]	; DCC load current timer value
;!!
;!!        mov   al, 36h                   ; set new interrupt rate
;!!        out   43h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bl                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!        mov   al, bh                    ;
;!!        out   40h, al                   ;
;!!        jmp   $+2                       ;
;!!
;!!@@retrace:
	sub	esi, esi
	sub	edi, edi
@@nextVBL:
	cmp	[__VBlankActive+esi], 0
	jz	@@noVBL

	call	[__VBlankAddress+edi]
@@noVBL:
	add	di, 4
	inc	si
	cmp	si, [__LastVBlankActive]
	jb	@@nextVBL

;!!	sub	ax, ax			; reset VblankCount
@@notVblank:
;!!	mov	[VblankCount], ax

	sub	esi, esi
	jmp	SHORT @@updateLoop

	; Remove timer/interrupt from active list.

@@remove:
	; First, unlink entry from active list.

	mov	ax, [esi+__TimerRecord.uwNext]
	mov	di, [esi+__TimerRecord.uwPrev]
	mov	[edi+__TimerRecord.uwNext], ax

	mov	ax, [esi+__TimerRecord.uwPrev]
	mov	di, [esi+__TimerRecord.uwNext]
	mov	[edi+__TimerRecord.uwPrev], ax

	; Then add node to head of free list.

	mov	di, [__uwFreeHead]
	mov	[edi+__TimerRecord.uwPrev], si
	mov	[esi+__TimerRecord.uwNext], di
	mov	[esi+__TimerRecord.uwPrev], 0ffffh
	mov	[__uwFreeHead], si

	; Set offset index to previous active record in list
	; (so when @@updateLoop gets next record it'll be correct).

	mov	si, ax

	; Top of loop: go through all active timer records.

@@updateLoop:
	mov	si, [esi+__TimerRecord.uwNext]	; get next record offset

	; See if last record reached (_TimerRecourd.uwNext == 0ffffh)?

	mov	ax, [esi+__TimerRecord.uwNext]
	inc	ax
	jz	@@done			;=> last record reached

	; See if record is being modified? If so, skip it.

	mov	al, [esi+__TimerRecord.ubFlags]
	test	al, MODIFY
	jnz	@@updateLoop		;=> this record is being updated

	; See if record is active? If not, delete it.

	test	al, ACTIVE
	jz	@@remove		;=> remove record

	; See if ulTrigger is zero (timer suspended)?

	sub	ah, ah
	mov	al, byte ptr [esi+2+__TimerRecord.ulTrigger]
	or	ax, word ptr [esi+__TimerRecord.ulTrigger]
	jz	@@updateLoop		;=> this timer is suspended

	; Add time elapsed since last interrupt to total time elapsed for this record.

	mov	ax, [__TimerCount]
	add	word ptr [esi+__TimerRecord.ulCounter], ax
	adc	word ptr [esi+2+__TimerRecord.ulCounter], 0

	; Compare total time elapsed with the trigger value (3 byte quantity).

	sub	bh, bh			;zero out 4th byte of trigger
	mov	bl, byte ptr [esi+2+__TimerRecord.ulTrigger]	;get 3rd byte

	mov	ax, word ptr [esi+__TimerRecord.ulCounter]
	sub	ax, word ptr [esi+__TimerRecord.ulTrigger]
	mov	dx, word ptr [esi+2+__TimerRecord.ulCounter]
	sbb	dx, bx
	jb	@@updateLoop		;=> not time to inc timer/call routine yet

	; Subtract out Trigger value and save new total time elapsed.

	mov	word ptr [esi+__TimerRecord.ulCounter], ax
	mov	word ptr [esi+2+__TimerRecord.ulCounter], dx

	; Increment/decrement timer or call routine?

	mov	al, [esi+__TimerRecord.ubFlags]
	test	al, ROUTINE
	jz	@@IncOrDec		;=> ROUTINE bit not set, inc/dec timer

	; Call user install routine.

	call	[esi+__TimerRecord.ulAddress]
	jmp	@@updateLoop

	; Increment or decrement memory location

@@IncOrDec:
	mov	edi, dword ptr [esi+__TimerRecord.ulAddress]

	test	al, COUNTUP
	jz	@@countDN		;=> decrement memory location

	; Increment memory location.

	and	al, 11b
	jnz	@@notBup

	inc	byte ptr [edi]
	jmp	@@updateLoop
@@notBup:
	dec	al
	jnz	@@notWup

	inc	word ptr [edi]
	jmp	@@updateLoop
@@notWup:
	inc	dword ptr [edi]
@@nextEntry:
	jmp	@@updateLoop

	; Decrement memory location.
@@countDN:
	and	al, 11b
	jnz	@@notBdn

	mov	al, byte ptr [edi]
	or	al, al
	jz	@@updateLoop		;=> byte counter already at zero
	dec	al
	mov	byte ptr [edi], al
	jmp	@@updateLoop
@@notBdn:
	dec	al
	jnz	@@notWdn

	mov	ax, word ptr [edi]
	or	ax, ax
	jz	@@updateLoop		;=> word counter already at zero
	dec	ax
	mov	word ptr [edi], ax
	jmp	@@updateLoop
@@notWdn:
	mov	eax, dword ptr [edi]
	or	eax, eax
	jz	@@updateLoop		;=> long word counter already at zero
	dec	eax
	mov	dword ptr [edi], eax
	jmp	@@updateLoop
@@done:
	mov	ax, [__TimerCount]

	mov	bx, [__NewTimerCount]
	mov	[__TimerCount], bx	; __TimerCount = __NewTimerCount

	mov	ax, vb60Hz	;!!!!!!
	add	[OldTimerCount], ax
	adc	[OldTimerCountH], 0
	jnz	@@CallOld		; => time to call old timer interrupt

	mov	al, 20h			;eoi
	out	20h, al

	; Restore registers:

	pop	es
	pop	ds

	pop	edi
	pop	esi
	pop	ebp
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax
	iretd

	; Call old timer interrupt routine.

@@CallOld:
	dec	[OldTimerCountH]	; update OldTimerCount

	; Restore registers:

	pop	es
	pop	ds

	pop	edi
	pop	esi
	pop	ebp
	pop	edx
	pop	ecx
	pop	ebx
	pop	eax

	jmp	cs:fword ptr [__TIM_oldhandler]

	ENDP	__TIM_pmhandler


;=====================================================================
;**
;** SetTimerRate
;**
;** PURPOSE
;**	Set the timer rate.
;**
;** INPUT
;**	BX		: timer rate to set (0 == 18.2/sec, vb60Hz == 60/sec)
;**
;** ASSUMES
;**	Interrupts have been disabled!
;**
;** REGISTER CHANGED
;**	AX
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	10/27/94 (dcc) - Created.
;**
	PROC	SetTimerRate

        mov	al, 36h			; Timer mode 3, load rate Low/High
        out	43h, al
        jmp	$+2
        mov	al, bl
        out	40h, al
        jmp	$+2
        mov	al, bh
        out	40h, al
        jmp	$+2

	ret

	ENDP	SetTimerRate


;=====================================================================
;**
;** __TIM_InitVBlank
;**
;** PURPOSE
;**	Initialize vertical blanking interrupt rate. Set timer
;**	rate to vb60Hz. The following variables are set:
;**
;**	
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	The old time rate was 65536 (about 18.2 times per second).
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	10/27/94 (dcc) - Created (based on Watcom protected-mode InitVblank).
;**
	PROC	__TIM_InitVBlank
	public	__TIM_InitVBlank

	push	eax
	push	ebx

	push	ds
	mov	ds,cs:[__TIM_dataseg]

	sub	eax, eax
	mov	[VblankCount], ax	; reset vblank timer count
	mov	[OldTimerCount], ax	; reset old timer count
	mov	[OldTimerCountH], ax

	mov	bx, vb60Hz
	mov	[__TimerCount], bx	; set current timer value
	mov	[__NewTimerCount], bx	; set new timer value
	mov	[VblankTrigger], bx	; before we need to call vbl routine
 	call	SetTimerRate

	pop	ds

	pop	ebx
	pop	eax
	ret

	ENDP	__TIM_InitVBlank


;=====================================================================
;**
;** __TIM_RestoreTimer
;**
;** PURPOSE
;**	Restore the old timer interrupt rate.
;**
;** INPUT
;**	NONE.
;**
;** ASSUMES
;**	
;**
;** REGISTER USED
;**	NONE.
;**
;** RETURN VALUE
;**	NONE.
;**
;** LIMITS/BUGS
;**	
;**
;** HISTORY
;**	09/13/93 (dcc) - created.
;**	03/18/94 (dcc) - go paranoid - reorder, disable int
;**	10/27/94 (dcc) - Created (based on Watcom protected-mode RemoveHandler).
;**
	PROC	__TIM_RestoreTimer
	public	__TIM_RestoreTimer

	push	eax
	push	ebx
	push	es

	sub	ebx,ebx
	call	SetTimerRate

	pop	es
	pop	ebx
	pop	eax
	ret

	ENDP	__TIM_RestoreTimer

	END

